c ---------------------------------------------------------------------
      subroutine methdr_aermet(io,ilog,fmet,ws,wd,tamb,ipg,MNONSEQ)
c ---------------------------------------------------------------------
c
c --- METSERIES   Version: 1.00       Level: 121112
c --- B.Zelt
c
c --- PURPOSE: Extract header information from aermod.sfc
c
c --- UPDATES:
c --- v1.0(121112)
c        - 
c
c ---------------------------------------------------------------------
c --- MODULES
      use mod_stddefs
      use mod_met
c --- INPUTS
      integer :: io     ! read io
      integer :: ilog   ! debug and list logging
      character*mxpath :: fmet ! path for met
      real(8) :: ws     ! wind speed, m/s
      real(8) :: wd     ! wind direction
      real(8) :: tamb   ! Temperature K
      integer :: ipg
      integer :: MNONSEQ 
c --- DECLARATIONS
      character*255 :: buffer
      real(8) :: tamb_tmp
      interface
      subroutine rdmet_aermet(io,ilog,ws,wd,tamb,ipg,iend)
         integer,intent(in):: io
         integer,intent(in):: ilog
         real(8),intent(out):: ws
         real(8),intent(out):: wd
         real(8):: tamb
         integer,intent(out):: iend,ipg
      end subroutine
      end interface
      
      zws = -1.      
      zwd = -1.      
      
c --- Check the MET header that it is a TIMESERIES.TSF output file
c --- with correct location and extraction elevation
c --------------------------------------------------
      open(io,file=fmet,ERR=90,status='old',action='read')

c --- read the header line, but junk it
      read(io,1901,ERR=90,IOSTAT=IOERRN) buffer
1901  format(A132)

c       SUBROUTINE MEOPEN
c      READ(MFUNIT,'(A132)',ERR=998,END=1000,IOSTAT=IOERRN) BUFFER
c      READ(BUFFER,1900,ERR=998,IOSTAT=IOERRN) ALAT, ALON, SPEC1,
c     &        CUSI, SPEC2, CSSI, SPEC3, COSI, C_METVER
c 1900    FORMAT(2A10,T31,A6,T38,A8,T48,A6,T55,A8,T65,A6,T72,A8,T93,A6)
      
c read first line to initialize 
      tamb_tmp = 0.0
      call rdmet_aermet(io,ilog,ws,wd,tamb_tmp,ipg,iend)
	tamb = tamb_tmp
      
c first date record is       
c     jdatebmi  
c     isecbmi
c Assume a 5-year data set, so convert current date record to a date 
c to initialize range in file
c end date is 5years less one hour
      call dedat(jdatebmi,iyr,jday,ihr)
      call mkdat(iyr  , jday, ihr, jdatebm)
      if(MNONSEQ.eq.1) then
         iyr = iyr  + 7000
      else
         iyr = iyr + 5
      end if    
      call incr(io,iyr,jday,ihr,-1)
      call mkdat(iyr, jday, ihr, jdateem)
      isecbm = isecbmi
      isecem = isecemi
c      jdatebmi = jdatebm
c      jdateemi = jdateem
c      isecbmi = isecbm
c      isecemi = isecem
      
      write(ilog,*) 'AERMET anemometer height set to(m):',zws      

      return
      
90    continue
      iend =1
      
      return 
      end

c ---------------------------------------------------------------------
      subroutine metget_aermet(io,ilog,jdatex,isecx,ws,wd,tamb,ipg,
     & MNONSEQ)
c ---------------------------------------------------------------------
c
c --- METSERIES   Version: 1.00       Level: 110828
c --- B.Zelt 
c
c --- PURPOSE: Extract met information from TSERIES.DAT 
c
c --- UPDATES:
c --- v1.0(110828)
c        - 
c
c ---------------------------------------------------------------------
c --- MODULES
      use mod_stddefs
      use mod_met
c --- INPUTS
      integer :: io
      integer :: ilog
      integer :: jdatex
      real(8) :: ws        ! wind speed 
      real(8) :: wd        ! wind direction
      real(8) :: tamb      ! ammbient temperature
      integer :: ipg    ! PG stability class
      integer :: MNONSEQ ! flag for non sequential dates (AERMOD SCREENING MET)

	integer :: mignoresequence
      interface
      subroutine rdmet_aermet(io,ilog,ws,wd,tamb,ipg,iend)
         integer,intent(in):: io
         integer,intent(in):: ilog
         real(8),intent(out):: ws
         real(8),intent(out):: wd
         real(8):: tamb
         integer,intent(out):: iend,ipg
      end subroutine
      end interface      
      
      mignoresequence = MNONSEQ
      
c --- DECLARATIONS

c --- ignore dates and return next value
      if(mignoresequence.EQ.1) then
c ... if MNONSEQ and its the first pass, then idatecur.eq.jdatebmi
c     Then don't read met since it is current                 
         if( jdatex.ne.jdatebmi) then
            call rdmet_aermet(io,ilog,ws,wd,tamb,ipg,iend)
         end if 
c set current date to met file date         
         jdatex = jdatebmi
         if(iend.eq.1) then
            jdatebm = jdatex 
            isecbm = 3600
            jdateem = jdatex 
            isecem = 3600
         end if 
         return    
      end if 
      
c ---    Compute difference in seconds between two dates
         call DELTSEC(jdatebm,isecbm,jdatex,isecx,ndelsec1)
         call DELTSEC(jdatex,isecx,jdateem,isecem,ndelsec2)
 10      call DELTSEC(jdatebmi,isecbmi,jdatex,isecx,ndelsec3)
         call DELTSEC(jdatex,isecx,jdateemi,isecemi,ndelsec4)

         if(ndelsec1.lt.0) then
c --- stop if beginning date requested by user is earlier
c                     than data availability
         write( ilog,*)'ERROR- beginning date/time requested is earlier'
            write( ilog,*)'than data in AERMET TIME SERIES file'
            write( ilog,*)'date requested = ',jdatex,isecx
            write( ilog,*)'beginning date in file = ',jdatebm,isecbm
            write(*,*)'ERROR- beginning date/time requested is earlier'
            write(*,*)'than data in AERMET TIME SERIES file'
            write(*,*)'date requested = ',jdatex,isecx
            write(*,*)'beginning date in file = ',jdatebm,isecbm
          stop
         elseif(ndelsec2.lt.0 ) then
c --- stop if end date requested by user is earlier
c                     than data availability
            write( ilog,*)'ERROR- end date/time requested is later'
            write( ilog,*)'than data in AERMET TIME SERIES file'
            write( ilog,*)'date requested = ',jdatex,isecx
            write( ilog,*)'end date in file = ',jdateem,isecem
            write(*,*)'ERROR- end date/time requested is later'
            write(*,*)'than data in AERMET TIME SERIES file'
            write(*,*)'date requested = ',jdatex,isecx
            write(*,*)'end date in file = ',jdateem,isecem
          stop
         elseif(ndelsec3.lt.0) then
            write( ilog,*)'***ERROR***- bad read sequence'
            write( ilog,*)'date requested = ',jdatex,isecx
            write( ilog,*)'beginning date in file = ',jdatebm,isecbm
            write( ilog,*)'end date in file = ',jdateem,isecem
            write( ilog,*)'beginning date read = ',jdatebmi,isecbmi
            write( ilog,*)'end date read = ',jdateemi,isecemi
            write( *,*)'***ERROR***- bad read sequence'
            write( *,*)'date requested = ',jdatex,isecx
            write( *,*)'beginning date in file = ',jdatebm,isecbm
            write( *,*)'end date in file = ',jdateem,isecem
            write( *,*)'beginning date read = ',jdatebmi,isecbmi
            write( *,*)'end date read = ',jdateemi,isecemi
          stop
         elseif((ndelsec1.ge.0) .and. (ndelsec2.ge.0)
     &         .and. (ndelsec4 .gt. 0)) then
c --- current met period is still valid
c     just return input values
c           ws = still good
c           wd = still good
            return
         endif

c         elseif(ndelsec4.lt.0) then
c --- read next time step
        call rdmet_aermet(io,ilog,ws,wd,tamb,ipg,iend)

        if(iend .gt. 0) goto 3001
c --- recursive call
c     call metget(io,ilog,jdatex,isecx,ws,wd)            
c        or
c     start at top again and see if date is valid
         goto 10

c --- end of file
c     set dates to end of file dates
 3001 continue
c      if(mignoresequence.EQ.1) then
cc --- ignore dates and return next value
cc          call dedat(jdatex ,iyr,ijul,ihr)
c          jdatebm = jdatex
c          isecem = 3600
c      end if          
      jdatebmi = jdatebm
      isecbmi = isecem
      jdateemi = jdatebm
      isecemi = isecem

      return
      end

c ---------------------------------------------------------------------
c
c
c 
c aermet files are free format
c SFC HEADER
c 1   2   3   4     5   6  7   8   9     10   11   12 13  14  15  16  17  18    19    20    21   22   23   24  25
c YY  MM  DD  jDay  HH  H  u*  w*  VPTG  Zic  Zim  L  Zo  Bo  r   Ws  Wd  Zref  TmpK  Ztemp ??   ??   ??   ??  ??
c
c
c ---------------------------------------------------------------------
      subroutine rdmet_aermet(io,ilog,ws,wd,tamb,ipg,iend)
c ---------------------------------------------------------------------
c --- MODULES
      use mod_met
c --- INPUTS
      integer :: io     ! read io
      integer :: ilog   ! debug and list logging
      real(8), intent(out) :: ws
      real(8), intent(out) :: wd
      real(8) :: tamb
      integer, intent(out) :: ipg
      integer, intent(out) :: iend
c --- DECLARATIONS
      integer :: iyr,imo,iday,ijday,ihour
      real(8) :: rH,ustar,wstar,vptg,zic,zim,el,zo,bow
      real(8) :: alb,zref,tmpk,ztemp    
      
c
c ---    Step through meteorological fields
c -----------------------------------------
c ---    Read line
         wd=9999.
         ws=9999.

c --- Wind speed, ws is (m/s)
c --- Wind Direction, wd
c --- Temperature, tmpK
c --- PG stability 
      read(io,*,end=3001,err=3001,iostat=imsg) iyr,imo,iday,ijday,ihour,rH,ustar,wstar,
     &          vptg,zic,zim,el,zo,bow,alb,ws,wd,zref,tmpk,ztemp
      if(imsg.ne.0) then 
            goto 3001
      end if 
      
c --- convert dates from 2 decimals to 4      
      if(iyr .lt. 50) then
         iyr = iyr + 2000
      else if (iyr .lt. 100) then
         iyr = iyr + 1900
      end if
c --- convert hours to 0-23      
      ihour = ihour -1 
      
      iyrb   = iyr
      imob   = imo
      idayb  = iday
      ijdayb = ijday 
      ihourb = ihour
c
      isecbmi=0
      isecemi=3599

c ---    Create timestamp with time at beginning
      call TIMESTAMP(iyr,ijday,ihour,jdateemi)
      call TIMESTAMP(iyrb,ijdayb,ihourb,jdatebmi)

c --- convert K to K
      tamb =  tmpK

c --- Calculate PG stability 
      call LTOPG(iPG,el,zo)

c --- set anememeter height
      zws = zref
      
      iend = 0
      return

 3001 iend =1
      return
      end


C-----------------------------------------------------------------------
C                LTOPG Module of AERMOD Model
C
C        PURPOSE:  Converts Monin-Obukhov length to PG stability class
C                  for use with FASTAREA option, based on Golder (1972)
C
C        MODIFIED: The original LTOPG routine in AERMOD was replaced 
C                  with the LSTAB function from the CTDMPLUS model, which
C                  more closely matches the PG-class curves in Figure 4 
C                  of the Golder (1972) paper.
C                  R. W. Brode, U.S. EPA, OAQPS, AQMG, 02/29/2012
C
C LTOPG is based on the LSTAB function in the CTDMPLUS model:
C
C FUNCTION: LSTAB
C
C PURPOSE: THIS FUNCTION CALCULATES A P-G STABILITY CLASS GIVEN THE
C               MONIN-OBUKHOV LENGTH (L) AND THE SURFACE ROUGHNESS
C               LENGTH (Z0).
C
C ASSUMPTIONS: THE DIVIDING LINES BETWEEN CATEGORIES ARE ASSUMED TO BE
C               LINEAR.
C
C LIMITATIONS: THIS FUNCTION IS ONLY VALID FOR 0.01 <= Z0 <= 0.5(M).
C              HOWEVER, RESULTS ARE EXTENDED TO OTHER VALUES OF Z0 BY
C              USING Z0 = 0.01 IF Z0 < 0.01 M, AND BY USING Z0 = 0.5
C              IF Z0 > 0.5 M.
C
C ARGUMENTS
C  PASSED:
C       EL      REAL    MONIN-OBUKHOV LENGHT (M)
C       ZR0     REAL    SURFACE ROUGHNESS LENGTH (M)
C  RETURNED FUNCTION VALUE:
C       LSTAB   INT     P-G STABILITY CATEGORY 1=A, 2=B, ETC.
C
C CALLING ROUTINES: SEQMOD
C
C EXTERNAL ROUTINES: NONE
C
C INTERNAL FUNCTIONS:
C       XL - EQUATION OF DIVIDING LINE BETWEEN P-G STABILITY CLASSES
C
C INTRINSIC FUNCTIONS: ALOG
C
C REFERENCES:
C       GOLDER, D. (1972): RELATIONS AMONG STABILITY PARAMETERS IN THE
C                       SURFACE LAYER, BOUNDARY-LAYER METEOROLOGY, 3:56.
C
C-----------------------------------------------------------------------
C
C-----------------------------------------------------------------------
      SUBROUTINE LTOPG(LSTAB,OBULEN,SFCZ0)
C-----------------------------------------------------------------------
C     Variable Declarations
      IMPLICIT NONE

c --- inputs      
      REAL(8) :: OBULEN, SFCZ0, ZR0
      INTEGER :: LSTAB

c --- declarations      
      REAL(8) :: XEL, XL, YY, XM, B, Z0, EL

c --- inline function declaration
      XL(YY,XM,B)=XM/(DLOG(YY)-B)
C
      EL  = OBULEN
      ZR0 = SFCZ0
C
      Z0 = ZR0
      IF(Z0 .GT. 0.5D0) Z0 = 0.5D0
      IF(Z0 .LT. 0.01D0) Z0 = 0.01D0
      IF(EL .LT. 0.0D0) THEN
          XEL = -EL
          IF(XEL .LE. XL(Z0,-70.0D0,4.35D0)) THEN
C             STABILITY A
              LSTAB=1
            ELSE IF(XEL .LE. XL(Z0,-85.2D0,0.502D0)) THEN
C             STABILITY B
              LSTAB=2
            ELSE IF(XEL .LE. XL(Z0,-245.0D0,0.050D0)) THEN
C             STABILITY C
              LSTAB=3
            ELSE
C             STABILITY D
              LSTAB=4
          ENDIF
        ELSE
          IF(EL .GE. XL(Z0,-327.0D0,0.627D0)) THEN
C             STABILITY D
              LSTAB=4
            ELSE IF(EL .GE. XL(Z0,-70.0D0,0.295D0)) THEN
C             STABILITY E
              LSTAB=5
            ELSE
C             STABILITY F
              LSTAB=6
          ENDIF
      ENDIF
C
      RETURN
      END
