c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c     totalvolume = total volume to be flared (10^3m3)
c     duration    = flaring duration (s)
c     rate        = mix raw gas flare rate (10^3m3/d)
c     fgr         = mix fuel gas ratio
c     mix         = mix gas composition
c     MSTRIP      = 0 normal;  1=stripped gases source 
c  MSTRIP = 0 : SO2 100%
c  MSTRIP = 1 : H2S (stripped fraction as cold)
c  MSTRIP = 2 : SO2 actual
c  MSTRIP = 3 : H2S (all H2S is cold)
c  MSTRIP = 4 : OU  (stripped fraction as cold)
c  MSTRIP = 5 : OU  (all H2S is cold)
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
c      subroutine emissions(iolog,gcraw,rate,gcfuel,fgr,gclift,totalvolume,
c     &   hs,dia_f,dia_unob,wspd10m,zref,iPG,
c     &  MSTRIP,MFUELGAS,
c     &  MFASSIST,MFASSISTSTEAM,MFASSISTAIR,MFASSISTHPFUEL,
c     &  MLIFTGAS, MFLAMEOUT,
c     &  qliftgas,stm_dia,stm_press,tstm,stm_flow,
c     &  stm_nports,rairsteam,air_dia,air_flow_assist,air_nports,
c     &  hpfuel_dia,hpfuel_flow,hpfuel_nports,odourppb)

      subroutine emissions(iolog,gcrawIN,rateIN,fgrIN,
     &  totalvolume,wspd10m,ipgIN)
     
c MODULES      
      use mod_stddefs
      use mod_globals
      use mod_combustion
      
      use mod_readinp 

      integer :: iolog
      
c INPUTS
      real(8)  :: gcrawIN(mxcomp)  ! raw gas mix
      real(8)  :: rateIN        ! raw gas flare rate (10^3m3/d)
c      real(8)  :: gcfuel(mxcomp) ! fuel gas mix 
      real(8)  :: fgrIN         ! fuel gas ratio
c      real(8)  :: gclift(mxcomp) ! liftgas mix 
      real(8)  :: totalvolume ! total volume to be flared (10^3m3)
c      real(8)  :: hs          ! stack height (m)
c      real(8)  :: dia_f       ! stack inside diameter (m)
c      real(8)  :: dia_unob    ! stack unobstructed flow diameter (m)
      real(8)  :: wspd10m        ! reference wind speed (m/s)
c      real(8)  :: zref        ! windspeed reference height (m)
      integer :: iPGIN       ! PG stability class
c      integer :: MSTRIP	   ! 
c      integer :: MFASSIST	! iflareassist:0 no assist; 1=yes 
c      integer :: MFASSISTAIR,MFASSISTSTEAM,MFASSISTHPFUEL
c      integer :: MLIFTGAS  !0=no;1=yes lift gas
c      integer :: MFLAMEOUT !0=combustion; 1=H2S raw,; 2=raw gas flammability
c      real    :: qliftgas  ! lift gas flow rate
c      real(8) :: stm_dia			! steam assist individual port diameter [m]
c      real(8) :: stm_press		! steam assist pressure of steam or -1 if unknown
c      real(8) :: tstm     		! K, steam assist temperature of steam or -1 if unknown
c      real(8) :: stm_flow		! steam assist mass flow rate kg/s or flow rate e3m3/d
c      real(8) :: rairsteam    ! steam assist air to steam entrainment ratio (wt/wt)
c      integer :: stm_nports	! number of steam assist injection ports
c      integer :: air_nports	! number of air assist injection ports
c      real(8) :: air_dia		! air assist individual port diameter[m]
c      real(8) :: air_flow_assist		! air assist flow rate e3m3/d (without steam assist induced air)
c      integer :: hpfuel_nports	! number of HP fuel  assist injection ports
c      real(8) :: hpfuel_dia		! HP fuel assist individual port diameter[m]
c      real(8) :: hpfuel_flow		! HP fuel assist flow rate e3m3/d
c      real(8) :: odourppb

c 
      real(8) :: air_flow_accumulated ! air assist flow rate e3m3/d (total=air_assist(direct and induced) + steam_assist(induced air) + HPfuel_assist(induced)


c DECLARATIONS

      real(8)  :: dia_f       ! stack inside diameter (m)
      real(8)  :: dia_unob    ! stack unobstructed flow diameter (m)
      real(8)  :: dia_lift    ! lift gas line inside diameter (m)
      real(8)  :: dia_fuel    ! fuel line inside diameter (m)

      real(8)  :: air_flow_assist  ! local verion of air_flow    
      real(8)  :: wsST,wsST_min,wsST_max             ! stack top wind speed (m/s)
      real(8)  :: mix0(mxcomp)   ! raw + fuel + lift gasmix
      real(8)  :: mixa(mxcomp)   ! (work space) raw + fuel gas + air mix
      real(8)  :: mixb(mxcomp)   ! (work space) raw + fuel gas + air + steam mix 
      real(8)  :: mix(mxcomp)    ! final (raw + fuel + air + steam + lift) mix
      real(8)  :: mix_noair(mxcomp)    ! final (raw + fuel + steam + lift) mix
      real(8)  :: mix_air(mxcomp)    ! final (raw + fuel + steam + lift + ALL air assist) mix
      real(8)  :: gcstm(mxcomp)
      real(8)  :: gcair(mxcomp)
      real(8)  :: mix0_m     ! fuel + raw + lift gas mass flow
      real(8)  :: mix0_v     ! fuel + raw + lift gas volume flow
      real(8)  :: lift_v     ! liftgas volume flow
      real(8)  :: lift_m     ! liftgas mass flow 
      real(8)  :: fuel_v     ! fuelgas volume flow
      real(8)  :: fuel_m     ! fuelgas mass flow 
      real(8)  :: mix_common_v ! flow going up common stack 
      real(8)  :: finert  ! fraction of inerts in air
      real(8) :: tflared
      LOGICAL  :: fWAITFORVALUE
      real(8) :: tmix, egoal_mix, egoal_ex, egoal_xx
      real(8) :: e_h2s, e_so2, e_ou
      real(8) :: air_assist_m  , air_assist_v
      
c DECLARATIONS      
      real(8)  :: rmm,rnhv,rghv, rmm_mixair, rmm_xx, rnhv_mass 
      real(8)  :: mix_exhaust(mxcomp)
      real(8)  :: xx_mix_exhaust(mxcomp) ! no EXCESS AIR
      real(8)  :: rk3     ! Buoyancy Flux-Heat Release Rate Constant (K3),m4/(s2J)

c AERFLARE comparison vector
      real(8) :: cell(700)
      
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      function calc_cpbar(mix,tempk,inprop2)
         real(8) :: mix(*)
         real(8) :: tempk
         real(8) :: calc_cpbar      
         integer,optional :: inprop2
      end function         
      function calc_flametemp(tinit, mix, eGoal)
         real(8)  :: mix(*)
         real(8) :: tinit
         real(8) :: eGoal
         real(8)  :: calc_flametemp
      end function

      integer*4 function get_free_io (lu_max)
      integer*4 lu_max
      end function 
      
      end interface
      
      
c ASSIGNMENTS ---------------------------------------------------
      dia_f    = ds       ! stack inside diameter (m)
      dia_unob = df       ! stack unobstructed flow diameter (m)
c      dia_lift = dlift    ! lift gas line inside diameter (m)
c      dia_fuel = dfuel    ! fuel line inside diameter (m)


c =============================================================================      
c Basic properities 1
c =============================================================================      
c set standard steam gas compositions      
      ier = initialize(gcstm,0.0D0)
      gcstm(jph2o) = 1.0
c set standard air gas compositions      
      ier = initialize(gcair,0.0D0)
      do i=1,mxcomp
         gcair(i) = prop(ipair,i)
      end do   
      ier  = irenormalize(gcair)      
      finert = 0.0
      do i=1,mxcomp
         if(prop(ipinert,i)>0.) then
            finert = finert + gcair(i)
         end if 
      end do

c --- Adjust Wind Speed -- Assume Wind Speed Constant Below 10 meters
      call wsstkht(wspd10m, zref, ipgIN, hs, wsST)      
c  Minimum Wind Speed at Exit Height (UMIN)	m/s
      call wsstkht(wsmin, zref, ipgIN, hs, wsST_min)      
c  Reference Wind Speed at Exit Height = Minimum Recommended Exit Velocity (VMIN)	m/s
      call wsstkht(wsmax, zref, ipgIN, hs, wsST_max)      
      if(mgroundflare.eq.1) then
        wsST = 0.001
        wsST_min = wsST
        wsST_max = wsST
      end if 

      rmmraw   = calc_molarmass(gcrawIN)  ! kg/kmole
      rmmstm   = calc_molarmass(gcstm)    ! prop(ipmmass,jph2o)      ! kg/kmole
      rmmair   = calc_molarmass(gcair)    ! prop(ipmmass,jpair)      ! kg/kmole
      rmmlift  = calc_molarmass(gclift)   ! kg/kmole
      rmmfuel  = calc_molarmass(gcfuel)   ! kg/kmole
      
      if(rmmlift.eq.0.d0) then
        gclift(jpn2) = 1.0
        rmmlift  = calc_molarmass(gclift)   ! kg/kmole
      end if 

c =============================================================================      
c --- Average Cp, Air
c =============================================================================      
      cpmol_air_tref  = calc_cpmix(tref, gcair)    ! nasa
      cpmol_air_tamb  = calc_cpmix(tamb, gcair)    ! nasa
c      cpmol_air      = calc_cp(jpair, tref)    ! nasa
      z_air_tamb      = fnZ(tamb,plocal,tcair,pcair)
      z_air_tref      = fnZ(tref,plocal,tcair,pcair)
      z_air_tamb      = 1.0
      z_air_tref      = 1.0
      
      den_air_tamb    = plocal*rmmair/ru/tamb/z_air_tamb
      den_air_tref    = plocal*rmmair/ru/tref/z_air_tref
c Air Density at air temperature and site pressure,kg/m
      den_air         = plocal*rmmair  /(ru*tamb)

      cpmol_air_ave_tamb  = calc_cp_ave_mix(tamb, tref, gcair)
      cpmass_air_ave_tamb = cpmol_air_ave_tamb /rmmair  ! is it the same???
      
c --- Calculate the sonic velocity of steam at exit temp and pressure      
c ratio of specific heats      
      rk_air       = cpmol_air_tamb/(cpmol_air_tamb-ru)
c air temp at exit      
      tair_exit   = tamb*(2./(rk_air+1.D0))   
c air pressure at exit
c estimate of minimum sonic pressure
      PDropRatio       = (2.D0/(rk_air+1.D0))**(rk_air/(rk_air-1.D0))
      air_press_exit   = plocal 
      z_air_tamb       = fnZ(tair_exit,air_press_exit,tcair,pcair)
      air_den_exit     = plocal* rmmair/(ru*tair_exit*z_air_tamb)
      
      call sonicVelocity(cpmol_air_tamb,  rmmair, tamb, z_air_tamb, c_air_tamb)
      call sonicVelocity(cpmol_air_tref,  rmmair, tref, z_air_tref, c_air_tref)
      
c update in the future to have air assist at different temperature than Tamb      
      DenRatioAir = 1.0
      EntrainmentRatioAirAir = EntrainmentRatio(rairair, DenRatioAir, Tamb, Plocal)
      
c =============================================================================      
c --- Average Cp, Steam
c =============================================================================   
      tstm = stm_temp
      pstm = stm_press
      
      cpmol_stm_tref      = calc_cpmix(tref, gcstm)
      cpmol_stm_tstm      = calc_cpmix(tstm, gcstm )
      
      hf_stm_tstm     = calc_hf(jph2o, tstm)
      hf_stm_tref     = calc_hf(jph2o, tref)
      z_stm_tstm      = fnZ(tstm,pstm,tch2o,pch2o)
      z_stm_tref      = fnZ(tref,plocal,tch2o,pch2o)
      den_stm_tref    = plocal*rmmstm/ru/tref   / z_stm_tref

c Steam Density at stm temperature and site pressure,kg/m
      den_stm     = plocal*rmmstm  /(ru*tstm)

c      cpmol_stm_ave_tstm   =  cpmol_stm
c      if(tstm .ne. tref .and. mfAssistSteam .eq. 1 .and. tstm .ne. 0. ) then 
c          cpmol_stm_ave_tstm   = (hf_stm_tref-hf_stm_tstm)/(tref-tstm)
c      end if       
c      cpmass_stm_ave_tstm  = cpmol_stm_ave_tstm / rmmstm
      cpmol_stm_ave_tstm  = calc_cp_ave_mix(tstm, tref, gcstm)
      cpmass_stm_ave_tstm = cpmol_stm_ave_tstm /rmmstm
      
      
c --- Calculate the sonic velocity of steam at exit temp and pressure      
c ratio of specific heats      
      rk_stm       = cpmol_stm_tstm/(cpmol_stm_tstm-ru)
c stm temp at exit      
      tstm_exit   = tstm*(2./(rk_stm+1.))   
c stm pressure at exit
      PDropRatio       = (2.D0/(rk_stm+1.D0))**(rk_stm/(rk_stm-1.D0))
      stm_press_exit   = pstm * PDropRatio
      z_stm_tstm_exit  = fnZ(tstm_exit,stm_press_exit,tch2o,pch2o)
      stm_den          = stm_press_exit* rmmstm/(ru*tstm_exit*z_stm_tstm_exit)
      
      call sonicVelocity(cpmol_stm_tstm,  rmmstm, tstm_exit, z_stm_tstm_exit, c_stm_tstm)
      call sonicVelocity(cpmol_stm_tstm,  rmmstm, tref, z_stm_tref, c_stm_tref)

      den_stm_tstm    = plocal*rmmstm/(ru * tstm_exit * z_stm_tstm_exit)
      DenRatioStm = den_air_tamb/ den_stm_tstm
      EntrainmentRatioAirStm = EntrainmentRatio(rairsteam, DenRatioStm, tstm_exit, Plocal)
      
      
c =============================================================================      
c --- HP Fuel Assist
c =============================================================================      
      cpmol_fuel = calc_cpmix(tref, gcfuel )  ! calc_byprop(gcfuel,  ipcpmole, ipcalc)
      
      den_fuel_tgas  = plocal*rmmfuel/ru/tgas
      DenRatioFuel    = den_air_tamb/ den_fuel_tgas  
      EntrainmentRatioAirFuel = EntrainmentRatio(rairfuel, DenRatioFuel, Tgas, Plocal)
      z_fuel_tgas = 1.0 ! assumption
      call sonicVelocity(cpmol_fuel,  rmmfuel, tgas, z_fuel_tgas, c_fuel_tgas)

c =============================================================================      
c --- Lift Gas
c =============================================================================      
      cpmol_lift = calc_cpmix(tref, gclift  )  ! calc_byprop(gclift,  ipcpmole, ipcalc)
      z_lift_tgas = 1.0 ! assumption
      call sonicVelocity(cpmol_lift,  rmmlift, tgas, z_lift_tgas, c_lift_tgas)
      
      
c =============================================================================      
c --- FLOW RATES
c =============================================================================      
      air_flow_accumulated = 0.0
c force or correct entries
      if(MLIFTGAS.eq.0) then
         qliftgas = 0.
      end if 
      
      if(MFASSIST.eq.0) then
         stm_flow = 0.   
         air_flow_assist = 0.   
         hpfuel_flow = 0.   
      else
         if(MFASSISTAIR.eq.1) then
            air_flow_assist = air_flow
            air_flow_assist = air_flow_assist + EntrainmentRatioAirAir * air_flow !* (rmmair/rmmair) ! note: air assist could be different temperature than tamb
            air_flow_accumulated = air_flow_assist 
         else
            air_flow_assist = 0.
         end if 
c ----------------------------------------------------------------------------------      
c if Steam Assist... 
c then this implies air assist by entrainment
c Calculate air flow (e3m3/d)
c
c NOTE:
c You can't have AIR_ASSIST and STEAM_ASSIST on the same stack
c 
c ----------------------------------------------------------------------------------      
         if(MFASSISTSTEAM.eq.1) then
            MFASSISTAIR = 1
            air_flow_assist = air_flow_assist + EntrainmentRatioAirStm * stm_flow !* (rmmstm/rmmair)
            air_flow_accumulated = air_flow_assist 
         else
            stm_flow = 0.
         end if 
         
         if(MFASSISTHPFUEL.eq.1) then
            MFASSISTAIR = 1
            air_flow_assist = air_flow_assist + EntrainmentRatioAirFuel * hpfuel_flow !* (rmmfuel/rmmair)
            air_flow_accumulated = air_flow_assist 
         else
            hpfuel_flow=0.
         end if 
      end if 
      dia_eff = dia_f
      

c --- 131128, changed ambient temperature to K default      
c      tambk = tamb + rk0
      if(debug .gt. 0 ) then
      write(iolog,*) 'wspd10m = ',wspd10m,'  tamb=',tamb,'  zref=',zref
      write(iolog,*) 'wsST = ',wsST,'   hs=',hs
      end if 

      
c ----------------------------------------------------------------------------------      
c Calculate the Required air 
c Two passes through the calculations are required. 
c First calculate the amount of air required without air assist
c then, adjust the air assist by removing the air required
c Only when EXCESS air is supplied, does it impact the calculations
c ----------------------------------------------------------------------------------      
      air_flow_adjusted = 0.   ! this is the net air contributing to dilution of fuel stream
      fWAITFORVALUE  = .FALSE.     
      if( MFASSIST .eq. 1) then ! .and. MFASSISTAIR .eq. 1
         fWAITFORVALUE = .TRUE.     
         ! _assist = air_flow  ! save for later
         air_flow_assist = 0.               ! first assume no air assist
      end if 
      
c *********************************************************************
c *********************************************************************
c if MFASSISTAIR 
c Then first do calculations with out air to determine combustion
c requirements, then loop back to here to determine the net air assist
c to determine if too much air has been added.      
c
c This portion of the code should be recoded to be a subroutine      
c
c *********************************************************************
c *********************************************************************
1000  continue      
c *********************************************************************
c *********************************************************************

      if( MFASSIST .eq. 1 ) then
c ... Calculate the net air assist since the combustion requires air          
         if(fWAITFORVALUE .eq. .FALSE.) then !  .and. MFASSISTAIR .eq. 1
            air_flow_assist   = air_flow_accumulated 
            air_flow_adjusted = air_flow_accumulated
            if(MFLAMEOUT .eq.0 .and. isFlammable.eq.1) then
              air_flow_adjusted = max(air_flow_assist - air_amb_v*(24.*3600.*.001),0.)
            end if 
         end if 
      end if 

c ----------------------------------------------------------------------------------      
c Calculate the flared mix
c adjust the air assist accounting for required air
c ----------------------------------------------------------------------------------      

c Calculate flare assisted final mix      
c Mix = RAW+FUEL ----- > mixb
      if( MFUELGAS .eq. 1 ) then   
         ierr = inormalize(gcrawIN, gcfuel, fgrIN, mixb)
         totalflaringrate_ve3m3 = calc_totalrate(rateIN, fgrIN)
      else
         ierr = inormalize(gcrawIN, gcrawIN, 1.0D0, mixb)
         totalflaringrate_ve3m3 = rateIN
      end if 
      
      if( MFASSISTHPFUEL.eq.1) then   
         ierr = inormalize(mixb, gcfuel, hpfuel_flow/totalflaringrate_ve3m3  , mixa)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3  + hpfuel_flow 
      else
         ierr = inormalize(mixb, mixb, 1.0D0, mixa)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
      end if 
      
c Mix = Mix + Lift  ----- > mix0
      if( MLIFTGAS .eq. 1) then   
         ierr = inormalize(mixa, gclift, qliftgas/totalflaringrate_ve3m3, mix0)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3 + qliftgas
      else
         ierr = inormalize(mixa, mixa, 1.0D0, mix0)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
      end if 
      
c Mix = Mix + STEAM  ----- > mix_noair
      if( MFASSIST.eq.1 .and. MFASSISTSTEAM .eq. 1 ) then 
         ierr = inormalize(mix0, gcstm, stm_flow/totalflaringrate_ve3m3, mix_noair)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3 + stm_flow
      else
         ierr = inormalize(mix0, mix0, 1.0D0, mix_noair)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
      end if 

c (Don't need this, now using two passes.)      
c no excess air mixture (pre flare assist)      
c???      totalflaringrate_ve3m3_noair = totalflaringrate_ve3m3  

c Mix = Mix + AIR  ----- > mix
      if( MFASSIST.eq.1 .and. MFASSISTAIR .eq. 1) then 
         ierr = inormalize(mix_noair, gcair, air_flow_adjusted/totalflaringrate_ve3m3, mix)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3 + air_flow_adjusted
      else
         ierr = inormalize(mix_noair, mix_noair,  1.0D0, mix)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
      end if 

c Mix = Mix + AIR  ----- > mix_air
c this is the mix with all added air
      if( MFASSIST.eq.1 .and. MFASSISTAIR .eq. 1) then 
         ierr = inormalize(mix_noair, gcair, air_flow_assist/(totalflaringrate_ve3m3), mix_air)
      else 
         ierr = inormalize(mix_noair, mix_noair, 1.0D0, mix_air)
      end if 
      
c =============================================================================      
c Basic properities 2
c =============================================================================  
      rmm_netair = calc_molarmass(mix)       ! kg/kmole, final mix 
      rmm_mixair = calc_molarmass(mix_air)   ! kg/kmole, final mix with ALL ASSISTED air
      rmm_noair  = calc_molarmass(mix_noair) ! kg/kmole
      rmmflare   = calc_molarmass(mix0)     ! kg/kmole
    
      rmm        = rmm_noair 
      if(fWAITFORVALUE .eq. .FALSE.) then
         rmm      = rmm_netair 
      end if       
      
c      if(MFUELGAS .eq. 1) then
c         mix0_v = (qliftgas+calc_totalrate(rate, fgr))* 1000./(24.*3600.)  ! volume flow rate, m3/s
c         mix0_m = mix0_v/spv*rmmflare              ! mass flow rate, kg/s
c      else
c         mix0_v = (qliftgas+calc_totalrate(rate, 0.D0))* 1000./(24.*3600.)  ! volume flow rate, m3/s
c         mix0_m = mix0_v/spv*rmmflare              ! mass flow rate, kg/s
c      end if 
      raw_v    = rateIN* 1000./(24.*3600.)          ! volume flow rate, m3/s
      raw_m    = raw_v/spv*rmmraw                 ! mass flow rate, kg/s
      fuel_v   = fgrIN/(1.+fgrIN)*calc_totalrate(rateIN, fgrIN)* 1000./(24.*3600.)  ! volume flow rate, m3/s
      fuel_m   = fuel_v/spv*rmmfuel               ! mass flow rate, kg/s
      stm_v    = stm_flow* 1000./(24.*3600.)      ! volume flow rate, m3/s
      stm_m    = stm_v/spv*rmmstm                 ! mass flow rate, kg/s
      air_v    = air_flow_assist* 1000./(24.*3600.)      ! volume flow rate, m3/s
      air_m    = air_v/spv*rmmair                 ! mass flow rate, kg/s
      air_adjusted_v  = air_flow_adjusted* 1000./(24.*3600.)      ! volume flow rate, m3/s
      air_adjusted_m  = air_adjusted_v/spv*rmmair                 ! mass flow rate, kg/s
      air_assist_v  = air_flow_assist* 1000./(24.*3600.)      ! volume flow rate, m3/s
      air_assist_m   = air_assist_v/spv*rmmair                 ! mass flow rate, kg/s
      hpfuel_v    = hpfuel_flow* 1000./(24.*3600.)      ! volume flow rate, m3/s
      hpfuel_m    = hpfuel_v/spv*rmmfuel                ! mass flow rate, kg/s
      lift_v      = qliftgas* 1000./(24.*3600.)     ! volume flow rate, m3/s
      lift_m      = lift_v/spv*rmmlift              ! mass flow rate, kg/s

      mix0_v = raw_v + fuel_v + lift_v  ! volume flow rate, m3/s
      mix0_m = raw_m + fuel_m + lift_m  ! mass flow rate, kg/s
      
c =============================================================================      
c EMISSIONS INIT
c =============================================================================      
      rnhv = calc_nhv(mix)             ! MJ/m3
      rghv = calc_ghv(mix)             ! MJ/m3
      rnhv_mass = rnhv/rmm_netair*spv  ! MJ/kg
      
      cpmol_mix  = calc_cpmix(tref,mix    )  ! calc_byprop(mix,     ipcpmole, ipcalc)
      cpmol_raw  = calc_cpmix(tref,gcrawin)  ! calc_byprop(gcrawIN,   ipcpmole, ipcalc)
      cpmol_fuel = calc_cpmix(tref,gcfuel )  ! calc_byprop(gcfuel,  ipcpmole, ipcalc)
      cpmol_stm  = calc_cpmix(tref,gcstm  )  ! calc_byprop(gcstm,   ipcpmole, ipcalc) ! or calc_cp(jph2o, tref)
      cpmol_air  = calc_cpmix(tref,gcair  )  ! calc_byprop(gcair,   ipcpmole, ipcalc) ! or calc_cp(jpair, tref)    ! fneq90
      cpmol_lift = calc_cpmix(tref,gclift  )  ! calc_byprop(gclift,  ipcpmole, ipcalc)
      cpmass_mix  = 0.
      cpmass_raw  = 0.
      cpmass_fuel = 0.
      cpmass_stm  = 0.
      cpmass_air  = 0.
      cpmass_lift = 0.
      if(rmm_netair .gt.0)    cpmass_mix   = cpmol_mix/ rmm_netair
      if(rmmraw     .gt. 0.)  cpmass_raw   = cpmol_raw/ rmmraw
      if(rmmfuel    .gt. 0.)  cpmass_fuel  = cpmol_fuel/ rmmfuel
      if(rmmstm     .gt. 0.)  cpmass_stm   = cpmol_stm/ rmmstm
      if(rmmair     .gt. 0.)  cpmass_air   = cpmol_air/ rmmair
      if(rmmlift    .gt. 0.)  cpmass_lift  = cpmol_lift/ rmmlift

c =============================================================================      
c IS FLAMMABLE ??
c =============================================================================       
c combustible fraction of mixture
      combfrac = calc_byprop(mix, ipcomb, ipcalc) 
c Lower Flammability Limit (LFL) of combustible flared gas
      rLFLcomb    = calc_lfl(mix)
c Lower Flammability Limit (LFL) of flared mixture
      rLFLflared  = 0.
      if(combfrac .gt.0.)then
          rLFLflared  = rLFLcomb/combfrac
      end if 
c Minimum Oxygen Concentration to support Combustion of combustibles
      rminoxyg    = rLFLcomb*calc_byprop(mix, ipstoico2, ipcalc, ipcomb)
     &                 / combfrac
c Maximum Inert Concentration of flared mixture
      rmaxinert   = 1. - rLFLcomb - rminoxyg
c Is proposed mixture flammable? (but we assume it is for now)
      isFlammable = 1
      if(1.-combfrac .gt. rmaxinert ) then
          isFlammable = 0
      end if 
      
      if(debug .gt. 0 ) then
      write(iolog,*) 'combfrac=',combfrac
      write(iolog,*) 'rLFLcomb=',rLFLcomb
      write(iolog,*) 'rLFLflared=',rLFLflared
      write(iolog,*) 'isFlammable(1=y,0=n)=',isFlammable
      end if       
            
      
c =============================================================================      
c EMISSIONS
c =============================================================================      
      totalflaringrate_v = totalflaringrate_ve3m3* 1000./(24.*3600.) ! m3/s at ref
      totalflaringrate_m = totalflaringrate_v*rmm_netair/spv                ! kg/s

      rnetheatreleased_a = totalflaringrate_ve3m3 * rnhv * rk1     ! cal/s
c n/r      rnetheatreleased_b   = totalflaringrate_v* rnhv*3600./1000.  ! GJ/h
      rnetheatreleased_c   = totalflaringrate_ve3m3 * rnhv * rk4     ! J/s
c H2S Release Rate at 0% conversion
      e_h2s_v   = totalflaringrate_v  * mix(jph2s)     ! m3/s
      e_h2s_m   = e_h2s_v * prop(ipmmass,jph2s)/spv * 1000. !g/s
c OU release rate at 0% converation
      e_ou_0  = totalflaringrate_v*
     &         (mix(jph2s)/odourppb*1.e9) ! OU/s

c SO2 Emission Rate at 100% conversion
      if(MFLAMEOUT.ge.1 .or. isFlammable.eq.0 ) then
        e_so2_m1 = 0.D0
      else
        e_so2_m1  = totalflaringrate_ve3m3*mix(jph2s) /
     &spv*prop(ipmmass,jpso2)   ! t/d
        e_so2_m   = e_so2_m1 * 1000.*1000./ (24.*3600.)                            ! g/s
      end if 
      e_s_m1    = e_so2_m1 * prop(ipmmass,jps)/ prop(ipmmass,jpso2)    ! t/d
      e_s_mtotal= e_s_m1 * totalvolume/ rateIN !  t
      
      if(calc_hydrocarbons(mix).eq.0.0) Then
          fuel_carbons=0.0  
      else    
         fuel_carbons = calc_byprop(mix, ipstoicc, iph)
     &      / calc_hydrocarbons(mix)
      end if    
      fuel_hydrogen= fuel_carbons*2. + 2.   
c Process CO2 Emission Rate, t/d      
      e_co2_process    = mix(jpco2)*prop(ipmmass,jpco2)/spv*
     &        totalflaringrate_ve3m3                            
c Combustion CO2 Emission Rate at 100% conversion, t/d     
      if(MFLAMEOUT.ge.1 .or. isFlammable.eq.0 ) then
        e_co2_combustion = 0.D0
      else
        e_co2_combustion = (calc_byprop(mix, ipstoicc, iph) + 
     &        mix(jpco)*prop(ipstoicc,jpco))* 
     &        prop(ipmmass,jpco2)/ spv * totalflaringrate_ve3m3 
      end if 
      e_co2_m1    = (e_co2_process+e_co2_combustion)
      e_co2_total = e_co2_m1  * totalvolume/ rateIN  ! t

c      ue_epa = 
c      usmax  = ???
c      diamin=1000.*sqrt(4./pi*totalflaringrate_v*(pref/plocal)*(tgas/tref)/ue_epa)
c      diamax=1000.*sqrt(4./pi*averageflaringrate_v *(pref/plocal)*(tgas/tref)/usmax)

      if(debug .gt. 0 ) then
         write(iolog,*) 'NHV (MJ/m3)=',rnhv
         write(iolog,*) 'GHV (MJ/m3)=',rghv
      end if                  

      
c =============================================================================      
c TFLARED
c =============================================================================      
c --- Temperature of gas being combusted with Assist
c =============================================================================      
c Flared Gas Enthalpy, kJ/s      
      hf_raw = raw_m/rmmraw   * 
     &         (calc_hf1mix(gcrawIN, tgas) - calc_hf1mix(gcrawIN, tref))  
      hf_lift= 0.
      hf_fuel= 0.
      hf_stm = 0.
      hf_air = 0.
      hf_hpfuel = 0.
      if(MLIFTGAS .eq. 1) then
         hf_lift= lift_m/rmmlift * 
     &         (calc_hf1mix(gclift, tgas) -calc_hf1mix(gclift, tref))
      end if 
      if(MFUELGAS .eq. 1) then
         hf_fuel= fuel_m/rmmfuel * 
     &         (calc_hf1mix(gcfuel, tgas) -calc_hf1mix(gcfuel, tref)) 
      end if 
      if( MFASSIST .eq. 1 ) then 
      if( MFASSISTSTEAM .eq. 1 ) then 
         hf_stm = stm_m/rmmstm   * 
     &         (calc_hf1mix(gcstm, tstm) -calc_hf1mix(gcstm, tref)) 
      end if          
      if( MFASSISTAIR.eq. 1 ) then 
         hf_air = air_adjusted_m/rmmair   * 
     &         (calc_hf1mix(gcair, tamb) -calc_hf1mix(gcair, tref)) 
      end if          
      if( MFASSISTHPFUEL .eq. 1 ) then 
         hf_hpfuel = hpfuel_m/rmmfuel   * 
     &         (calc_hf1mix(gcfuel, tgas) -calc_hf1mix(gcfuel, tref)) 
      end if          
      end if 
      hf_mix = hf_raw + hf_lift + hf_fuel + hf_stm + hf_air + hf_hpfuel
c Estimated Temperature of Flared Gas, K
      tmix = hf_mix/ (cpmass_mix*totalflaringrate_m) + tref
c      rmdotcp_raw = raw_m *cpmass_raw *tgas
c      rmdotcp_lift= lift_m*cpmass_lift*tgas
c      rmdotcp_fuel= fuel_m*cpmass_fuel*tgas
c      rmdotcp_stm = stm_m *(cpmol_stm/rmmstm)  *tstm 
c      rmdotcp_air = air_m *(cpmol_air/rmmair)  *tamb 
c      rmdotcp_mix = rmdotcp_raw + rmdotcp_lift + 
c     &              rmdotcp_fuel +rmdotcp_stm +rmdotcp_air
c Estimated Temperature of Flared Gas, K
c      tmix  = rmdotcp_mix/totalflaringrate_m/cpmass_mix 
c goal enthalpy
c      egoal = rmdotcp_mix/totalflaringrate_m*rmm - cpmol_mix*tref

c goal enthalpy
      egoal_mix = hf_mix/totalflaringrate_m*rmm_netair 
c Temperature of Flared Gas, K
      extemp   = calc_flametemp(tmix, mix, egoal_mix)  
c      extemp   = MAX(extemp, tamb+rk0)
      tflared = extemp   ! check : this should be same as tgas

c mixed flared gas Density at release temperature and site pressure,kg/m
c NOTE: THIS density of flare: is calculated with the total stack top flow
c including ALL air assist
      den_flare   =plocal*rmm/(ru*tflared)
      den_mix     =plocal*rmm_mixair/(ru*tflared)
c =============================================================================        
c --- unassisted flare gas sonic velocity    
      rk_mix = cpmol_mix/(cpmol_mix - ru)
      z_mix_tgas = 1.0 ! assumption
      call sonicVelocity(cpmol_mix,  rmm_netair, tflared, z_mix_tgas, c_flare_tgas)
       ! tflared
     
c =============================================================================      

      
c =============================================================================      
c UEFLARE  & 
c MOMENTUM 
c =============================================================================      
c -------------------------------------------------------------------------------      
c Exit Velocity
c --- MOMENTUM
c Exit Velocity (VE) at release temperature and site pressure,m/s
c (Raw + Fuel + Lift) Exit Velocity at flare tip temperature and site pressure      
c -------------------------------------------------------------------------------      
      mix_common_v = raw_v
      if( mdlift .eq.0) then
         mix_common_v = mix_common_v + lift_v
      end if 
      if( mdfuel .eq.0) then
         mix_common_v = mix_common_v + fuel_v
      end if 
      
      if( mgroundflare.eq.1) then
        ueflare = c_flare_tgas
        uelift = c_lift_tgas
        uefuel = c_fuel_tgas
      else 
        ueflare = min(mix_common_v*4./pi/(dia_unob*dia_unob)*tgas/tref
     &        *pref/plocal,c_flare_tgas)

        uelift = ueflare
        if( mdlift .eq. 1) then 
           uelift = min(lift_v*4./pi/(dlift*dlift)*(tgas/tref)
     &        *(pref/plocal),c_lift_tgas)
        end if 

        uefuel = ueflare
        if( mdfuel .eq. 1) then 
          uefuel = min(fuel_v*4./pi/(dfuel*dfuel)*(tgas/tref)
     &        *(pref/plocal),c_fuel_tgas)
        end if 
      end if 
      
      uestm   =0.0
      if( stm_v .gt. 0.) then
         uestm = c_stm_tstm
      end if 
      
      ueair   = 0.
      if(air_v .gt. 0.) then
         ueair = c_air_tamb
      end if 
      
      uehpfuel   = 0.
      if(hpfuel_v .gt. 0.) then
         uehpfuel = c_fuel_tgas
      end if 

      
c =============================================================================      
c --- MOMENTUM  
c =============================================================================      
c Note that momentum flux is RELATIVE TO AIR 
c Flared Incremental Momentum Flux FM,m4/s2
      FM_flare  = ueflare*raw_m/den_air/pi
c Lift Gas Increment Momentum Flux FM,m4/s2 (if separate stream pipe)
      FM_lift  = uelift*lift_m/den_air/pi
c Fuel Gas Increment Momentum Flux FM,m4/s2 (if separate stream pipe)
      FM_fuel  = uefuel*fuel_m/den_air/pi
c Steam Incremental Momentum Flux FM,m4/s2
      FM_stm    = uestm*stm_m/den_air/pi
c Air Incremental Momentum Flux FM,m4/s2
      FM_air    = ueair*air_m/den_air/pi
c HPFuel Incremental Momentum Flux FM,m4/s2
      FM_hpfuel = uehpfuel*hpfuel_m/den_air/pi
c =============================================================================      

      

      
c =============================================================================      
c --- COMBUSTION CALCULATIONS 1 -- With Excess air
c =============================================================================      
      

c air for MOC (mole air/ mole fuel)
      rmamf_moc = (rminoxyg-mix(jpo2))/(1.-finert-rminoxyg)
c mole air/ mole fuel to give LFL
      rmamf = (1.-rLFLflared)/rLFLflared
c stoichiometric mole O2 required / mole fuel
      stoic_o2 =calc_byprop(mix, ipstoico2, ipcalc)- mix(jpo2)

c stoichiometric mole air required / mole fuel
      stoic_air=stoic_o2/(1.-finert)
c Flame Excess Air for LFL or MOC,% 
      fealfl   =(DMAX1(rmamf_moc,rmamf)-stoic_air)/stoic_air    
c mole O2 supplied / mole fuel
      rfsupo2    =DMAX1(rmamf_moc,rmamf)*(1.-finert) + mix(jpo2)
c mole wet exhaust / mole fuel
      exwet    = calc_byprop(mix, ipstoicp, ipcalc)  
     &        + DMAX1(rmamf_moc,rmamf)/(1.+fealfl)*(finert+fealfl)  
     &        + DMAX1(rmamf_moc,rmamf)  * gcair(jph2)
c mole dry exhaust / mole fuel
      exdry    =exwet
     &        - calc_byprop(mix, ipstoich2o, ipcalc)
     &        - DMAX1(rmamf_moc,rmamf)  * gcair(jph2)
c mole H2O / mole fuel
      rfh2o    =exwet-exdry  + DMAX1(rmamf_moc,rmamf)*gcair(jph2)
c mole O2 / mole fuel
      rfo2     =rfsupo2-stoic_o2
c mole Ar / mole fuel
      rfar     =mix(jpar) + DMAX1(rmamf_moc,rmamf)*gcair(jpar)
c N2	mole exhaust / mole fuel	
      rfn2     = mix(jpn2) + DMAX1(rmamf_moc,rmamf)*gcair(jpn2)
c He	mole exhaust / mole fuel	
      rfhe     = mix(jphe) + DMAX1(rmamf_moc,rmamf)*gcair(jphe)
c Ne	mole exhaust / mole fuel	
      rfne     = mix(jpne) + DMAX1(rmamf_moc,rmamf)*gcair(jpne)
c Kr	mole exhaust / mole fuel   
      rfkr     = mix(jpkr) + DMAX1(rmamf_moc,rmamf)*gcair(jpkr)
c N2O	mole exhaust / mole fuel	
      rfn2o    = mix(jpn2o) + DMAX1(rmamf_moc,rmamf)*gcair(jpn2o)
c Xe	mole exhaust / mole fuel	
      rfxe    = mix(jpxe) + DMAX1(rmamf_moc,rmamf)*gcair(jpxe)
c mole CO2 / mole fuel
      rfco2    = calc_byprop(mix, ipstoicc, ipcalc) 
     &      + DMAX1(rmamf_moc,rmamf)*(gcair(jpco2)+gcair(jpch4)+gcair(jpco))
c mole SO2 / mole fuel
      rfso2    = mix(jph2s)
c mole Excess Air / mole fuel
      excessair=rfo2/(1.-finert)
c -------------------------------------------------------------          
c O2 fraction in exhaust wet	%	
      o2FracExWet = rfo2/exwet
c O2 fraction in exhaust dry	%	
      o2FracExDry = rfo2/exdry
c Exhaust Flow	kmol/s	
      exFlow_m = totalflaringrate_v/spv*exwet
c Exhaust flow at Pref and Tref	Sm/s	
      if(rnhv .le. 0. ) then 
         exFlow_v = totalflaringrate_v
      else 
         exFlow_v = totalflaringrate_v*exwet
      end if 
c Air required at Pref and Tref	 m/s	
      airFlow_v = exFlow_v*rmamf/exwet
c Exhaust is 	--	=IF(rnhv <=0,"Source Gas","Combustion Gas")

c --- initialize exhaust mixture
      ier = initialize(mix_exhaust,0.0D0)
      mix_exhaust(jpair) = 0
      mix_exhaust(jph2o) = rfh2o/ exwet
      mix_exhaust(jpo2)  = rfo2/ exwet
      mix_exhaust(jpar)  = rfar/ exwet
      mix_exhaust(jpn2)  = rfn2/ exwet
      mix_exhaust(jphe)  = rfhe/ exwet
      mix_exhaust(jpne)  = rfne/ exwet
      mix_exhaust(jpkr)  = rfkr/ exwet
      mix_exhaust(jpn2o) = rfn2o/ exwet
      mix_exhaust(jpxe)  = rfxe/ exwet
      mix_exhaust(jpco2) = rfco2/ exwet
      mix_exhaust(jpso2) = rfso2/ exwet

c      r=0.D0
c      do j=1,mxcomp
c         r = r + mix_exhaust(j)
c      end do
      
c Molar Mass	kg/kmol	
c Molar Specific Heat at Tref	kJ/(kmolK)	=IF(rnhv_mass<=0,C99,SUMPRODUCT(PROPERTIES!$F$53:$F$63,C390:C400))
c Mass Specific Heat at Tref	kJ/(kgK)	=C403/C402
c Molar Specific Heat [Average Texh/Tref]	kJ/(kmolK)	=@IF(rnhv_mass<=0,Cpaverage((Tref_K),C451,PROPERTIES!$O$37:$W$37,PROPERTIES!$X$37:$AF$37),Cpaverage((Tref_K),C451,PROPERTIES!$O$64:$W$64,PROPERTIES!$X$64:$AF$64))
c Mass Specific Heat  [Average Texh/Tref]	kJ/(kgK)	=C405/C402
c Exhaust to Fuel Mass Ratio	kg/kg	=C367*C402/C98
c Air to Fuel Mass Ratio	kg/kg	=C362*$L$98/C98
c Excess Air to Fuel Mass Ratio	kg/kg	=C380*$L$98/C98

c =============================================================================      
c Exhaust Calculated Properties
c =============================================================================      
c Exhaust Molar Mass ,kg/kmol
      if(rnhv .le. 0.) then 
         rmm_ex = rmm 
      else 
         rmm_ex = calc_molarmass(mix_exhaust)
      end if 
c Exhaust Molar Specific Heat at 15C,kJ/(kmolK)
      excp  = calc_cpbar(mix_exhaust, tref)
c Exhaust Mass Specific Heat at 15C,kJ/(kgK)
      if (rmm_ex .ne. 0.)  excp_m = excp/rmm_ex 
c Exhaust to Fuel Mass Ratio,kg/kg
      exfmr =exwet*rmm_ex/rmm   !  ??? was rmm_noair
c Air to Fuel Mass Ratio,kg/kg
      tafmr = rmamf*prop(ipmmass,jpair)/rmm_noair
c Excess Air to Fuel Mass Ratio,kg/kg
      rexcessair=excessair*prop(ipmmass,jpair)/rmm_noair
c =============================================================================      
      
c Stoichiometric Exhaust to Fuel Mass Ratio, kg/kg
c      sexfmr = exfmr-rexcessair
      
      
c Stoichiometric Exhaust Molar Mass,kg/kmol
c      sexmm = (rfh2o*prop(ipmmass,jph2o)+
c     &stoic_air*calc_byprop(gcair,ipmmass,ipinert)+
c     &rfco2*prop(ipmmass,jpco2)+rfso2*prop(ipmmass,jpso2))
c     &/(rfh2o+stoic_air*finert+rfco2+rfso2)
c Stoichiometric Exhaust Mass Specific Heat,kJ/(kgK)
c      sexcp = (rfh2o*calc_cp(jph2o,tref)+
c     &stoic_air*calc_cpbar(gcair,tref,ipinert)+
c     &rfco2*calc_cp(jpco2,tref)+rfso2*calc_cp(jpso2,tref))
c     &/(rfh2o+stoic_air*finert+rfco2+rfso2)/sexmm

c =============================================================================      
c END Combustion 1
c =============================================================================      

c =============================================================================      
c --- COMBUSTION CALCULATIONS 1a -- NO EXCESS AIR
c =============================================================================      

c stoichiometric mole air required / mole fuel
c      stoic_air=stoic_o2/(1.-finert)
c Flame Excess Air for LFL or MOC,% 
      xx_fealfl   = 0.
c mole O2 supplied / mole fuel
      xx_rfsupo2    =DMAX1(rmamf_moc,stoic_air)*(1.-finert) + mix(jpo2)
c mole wet exhaust / mole fuel

c      aaaa1 = calc_byprop(mix, ipstoicp, ipcalc)  
c      aaaa2 = DMAX1(rmamf_moc,stoic_air)/(1.+xx_fealfl)*(finert+xx_fealfl)  
c      aaaa3 =  DMAX1(rmamf_moc,stoic_air)  * gcair(jph2)
      xx_exwet    = calc_byprop(mix, ipstoicp, ipcalc)  
     &        + DMAX1(rmamf_moc,stoic_air)/(1.+xx_fealfl)*(finert+xx_fealfl)  
     &        + DMAX1(rmamf_moc,stoic_air)  * gcair(jph2)
c mole dry exhaust / mole fuel
      xx_exdry    =xx_exwet
     &        - calc_byprop(mix, ipstoich2o, ipcalc)
     &        - DMAX1(rmamf_moc,stoic_air)  * gcair(jph2)
c mole H2O / mole fuel
      xx_rfh2o    =xx_exwet-xx_exdry  + DMAX1(rmamf_moc,stoic_air)*gcair(jph2)
c      aaaaa1 = xx_exwet-xx_exdry
c      aaaaa2 = MAX(rmamf_moc,stoic_air)
c      aaaaa3 = gcair(jph2)
c      aaaaa4 = aaaaa1 + aaaaa2 * aaaaa3
c mole O2 / mole fuel
      xx_rfo2     =xx_rfsupo2-stoic_o2
c mole Ar / mole fuel
      xx_rfar     =mix(jpar) + DMAX1(rmamf_moc,stoic_air)*gcair(jpar)
c N2	mole exhaust / mole fuel	
      xx_rfn2     = mix(jpn2) + DMAX1(rmamf_moc,stoic_air)*gcair(jpn2)
c He	mole exhaust / mole fuel	
      xx_rfhe     = mix(jphe) + DMAX1(rmamf_moc,stoic_air)*gcair(jphe)
c Ne	mole exhaust / mole fuel	
      xx_rfne     = mix(jpne) + DMAX1(rmamf_moc,stoic_air)*gcair(jpne)
c Kr	mole exhaust / mole fuel   
      xx_rfkr     = mix(jpkr) + DMAX1(rmamf_moc,stoic_air)*gcair(jpkr)
c N2O	mole exhaust / mole fuel	
      xx_rfn2o    = mix(jpn2o) + DMAX1(rmamf_moc,stoic_air)*gcair(jpn2o)
c Xe	mole exhaust / mole fuel	
      xx_rfxe    = mix(jpxe) + DMAX1(rmamf_moc,stoic_air)*gcair(jpxe)
c mole CO2 / mole fuel
      xx_rfco2    = calc_byprop(mix, ipstoicc, ipcalc) 
     &      + DMAX1(rmamf_moc,stoic_air)*(gcair(jpco2)+gcair(jpch4)+gcair(jpco))
c mole SO2 / mole fuel
      xx_rfso2    = mix(jph2s)
c mole Excess Air / mole fuel
      xx_excessair=xx_rfo2/(1.-finert)
c -------------------------------------------------------------          
      
      
c--- initialize exhaust mixture
      ier = initialize(xx_mix_exhaust,0.0D0)
      xx_mix_exhaust(jpair) = 0
      xx_mix_exhaust(jph2o) = xx_rfh2o/ xx_exwet
      xx_mix_exhaust(jpo2)  = xx_rfo2/ xx_exwet
      xx_mix_exhaust(jpar)  = xx_rfar/ xx_exwet
      xx_mix_exhaust(jpn2)  = xx_rfn2/ xx_exwet
      xx_mix_exhaust(jphe)  = xx_rfhe/ xx_exwet
      xx_mix_exhaust(jpne)  = xx_rfne/ xx_exwet
      xx_mix_exhaust(jpkr)  = xx_rfkr/ xx_exwet
      xx_mix_exhaust(jpn2o) = xx_rfn2o/ xx_exwet
      xx_mix_exhaust(jpxe)  = xx_rfxe/ xx_exwet
      xx_mix_exhaust(jpco2) = xx_rfco2/ xx_exwet
      xx_mix_exhaust(jpso2) = xx_rfso2/ xx_exwet

      
c      do j=1,mxcomp
c         r = xx_mix_exhaust(j)
c      end do
      
      
      xx_flow_mol = xx_exwet * totalflaringrate_v / spv
      rmm_xx = calc_molarmass(xx_mix_exhaust )       ! kg/kmole, no excess air
c      cpmol_xx  = calc_byprop(xx_mix_exhaust, ipcpmole, ipcalc)
c      if(rmm_xx .gt.0)  cpmass_xx  = cpmol_xx/ rmm_xx

c Exhaust Molar Specific Heat at 15C,kJ/(kmolK)
      xx_excp  = calc_cpbar(xx_mix_exhaust, tref)
c Exhaust Mass Specific Heat at 15C,kJ/(kgK)
      if (rmm_xx .ne. 0.)  xx_excp_m = xx_excp/rmm_xx 
c Exhaust to Fuel Mass Ratio,kg/kg
      xx_exfmr=xx_exwet*rmm_xx/rmm_noair
c Air to Fuel Mass Ratio,kg/kg
      xx_tafmr = stoic_air*prop(ipmmass,jpair)/rmm_noair
c Excess Air to Fuel Mass Ratio,kg/kg
      xx_rexcessair=xx_excessair*prop(ipmmass,jpair)/rmm_noair

c =============================================================================      
c END Combustion 1a
c =============================================================================      

c =============================================================================      
c TFLARED
c =============================================================================      
c --- Temperature of gas being combusted with Assist
c =============================================================================      
c Flared Gas Enthalpy, kJ/s      
      hf_raw = raw_m/rmmraw   * 
     &         (calc_hf1mix(gcrawIN, tgas) - calc_hf1mix(gcrawIN, tref))  
      hf_lift= 0.
      hf_fuel= 0.
      hf_stm = 0.
      hf_air = 0.
      hf_hpfuel = 0.
      if(MLIFTGAS .eq. 1) then
         hf_lift= lift_m/rmmlift * 
     &         (calc_hf1mix(gclift, tgas) -calc_hf1mix(gclift, tref))
      end if 
      if(MFUELGAS .eq. 1) then
         hf_fuel= fuel_m/rmmfuel * 
     &         (calc_hf1mix(gcfuel, tgas) -calc_hf1mix(gcfuel, tref)) 
      end if 
      if( MFASSIST .eq. 1 ) then 
      if( MFASSISTSTEAM .eq. 1 ) then 
         hf_stm = stm_m/rmmstm   * 
     &         (calc_hf1mix(gcstm, tstm) -calc_hf1mix(gcstm, tref)) 
      end if          
      if( MFASSISTAIR.eq. 1 ) then 
         hf_air = air_adjusted_m/rmmair   * 
     &         (calc_hf1mix(gcair, tamb) -calc_hf1mix(gcair, tref)) 
      end if          
      if( MFASSISTHPFUEL .eq. 1 ) then 
         hf_hpfuel = hpfuel_m/rmmfuel   * 
     &         (calc_hf1mix(gcfuel, tgas) -calc_hf1mix(gcfuel, tref)) 
      end if          
      end if 
      hf_mix = hf_raw + hf_lift + hf_fuel + hf_stm + hf_air + hf_hpfuel
c Estimated Temperature of Flared Gas, K
      tmix = hf_mix/ (cpmass_mix*totalflaringrate_m) + tref
c      rmdotcp_raw = raw_m *cpmass_raw *tgas
c      rmdotcp_lift= lift_m*cpmass_lift*tgas
c      rmdotcp_fuel= fuel_m*cpmass_fuel*tgas
c      rmdotcp_stm = stm_m *(cpmol_stm/rmmstm)  *tstm 
c      rmdotcp_air = air_m *(cpmol_air/rmmair)  *tamb 
c      rmdotcp_mix = rmdotcp_raw + rmdotcp_lift + 
c     &              rmdotcp_fuel +rmdotcp_stm +rmdotcp_air
c Estimated Temperature of Flared Gas, K
c      tmix  = rmdotcp_mix/totalflaringrate_m/cpmass_mix 
c goal enthalpy
c      egoal = rmdotcp_mix/totalflaringrate_m*rmm - cpmol_mix*tref

c goal enthalpy
      egoal_mix = hf_mix/totalflaringrate_m*rmm_netair 
c Temperature of Flared Gas, K
      extemp   = calc_flametemp(tmix, mix, egoal_mix)  
c      extemp   = MAX(extemp, tamb+rk0)
      tflared = extemp   ! check : this should be same as tgas

c mixed flared gas Density at release temperature and site pressure,kg/m
c NOTE: THIS density of flare: is calculated with the total stack top flow
c including ALL air assist
      den_flare   =plocal*rmm/(ru*tflared)
      if(mflameout.ge.1 .or. isFlammable.eq.0 ) then 
        den_mix     =plocal*rmm_netair/(ru*tflared)
      else
        den_mix     =plocal*rmm_mixair/(ru*tflared)
      end if 
c =============================================================================      

      
c =============================================================================      
c --- combustion efficiency  
c --- Exit Velocity
c =============================================================================      
c Flare Tip Momentum, m/s.kg/s
c Steam Flare Assist Orifice(s) Momentum, m/s.kg/s
c Air Flare Assist Orifice(s) Momentum, m/s.kg/s
c Effective Release Exit Velocity, m/s
c flared gas exit velocity, m/s
c      ue = (ueflare*mix0_m + uestm*stm_m + ueair*air_m  + uehpfuel*hpfuel_m)/
c     &            totalflaringrate_m
      
      total_m = (mix0_m + stm_m + air_m + hpfuel_m)
      areaStack = dia_f*dia_f
      if(mdlift .eq. 1) then
         areaStack = areaStack + dlift*dlift
      end if 
      if(mdfuel .eq. 1) then
         areaStack = areaStack + dfuel*dfuel
      end if 
      ue = total_m/den_mix * (4.D0/(pi*areaStack))
      call sonicVelocity(cpmol_mix, rmm_mixair, tflared, z_mix_tgas, c_flare_tflared)
c      c_flare_tflared   = sqrt((cpmol_mix/(cpmol_mix-ru))*
c     &            ru/rmm_mixair*tflared*1000.*z_mix_tgas)
      ue = min(ue, c_flare_tflared)
      if(mgroundflare.eq.1)then
        ue = c_flare_tflared
      end if 
      dia_eff = sqrt( total_m*(4/(pi*ue*den_mix)) )  ! Equivalent diameter
      
      if(mgroundflare.eq.1) then
        dia_f = dia_eff
        dia_unob = dia_eff
      end if 
      
      if(debug .gt. 0 ) then
      write(iolog,*) 'total_m=',total_m
      write(iolog,*) 'ue=',ue
      write(iolog,*) 'den_mix=',den_mix
      write(iolog,*) 'dia_eff=',dia_eff
      end if 

c Maximum Possible Conversion Efficiency 	% (out/in)
      eff_max = max(1.-uofaa/(rnhv_mass)**3.,0.)
c  Heating Value for 98% Conversion Efficiency at Maximum Wind Speed	MJ/kg at 15C ------------------- (not used)
      rnhv_m_98 = (uofaa/(1.D0-effminERCB)*EXP(MIN(700.,uofab*wsST_max/(grav*ue*dia_eff))))**(rk13)
c Total Flaring Rate for 98% Conversion Efficiency at Maximum Wind Speed, 10m/d (15C, 101.325 kPa) - (not used)
      if(rnhv_mass.le.0.) then 
         rate_98 = 0.D0
      else 
         rate_98 = pi*0.25D0/grav * dia_unob/
     &               ( log((1.D0-effminERCB)*rnhv_mass**3.D0/uofaa)/uofab/wsST_max)**3.D0  
     &               *3600.D0*24.D0/1000.D0
      end if
c Diameter to give 98% at Maximum Wind Speed	m	0.0272  --------------------------------------------- (not used)
      dia_98  = 4.D0*(rate_98*1000.D0/24.D0/3600.D0)/(pi*wsST_max)
      
c Minimum Wind Speed Conversion Efficiency	% (out/in)
      if(rnhv_mass.le.0.) then 
         carb_wsmin  = 0.D0
      else 
         carb_wsmin = carbeff(rnhv_mass, wsST_min, dia_eff, ue  )
      end if      
      if(mgroundflare.eq.1) then
        carb_wsmin = eff_max
      end if 
      
c Modelled Conversion efficiency (CONV fraction) % (out/in)
      carb  = carbeff(rnhv_mass, wsST, dia_eff, ue  )    ! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< Carb or CONV or CONVERSION EFFICIENCY
      conv  = carb
      if(mgroundflare.eq.1) then
        carb = eff_max
        conv = eff_max
      end if 
      
c Maximum Wind Speed Conversion Efficiency	% (out/in)	
      if(rnhv_mass.le.0.) then 
         carb_wsmax  = 0.D0
      else 
         carb_wsmax = carbeff(rnhv_mass, wsST_max, dia_eff, ue  )
      end if    
      if(mgroundflare.eq.1) then
        carb_wsmax = eff_max
      end if 
      
c Vj/Ua - Jet to Air velocity ratio 
      Vj_Ua = min(110.,ue/wsST)

c R - Jet to Air momentum flux ratio
      rjet  = den_mix/den_air*Vj_Ua*Vj_Ua

c Fraction of stripped flared gas that is cold and unconverted (fs)
      fs      = min(0.9,max(0.1,0.5-0.0869*log(rjet)))
c Fraction of unconverted flared gas stream that is stripped and cold      
      fs_cold = (1.D0-carb)*fs
      fs_hot  = (1.D0-carb)*(1.D0-fs)

c  MSTRIP = 3 : H2S (all H2S is cold)
c  MSTRIP = 5 : OU  (all H2S is cold)
      if(MSTRIP.eq.3 .or. MSTRIP.eq.5) then
         fs = 1.0
      end if 
c Modelled Combustion Efficiency of combusted portion (COMB fraction)
      comb  = carb/(carb + fs_cold + fs_hot) 
c Modelled Net Conversion Efficiency (NET fraction)
      comb_net = carb + fs_hot
      
      if(debug .gt. 0 ) then
      write(iolog,*) 'conv eff=',carb
      write(iolog,*) 'comb eff=',comb
      write(iolog,*) 'cold fraction,fs=',fs
      end if 
c =============================================================================      

c Exhaust flow at Pamb and Tamb,m/s
      exrate_amb_v = totalflaringrate_v*exwet*pref/plocal*
     &(tgas/tref)*(tamb/tgas)
     &*(comb)
c     &*(carb+(1.-carb)*(1.-fs))
c air required at Pamb and Tamb,m/s      
      air_amb_v = exrate_amb_v *rmamf/exwet

c *********************************************************************
c *********************************************************************
c if Air Assist
c then use the calculated air_amb_v and reduce the air assist by
c this amount
c
c Start Again
      if(MFASSIST.eq.1 .and. MFASSISTAIR.eq.1) then
         if(fWAITFORVALUE) then
            fWAITFORVALUE = .FALSE.
            goto 1000
         end if 
      end if 
c *********************************************************************
c *********************************************************************
      
c now use below 'Fuel Molar Average Specific Heat'      
c Fuel Molar Specific Heat at 15C,kJ/(kmolK)
      fcp  = calc_byprop(mix, ipcpmole, ipcalc) 
c Fuel Mass Average Specific Heat at 15C kJ/(kgK)
      fcp_m = fcp/rmm_netair
      
c Molar Specific Heat  [Average Tfuel/Tref]	kJ/(kmolK)
c Fuel Molar Average Specific Heat at Fuel Temperature,kJ/(kmolK)
      temp_hf = tflared
      temp_h0 = tref
      cpfuel_ave  = fcp_m
      if(temp_hf .ne. temp_h0 ) then                                    
          cpfuel_ave = (calc_hfbar(mix,temp_hf)-calc_hfbar(mix,temp_h0))/
     &    (temp_hf-temp_h0)
      end if
c Mass Specific Heat  [Average Tfuel/Tref]	kJ/(kgK)
c Fuel Mass Average Specific Heat at Fuel Temperature,kJ/(kgK)
      cpmass_fuel_ave = cpfuel_ave/rmm_netair
c =============================================================================      
c END Combustion efficiency
c =============================================================================      
      
      

c =============================================================================      
c FM Calculation  
c     (requires combustion efficiency results)
c =============================================================================      
c Stripped Gas Momentum Flux (loss)	m4/s2
      FM_streams = FM_flare + FM_lift + FM_fuel + FM_stm + FM_air + FM_hpfuel
      FM_strip   = (FM_streams)*fs_cold
c Total momemtum      
      FM = FM_streams - FM_strip
c =============================================================================      
      
c =============================================================================      
c Emissions at combustion efficiency      
c =============================================================================      
      e_co2 = (e_co2_process+e_co2_combustion *carb) + 
     &(1.-carb)*mix(jpch4)*
     &totalflaringrate_ve3m3 *prop(ipmmass,jpch4)/spv*25.  ! t/d
      e_h2s = fs_cold*e_h2s_m                   ! g/s
      e_so2 = e_so2_m*(carb + fs_hot)           ! g/s
c      e_ou  = totalflaringrate_v*
c     &         mix(jph2s)*(1.-carb)*(mix(jph2s)/odourppb*1.e9)*fs ! OU/s
      e_ou  = totalflaringrate_v*
     &         fs_cold*(mix(jph2s)/odourppb*1.e9) ! OU/s

      if(debug .gt. 0 ) then
      write(iolog,*) 'e_co2=',e_co2,' t/d'
      write(iolog,*) 'e_so2=',e_so2, e_so2_m,' g/s'
      write(iolog,*) 'e_h2s=',e_h2s,' g/s as unoxydized raw H2S stripped'
      write(iolog,*) 'e_ou=',e_ou,' OU/s stripped'
      end if 

c =============================================================================      
c --- Combustion Products
c =============================================================================      
c was here      


c =============================================================================      
c --- STRIPPED GAS STREAM
c =============================================================================      
c --- Stripped Gas Stream with Assumed Temperature Pseudo Parameters
c Flared Gas, uncombusted mass flow rate	kg/s	
c      = totalflaringrate_m
c Stripped Mass (cold) Fraction		
c      = fs_cold
c Stripped Mass (cold) flow rate	kg/s	
c uses an assumed temperature that is constant (ie, not a function
c of wind speed nor stripped mass flow      
      stripped_m = totalflaringrate_m * fs_cold
c Air flow for Stripped Mass at LFL	kg/s	
      stripped_air = stripped_m * tafmr
c Stripped Mass plus air flow rate	kg/s	
      stripped_tot = stripped_m + stripped_air 
c Auto-ignition temperature of combustibles, K      
      if(totalflaringrate_m .le. 0.) then
         tautoig = 10000.
      else    
         tautoig = calc_autoig(mix)
      end if 
c Assumed temperature of stripped mass	K	
c Assumed temperature of stripped mass, K      
      tstrip = 0.5*(tautoig + tgas)

c stripped mass cp,kJ/kmol/K
c Specific heat of stripped mass at T, kJ/(kgK)
c stripped mass cp,kJ/kg/K
      hf_sm = calc_hf1mix(mix, tstrip) - 
     &            calc_hf1mix(mix, tamb)
      smcp     = cpmass_mix
      if( hf_sm.gt. 0. .and. tstrip .ne. tamb) then
          smcp    = hf_sm/(tstrip-tamb)
      end if               
      smcp    = smcp/rmm_noair
c Specific heat of air at T, kJ/(kgK)
c      smaircp= calc_cp(jpair, tstrip)/rmmair    
      hf_smair =  calc_hf(jpair, tstrip) - 
     &            calc_hf(jpair, tamb)
      smaircp= cpmass_mix
      if( hf_smair.gt. 0. .and. tstrip.ne.tamb) then
          smaircp    = hf_smair/(tstrip-tamb)
      end if               
      smaircp    = smaircp/rmmair
c Specific heat of stripped mass and air at T, kJ/(kgK)
      smtotcp=(stripped_m*smcp+stripped_air*smaircp)/stripped_tot
c Molar mass at air and fuel,kg/kmol
      rmmsm   = (rmm_noair+rmmair*rmamf)/(1.+rmamf)
c Buoyancy Flux if unconverted and air is heated up,m4/s3
      FB_stripped   =grav*ru/(pi*plocal*rmmair)*
     &     (smtotcp/cpmass_air_ave_tamb *
     &     (tstrip-tamb)-(1.-rmmair/rmmsm)*tamb)
     &     *stripped_tot
c Buoyancy Flux of heated stripped gas	m4/s3	
      FB_strip2  =grav*ru/(pi*plocal*rmmair)*
     &     (smcp/cpmass_air_ave_tamb*(tstrip-tamb))*stripped_m
c Buoyancy Flux of heated air	m4/s3	
      FB_strip3  =grav*ru/(pi*plocal*rmmair)*
     &     (smaircp/cpmass_air_ave_tamb*(tstrip-tamb))*stripped_air
c Buoyancy Flux of total stripped mixture (check) (FB_stripped)	m4/s3	
      htot =grav*ru/(pi*plocal*rmmair)*
     &     (smtotcp/cpmass_air_ave_tamb*(tstrip-tamb))*stripped_tot
c Stripped gas and air heat     , kW
      qstrip =htot*pi*plocal*cpmass_air_ave_tamb*rmmair/grav/ru
c Stripped Mixture Momentum Flux (FM_stripped)	m4/s	
      FM_stripped = FM_strip
c Flow Rate, at reference conditions	Sm/s	
      flow_stripped_ref_v = stripped_tot/rmmsm* spv
c =============================================================================      
c END Stripped gas 
c =============================================================================      
      
      
      
      
c =============================================================================      
c TFLAME
c =============================================================================      

c --- Adjusted Heating Value
c Complete combustion heat release with no losses,MJ/m
      h1 = rnhv
c Heat loss due to incomplete combustion, MJ/m
      h2 = -(1.-comb)*h1
c Fuel or Mix sensible heat,MJ/m
      h3 = cpfuel_ave*(tflared-tref)/spv/1000.
c Mix kinetic energy	MJ/m
      h4 = 0.5D0*(ue+wsST)**2.D0 * (totalflaringrate_m/totalflaringrate_v)*1.D-6      
c Air sensible heat,MJ/m
      h5 = (rmamf) * cpmol_air_ave_tamb *(tamb-tref)/spv/1000.D0
c Heat transferred to stripped fraction,MJ/m
      h6 =-qstrip/(totalflaringrate_v*1000.D0) 
c Radiation heat losses from combustion heat released (25% assumed),MJ/m
      h7 =-radloss*(h1+h2)
c Total heat to plume (sum),MJ/m
      heatp=max(0.d0,h1 + h2 + h3 + h4 + h5 + h6 + h7)

c Temperature with losses using air properties,K
      t0 =comb*heatp*spv/rmm*1000./(exwet*rmmair/rmm)/cpmass_air+(tref)
c Complete combustion heat release with no losses,K
      t1 =(rnhv_mass)*1000./exfmr/excp_m
c Heat loss due to incomplete combustion, K
      t2 =-t1*(1.-comb)
c Fuel sensible heat,K
      t3 =fcp_m/excp_m*(tflared-tref)/exfmr
c Mix kinetic energy, K
      t4 = 0.5*(ue+wsST)**2.D0/1000/ (excp_m*exfmr)
c Air sensible heat,K
      t5 =cpmass_air *(tamb-tref)*tafmr/ (excp_m*exfmr)
c Heat transferred to stripped fraction, K
      t6 = t1*h6/h1
c Radiation heat losses from combustion heat released (25% assumed),K
      t7 =-radloss*(t1+t2+t3+t4+t5+t6)
c Reference temperature,K
      t8 =tref
c Temperature with constant specific heat at 15C sum,K
      tguess   =MAX(t1+t2+t3+t4+t5+t6+t7+t8,tamb) 

c enthalpy goal, kJ/kmol
      egoal_ex = heatp * 1000. * spv / exwet
c Estimated Flame Temperature with specific heat function of temperature,K
c Estimated Flame Temperature with Cp function of temperature,K
      extemp   = calc_flametemp(tguess,mix_exhaust, egoal_ex)
      if(heatp .lt. 0) then
          extemp   = tamb
      end if 
      extemp = MAX(extemp, tamb)
c tflame, flame temperature in K      
      tflame = extemp
      
c specific heat of flared gas, kJ/kmol/K
c      cpflared = calc_cpmix(tflame, mix_exhaust) 
c flared gas average cp,kJ/kmol/K
c      cpflared_ave = calc_cp_ave_mix(tref, tflame, mix_exhaust)
c enthalpy of flared gas, kJ/kmol
c      hf_flared = calc_hf1mix(mix_exhaust, tflame) - 
c     &            calc_hf1mix(mix_exhaust, tref)
c flared gas average cp,kJ/kmol/K
c      cpflared_ave = cpflared
c      if( hf_flared .gt. 0.) then
c          cpflared_ave = hf_flared/(tflame-tref)
c      end if               

c flared gas exhaust flow, kmol/s
      exflow_mol = exflow_v /spv

c heat to plume, kW
      heat_p  = exflow_mol * egoal_ex   

c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 
c     with excess air 
c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 
c Exhaust Molar Average Specific Heat at Estimated Temperature,kJ/(kmolK)
c      excp_extemp = (calc_hf1mix(mix_exhaust, tflame)-
c     &calc_hf1mix(mix_exhaust, tref))/
c     &(tflame-tref)  ! calc_cpbar(mix_exhaust, tflame) 
c Exhaust Mass Average Specific Heat at Estimated Temperature,kJ/(kgK)
c      excpmass_texh_ave=excp_extemp/rmm_ex

c Molar Specific Heat [Average Texh/Tref]	kJ/(kmolK)	
      if(rnhv .le. 0.) then 
         excp_texh_ave  = calc_cp_ave_mix(tref, tflame, mix) 
      else 
         excp_texh_ave  = calc_cp_ave_mix(tref, tflame, mix_exhaust) 
      end if 
c Mass Specific Heat  [Average Texh/Tref]	kJ/(kgK)	
      excpmass_texh_ave = excp_texh_ave / rmm_ex 
c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 
c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 


c =============================================================================      
c xx_TFLAME  (no Excess air)
c =============================================================================      

c --- Adjusted Heating Value
c Complete combustion heat release with no losses,MJ/m
      h1_xx = rnhv
c Heat loss due to incomplete combustion, MJ/m
      h2_xx = -(1.-comb)*h1_xx
c Fuel or Mix sensible heat,MJ/m
      h3_xx = cpfuel_ave*(tflared-tref)/spv/1000.
c Mix kinetic energy	MJ/m
      h4_xx = 0.5D0*(ue+wsST)**2.D0 * (totalflaringrate_m/totalflaringrate_v)*1.D-6      
c Air sensible heat,MJ/m
      h5_xx = (stoic_air) * cpmol_air_ave_tamb *(tamb-tref)/spv/1000.D0
c Heat transferred to stripped fraction,MJ/m
      h6_xx =-qstrip/(totalflaringrate_v*1000.D0) 
c Radiation heat losses from combustion heat released (25% assumed),MJ/m
      h7_xx =-radloss*(h1_xx+h2_xx)
c Total heat to plume (sum),MJ/m
      heatp_xx=max(0.d0,h1_xx + h2_xx + h3_xx + h4_xx + h5_xx + h6_xx + h7_xx)

c Temperature with losses using air properties,K
      t0_xx =comb*heatp_xx*spv/rmm*1000./(xx_exwet*rmmair/rmm)/cpmass_air+(tref)
c Complete combustion heat release with no losses,K
      t1_xx =(rnhv_mass)*1000./xx_exfmr/xx_excp_m
c Heat loss due to incomplete combustion, K
      t2_xx =-t1_xx*(1.-comb)
c Fuel sensible heat,K
      t3_xx =fcp_m/xx_excp_m*(tflared-tref)/xx_exfmr
c Mix kinetic energy, K
      t4_xx = 0.5*(ue+wsST)**2.D0/1000.D0/ (xx_excp_m*xx_exfmr)
c Air sensible heat,K
      t5_xx =cpmass_air *(tamb-tref)*tafmr/ (xx_excp_m*xx_exfmr)
c Heat transferred to stripped fraction, K
      t6_xx = t1_xx*h6_xx/h1_xx
c Radiation heat losses from combustion heat released (25% assumed),K
      t7_xx =-radloss*(t1_xx+t2_xx+t3_xx+t4_xx+t5_xx+t6_xx)
c Reference temperature,K
      t8_xx =tref
c Temperature with constant specific heat at 15C sum,K
      tguess_xx   =MAX(t1_xx+t2_xx+t3_xx+t4_xx+t5_xx+t6_xx+t7_xx+t8_xx,tamb) 

c enthalpy goal, kJ/kmol
      egoal_xx = heatp_xx * 1000. * spv / xx_exwet
c Estimated Flame Temperature with specific heat function of temperature,K
c Estimated Flame Temperature with Cp function of temperature,K
      xx_extemp   = calc_flametemp(tguess_xx,xx_mix_exhaust, egoal_xx)
      if(heatp_xx .lt. 0) then
          xx_extemp   = tamb
      end if 
      xx_extemp = MAX(xx_extemp, tamb)
c tflame, flame temperature in K      
      xx_tflame = xx_extemp
      
c specific heat of flared gas, kJ/kmol/K
      xx_cpflared     = calc_cpmix(xx_tflame, xx_mix_exhaust ) 
c flared gas average cp,kJ/kmol/K
      xx_cpflared_ave = calc_cp_ave_mix(xx_mix_exhaust, xx_tflame, tref)
c flared gas exhaust flow, kmol/s
      xx_exflow_mol = xx_exflow_v /spv
c heat to plume, kW
      xx_heat_p  = xx_exflow_mol * egoal_xx   

c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 
c     no excess air 
c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 

c Molar Specific Heat [Average Texh/Tref]	kJ/(kmolK)	
      if(rnhv .le. 0.) then 
         xx_excp_texh_ave  = calc_cp_ave_mix(tref, xx_tflame, mix) 
      else 
         xx_excp_texh_ave  = calc_cp_ave_mix(tref, xx_tflame, xx_mix_exhaust) 
      end if 
c Mass Specific Heat  [Average Texh/Tref]	kJ/(kgK)	
      xx_excpmass_texh_ave = xx_excp_texh_ave / rmm_xx 

c [Average With & No Excess Air] Mass Specific Heat  [Average Texh/Tref]	kJ/(kgK)
      cpmass_xx_ave= cpmass_air_ave_tamb*((excpmass_texh_ave/cpmass_air_ave_tamb-1.D0)*exfmr/xx_exfmr+1.D0)
c [Average With & No Excess Air] Molar Specific Heat [Average Texh/Tref]	kJ/(kmolK)
      cpmol_xx_ave = cpmass_xx_ave * rmm_xx
c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 
c  -- -- --  -- -- --   -- -- --   -- -- --   -- -- --   -- -- -- 


c
c Exhaust Molar Specific Heat at Estimated Temperature, kJ/(kmolK)
c      excpest  = calc_cpbar(mix_exhaust, tflame)
c Exhaust Mass Specific Heat at Estimated Temperature,kJ/(kgK)
c      excpest_m = excpest/rmm_ex 
c Stoichiometric Exhaust Mass Specific Heat at Estimated Temperature, kJ/(kgK)
c      sexcp_test =cpmass_air_ave_tamb*
c     &             ((excpest_m/cpmass_air_ave_tamb-1.)*exfmr/sexfmr+1.)
c

      
c Stoichiometric Exhaust Mass Specific Heat,kJ/(kgK) at estimated Temperature  (C293)
c      sexcp_tex  = cpmass_air_ave_tamb*((excpmass_texh_ave/cpmass_air_ave_tamb-1.)*
c     &            exfmr/sexfmr + 1.)
c =============================================================================      
c END TFLAME
c =============================================================================      

      
      
      
c =============================================================================      
c --- BUOYANCY
c --- Buoyancy Flux at Site Conditions
c =============================================================================      

c Flared gas with assist less stripped gas rate unit factor	m4/(s3 K)
      fruf =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m
c Combustion heat release with losses,m4/s3
      hre1 = fruf * (rnhv_mass)*max((h1+h2+h4+h6+h7)/h1,0.D0)/cpmass_air_ave_tamb*1000.D0
c Mix kinetic energy	m4/s3
      hre2 = fruf * (0.5*(ue+wsST)**2.D0/1000.D0/ (cpmass_air_ave_tamb))
c Mix sensible heat relative to reference temperature,	m4/s3
      hre3 = fruf * cpmass_fuel_ave/cpmass_air_ave_tamb*(tflared-tref)
c Difference between molar mass of exhaust and air,	m4/s3
      hre4 = fruf * xx_exfmr*(rmmair/rmm_xx-1.D0)*tamb
c Difference between specific heat of exhaust and air,	m4/s3
      hre5 = fruf * -xx_exfmr*(cpmass_xx_ave/cpmass_air_ave_tamb-1.D0)*(tamb-tref)
c Difference between ambient and reference temperature,m4/s3
      hre6 = fruf * -(tamb-tref)
      
c --- "Buoyancy Flux based on Heat to Plume and Molar Mass (THESE ARE USED)",m4/s3
      FB   = hre1 + hre2 + hre3 + hre4 + hre5 + hre6
c --- use FB below at estimated temperature
c     Buoyancy Flux with specific heat at estimated temperatures sum      
      FB   = MAX(FB,0.)

      
c --- Buoyancy Flux, FB infinity (same as above), m4/s3
c      as in should be the same as FB_tref
c      exact solution, based on properties that change with temperature, Tamb varies with mPSEUDO
      FB_infinity = fruf * exfmr* Tamb* ((excpmass_texh_ave/cpmass_air_ave_tamb)*(tflame/Tamb-1.D0)+(rmmair/rmm_xx-1.D0))
     &                       + hre2

c - Buoyancy Flux-Heat Release Rate Constant (K3), m4/(s2J)
c      FB =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m*exfmr*
c     &         (excpmass_texh_ave/cpmass_air_ave_tamb*tflame-
c     &         Tamb*(1.-rmmair/rmm_ex+excpmass_texh_ave/cpmass_air_ave_tamb))
c     &         *(carb + (1.-carb)*(1.-fs))
c      FB  =MAX(FB,0.)

c --- Alternate Buoyancy Flux based on simple heat release rate to plume,	m4/s3
c     constant, independent of excess air and flame temperature		
      rk3      = grav*ru/
     &                (rmmair*pi*plocal*(cpmol_air/rmmair*1000.))
      FB_alt   = rk3*rnetheatreleased_c*carb*(1.-radloss)

c Buoyancy Flux at reference pressure (previously used),m4/s3
      FB_old   = 0.000037*rnetheatreleased_a*(effminERCB-radloss)
      FB_old  =max(FB_old,0.)   

c Steam Assisted buoyancy flux      
      FB_stm =grav*ru/(rmmair*pi*plocal)*stm_m*
     & (cpmass_stm_ave_tstm/cpmass_air_ave_tamb*tstm-
     & Tamb*(1.-rmmair/rmmstm+cpmass_stm_ave_tstm/cpmass_air_ave_tamb))
c Air Assisted buoyancy flux      
      FB_air =grav*ru/(rmmair*pi*plocal)*air_adjusted_m*
     & (cpmass_air_ave_tamb/cpmass_air_ave_tamb*tamb-
     & tamb*(1.-rmmair/rmmair+cpmass_air_ave_tamb /cpmass_air_ave_tamb))
c =============================================================================      
c END Buoyancy
c =============================================================================      


c =============================================================================      
c --- FLAMEOUT BUOYANCY
c --- Buoyancy Flux at Site Conditions
c =============================================================================      

c Flared gas with assist less stripped gas rate unit factor	m4/(s3 K)
      fruf_fo =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m
c Combustion heat release with losses,m4/s3
      hre1_fo = 0.d0
c Mix kinetic energy	m4/s3
      hre2_fo = fruf_fo * (0.5*(ue+wsST)**2.D0/1000.D0/ (cpmass_air_ave_tamb))
c Mix sensible heat relative to reference temperature,	m4/s3
      hre3_fo = fruf_fo * cpmass_fuel_ave/cpmass_air_ave_tamb*(tflared-tref)
c Difference between molar mass of exhaust and air,	m4/s3
      hre4_fo = fruf_fo * 1.D0*(rmmair/rmm_mixair-1.D0)*tamb
c Difference between specific heat of exhaust and air,	m4/s3
      hre5_fo = fruf_fo * -1.D0*(cpmass_fuel_ave/cpmass_air_ave_tamb-1.D0)*(tamb-tref)
c Difference between ambient and reference temperature,m4/s3
      hre6_fo = fruf_fo * -(tamb-tref)
      
c --- "Buoyancy Flux based on Heat to Plume and Molar Mass (THESE ARE USED)",m4/s3
      FB_fo   = hre1_fo + hre2_fo + hre3_fo + hre4_fo + hre5_fo + hre6_fo
c --- use FB below at estimated temperature
c     Buoyancy Flux with specific heat at estimated temperatures sum      
      FB_fo   = MAX(FB_fo,0.)
c =============================================================================      
c END Buoyancy
c =============================================================================      
      

c =============================================================================      
c Brzustowski Flare & Flame Parameters/Size/Geometry Model
c =============================================================================      
    
c Adjustment factor for minimum fuel gas,previous / current
c Sensible Heat Release Rate to Rise (ES) for Best Fit,cal/s
c      ES =FB/0.000037

c     Source Parameters for Ambient Wind Speed
      call brzflare(mix, dia_f, ue, wsST, den_mix, den_air, XL, zL, FA)

c Vertical velocity based on flared gas momentum,m/s
      uv = ue/exfmr
c Horizontal velocity based on average wind speed momentum,m/s
      uu = tafmr*wsST/exfmr
c Angle of plume from horizontal,degrees
      dangle = ATAN2(uv, uu)
      plumeangle =90.D0 - DEGREES(dangle)
c Angle of flame from Vertical,degrees
      flameangle =90.-DEGREES(ATAN2(ZL,XL))
c Total velocity,m/s
      us = max(sqrt(uv*uv + uu*uu),0.001)

c Exhaust flow at Pamb and Texh	m/s	
c see also: exrate_amb_v
      exrate_v   = exflow_v*(tflame/Tref)*(Pref/plocal)
c      exrate_v   = exrate_amb_v*tflame/tamb

c Air required at Pamb and Tamb, m/s
c      rair     =exrate_v*rmamf/exwet
c Diameter of plume,m
      dia2 =sqrt(4.*exrate_v/(pi*us))
c Estimated source temperature,K
c      = tflame
c Converted SO2,ppm wet
      conv_so2  = mix_exhaust(jpso2)*1000000.*comb_net
c Residual so2 or trs or H2S, if mixed in plume,ppm wet
      resid_h2s = mix_exhaust(jpso2)*1000000.*(1.-comb_net) 
c     &            *prop(ipmmass,jph2s)/prop(ipmmass,jpso2)
c Conversion In-efficiency,
      eta=(resid_h2s/(conv_so2+resid_h2s))
c Downwind distance from flare tip,m
c      xf = XL
c Elevation  above flare tip,m
      hf = max(hs+2.*min(dia_f*(ue/wsST-1.5),0.)+zL,0.)-hs

c Flame length from flare tip to flame tip	m
      flame_length = SQRT(XL*XL + ZL*ZL)      
c Flame volume (outer-inner cone)	m
      flame_volume = rk13*(dia2*dia2+dia_unob*dia2)*PI*0.25*flame_length
c Residence Time	s
      residence_time = flame_volume / exrate_v
      
      if(debug .gt. 0) then
         write(iolog,*) 'XL=',XL
         write(iolog,*) 'ZL=',ZL
         write(iolog,*) 'FA=',FA
         write(iolog,*) 'uu,uv=',uu,uv
         write(iolog,*) 'plume angle (deg)=',plumeangle
         write(iolog,*) 'flame angle (deg)=',plumeangle
         write(iolog,*) 'exhaust rate (m3/s)=',exrate_v
      end if 
      e_ou  = totalflaringrate_v*
     &         (1.-carb)*(mix(jph2s)/odourppb*1.e9)*fs ! OU/s
c =============================================================================      
c END Brzustowski related calcs
c =============================================================================      
 
c =============================================================================      
c START Bradley no wind flame model related calcs
c =============================================================================     
 
c Heat Release Rate	kW
      heat_released_nowind = rnetheatreleased_c*0.001
      if(mgroundflare.eq.1) then
        heat_released_nowind = heat_released_nowind/ float(max(nheads,1))
      endif 
      
c Assumed Flare Head Diameter	m
      dia_h = dia_f
      if(mgroundflare.eq.1) then
        dia_h = dia_eff
      end if 
      
c Dimensionless heat flux group, Q*	--
      heat_flux = heat_released_nowind/(cpmass_air_ave_tamb*tflared*den_flare
     &      *SQRT(grav)*dia_h**(5./2.))

c Flame Height (subsonic)	m
      z_flame_subsonic =dia_h*(3.4*(heat_flux)**(2./5.)-0.6)
c Flame Height (sonic)	m
      z_flame_sonic = dia_h*1.9*(heat_flux)**(2./5.)
c Flame Height	m
      z_flame_nowind = z_flame_subsonic
      if(heat_flux>100.d0) then
        z_flame_nowind = z_flame_sonic
      end if 
      
  
c =============================================================================      
c END Bradley no wind flame model related calcs
c =============================================================================  
  
  
c =============================================================================      
c --- FLAMEOUT 
c =============================================================================      
c     0: normal combustion
c     1: model uncombusted H2S 
c     2: model raw gas mix
c =============================================================================      
      if((MFLAMEOUT.eq.1) .or. (isFlammable.eq.0) ) then
         
c reassign  fuel gas ratio pseudo =1         
         xx_exfmr          = 1.0
         exfmr             = 1.0
c reassign cp to be flared gas mix kJ/(kg.K)         
         cpmass_xx_ave = cpmass_fuel_ave
c reassign exhaust mm to be mix 
         rmm_xx = rmm
c reassign tflame to be tginit
         tflame = tflared
         tstrip = tflared
c reassign cp of exhaust to be cp of the fuel
      excpmass_texh_ave = cpmass_fuel_ave


c Flared gas with assist less stripped gas rate unit factor	m4/(s3 K)
      fruf =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m
c Combustion heat release with losses,m4/s3
      hre1 = 0.D0
c Mix kinetic energy	m4/s3
      hre2 = fruf * (0.5*(ue+wsST)**2.D0/1000.D0/ (cpmass_air_ave_tamb))
c Mix sensible heat relative to reference temperature,	m4/s3
      hre3 = fruf * cpmass_fuel_ave/cpmass_air_ave_tamb*(tflared-tref)
c Difference between molar mass of exhaust and air,	m4/s3
      hre4 = fruf * xx_exfmr*(rmmair/rmm_xx-1.D0)*tamb
c Difference between specific heat of exhaust and air,	m4/s3
      hre5 = fruf * -xx_exfmr*(cpmass_xx_ave/cpmass_air_ave_tamb-1.D0)*(tamb-tref)
c Difference between ambient and reference temperature,m4/s3
      hre6 = fruf * -(tamb-tref)
      
c --- "Buoyancy Flux based on Heat to Plume and Molar Mass (THESE ARE USED)",m4/s3
      FB   = hre1 + hre2 + hre3 + hre4 + hre5 + hre6
c --- use FB below at estimated temperature
c     Buoyancy Flux with specific heat at estimated temperatures sum      
      FB   = MAX(FB,0.)

      
c --- Buoyancy Flux, FB infinity (same as above), m4/s3
c      as in should be the same as FB_tref
c      exact solution, based on properties that change with temperature, Tamb varies with mPSEUDO
      FB_infinity = fruf * exfmr* Tamb* ((excpmass_texh_ave/cpmass_air_ave_tamb)*(tflame/Tamb-1.d0)+(rmmair/rmm_xx-1.D0))
     &                       + hre2
c      =C516*(Ta_K)*1*(C357/$C$464*(C425/(Ta_K)-1)+($L$98/C98-1))+C518
c Total buoyancy         
c         FB =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m*exfmr*
c     &         (excpmass_texh_ave/cpmass_air_ave_tamb*tflame-
c     &         Tamb*(1.-rmmair/rmm_ex+excpmass_texh_ave/cpmass_air_ave_tamb))
c         if(FB.LT.0.) then
c            FB = 0.0
c         end if 
         
c Total momemtum      
         FM = FM_flare + FM_lift + FM_fuel + FM_stm + FM_air  + FM_hpfuel  - FM_strip
         
c makes H2S emission and Odour emissions same as 0% conversion 
         e_so2_m = e_h2s_m
         e_h2s = e_h2s_m
         e_ou = e_ou_0
c makes the stripped gas equal to full gas         
         FB_stripped = FB
         FM_strip = FM

c     Source Parameters for Ambient Wind Speed
         XL = 0. 
         zL = 0. 
         FA = 90.
         
      if(debug .gt. 0 ) then
      write(iolog,'(A82)') 'FM,FB,fm_strip,FB_stripped'
      write(iolog,'(15G15.4)') FM,FB,fm_strip,FB_stripped
      end if 
         
         
c =============================================================================      
c FlameOut for Raw Gas Mix
c This is used for modelling total gas and comparison to LFL
c =============================================================================      
      elseif(MFLAMEOUT.eq.2) then
c reassign  fuel gas ratio pseudo =1         
         xx_exfmr          = 1.0
         exfmr             = 1.0
c reassign cp to be flared gas mix kJ/(kg.K)         
         cpmass_xx_ave = cpmass_fuel_ave
c reassign exhaust mm to be mix 
         rmm_xx = rmm
c reassign tflame to be tginit
         tflame = tflared
         tstrip = tflared
c reassign cp of exhaust to be cp of the fuel
      excpmass_texh_ave = cpmass_fuel_ave


c Flared gas with assist less stripped gas rate unit factor	m4/(s3 K)
      fruf =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m
c Combustion heat release with losses,m4/s3
      hre1 = 0.D0
c Mix kinetic energy	m4/s3
      hre2 = fruf * (0.5*(ue+wsST)**2.D0/1000.D0/ (cpmass_air_ave_tamb))
c Mix sensible heat relative to reference temperature,	m4/s3
      hre3 = fruf * cpmass_fuel_ave/cpmass_air_ave_tamb*(tflared-tref)
c Difference between molar mass of exhaust and air,	m4/s3
      hre4 = fruf * xx_exfmr*(rmmair/rmm_xx-1.D0)*tamb
c Difference between specific heat of exhaust and air,	m4/s3
      hre5 = fruf * -xx_exfmr*(cpmass_xx_ave/cpmass_air_ave_tamb-1.D0)*(tamb-tref)
c Difference between ambient and reference temperature,m4/s3
      hre6 = fruf * -(tamb-tref)
      
c --- "Buoyancy Flux based on Heat to Plume and Molar Mass (THESE ARE USED)",m4/s3
      FB   = hre1 + hre2 + hre3 + hre4 + hre5 + hre6
c --- use FB below at estimated temperature
c     Buoyancy Flux with specific heat at estimated temperatures sum      
      FB   = MAX(FB,0.)

c Total momemtum      
         FM = FM_flare + FM_lift + FM_fuel + FM_stm + FM_air  + FM_hpfuel   + FM_strip
         
c        assumes MSTRIP.eq.0         
         e_so2_m = totalflaringrate_m *1000. ! g/s

c     Source Parameters for Ambient Wind Speed
         XL = 0. 
         zL = 0. 
         FA = 90.
         
      end if 
c =============================================================================      
c END FlameOut
c =============================================================================      
          
c =============================================================================      
c --- PSEUDO-STACK PARAMETERS
c =============================================================================      
c
c  MSTRIP = 0 : SO2 100%
c  MSTRIP = 1 : H2S 
c  MSTRIP = 2 : SO2 actual
c
c =============================================================================      
      if( MSTRIP .eq. 0 .or. MSTRIP.eq.2) then
c set output emissions
         if( MSTRIP.eq.0 ) then
            qemit = e_so2_m
c calculate actual SO2 emitted using combustion efficiency
         else if( MSTRIP.eq.2 ) then
            e_so2_ma= e_so2_m *(carb)
            qemit   = e_so2_ma
         end if 

c Three equations
c Pseudo Stack Parameters --- AERflare Method
c Conserve FB, FM and Volumetric Flow         

c ---------------------------------------------------------------
c Reduced Parameters
c Reduced exhaust temperature, Ts'	--	
         tsp     = tflame/Tref
c Reduced ambient temperature, Ta'	--	
         tap     = Tamb/Tref
c Reduced ambient pressure, Pa'	--	
         pap     = plocal/Pref
c Dimensionless buoyancy flux to volume flow ratio term, alpha	--	
         IF(mflameout .eq. 0 .and. isFlammable.eq.1 ) then 
            alpha   = pi*pap*FB/(grav*exflow_v)
         else
            alpha   = pi*pap*FB/(grav*totalflaringrate_v)
         end if 
c Buoyancy flux to Momentum Flux Ratio, beta	--	
         beta    = FB/FM
c ---------------------------------------------------------------
          
c Pseudo---Stack Height, Hs (m)          
         hs_x = max(hs+2.*min(dia_f*(ue/wsST-1.5),0.),0.) 
         if((mFlameout .eq. 0) .and. (isFlammable .eq. 1)) then
            hs_x = hs_x + zL
         end if 
         if(mgroundflare.eq.1) then
           hs_x = Hs+ z_flame_nowind
           if(mflameout.ge.1 .or. isFlammable.eq.0 ) then
             hs_x = hs
           end if 
         end if 
c Pseudo---Stack gas Exit Temperture, Te (K)
         extemp = (alpha+tap)*(Tref)
c Pseudo---Diameter, De (m)
         if( FB .eq. 0.D0 .or. extemp .eq. tamb) then
            de_x = dia_eff
         else 
            de_x = 2.D0*beta*SQRT(FM)/(grav*alpha)*SQRT(tap*(tap+alpha))
         end if 
         de_x2 = de_x * de_x
c Pseudo---Stack Gas Exit Velocity, Ve (m/s)
         if( FB .eq. 0.) then 
            ue_x = max(uemin,min(c_flare_tgas,ue))
         else 
            ue_x = max(uemin,min(c_flare_tgas,grav*alpha/(beta*tap)))
         end if 
         ue_x2 = ue_x * ue_x

c Volumetric Flow ----  (Rm3/s)
         flow_pseudo_v = ue_x*pi*0.25*de_x2*(Tref/extemp)*(plocal/Pref)
c Momentum        ----  m4/s2
         FM_pseudo = 0.25*Tamb/extemp*(ue_x*de_x)**2.0
c Buoyancy        ----  m4/s3
         FB_pseudo = 0.25*grav*(extemp-tamb)/extemp*ue_x*de_x2

c Correct Momentum Flux,m4/s2
          FM_check = 0.25*tamb/extemp *ue_x2*de_x2
c Correct  Buoyancy Flux (FB infinity) at site conditions,m4/s3
          tdiff   = extemp-tamb
          FB_check = 0.25*grav*tdiff/extemp*ue_x*de_x2

c =============================================================================      
c --- PSEUDO-STACK PARAMETERS
c =============================================================================      
c
c --- STRIPPED GAS STREAM -----------------------------------------------
c --- specify stripped gases source parameters
c
c =============================================================================      
      else if( MSTRIP .eq. 1 .or. 
     &    MSTRIP .eq. 3 .or.  
     &    MSTRIP .eq. 4 .or.  
     &    MSTRIP .eq. 5) then

c Three equations
c Pseudo Stack Parameters --- AERflare Method
c Conserve FB, FM and Volumetric Flow         

c ---------------------------------------------------------------
c Reduced Parameters
c Reduced exhaust temperature, Ts'	--	
         tsp = tstrip/Tref
c Reduced ambient temperature, Ta'	--	
         tap = Tamb/Tref
c Reduced ambient pressure, Pa'	--	
         pap = plocal/Pref
c Dimensionless buoyancy flux to volume flow ratio term, alpha	--	
         alpha = pi*pap*FB_strip/(grav*flow_stripped_ref_v)
c Buoyancy flux to Momentum Flux Ratio, beta	--	
         beta = FB_strip/FM_strip
c ---------------------------------------------------------------
          
c Pseudo---Stack Height, Hs (m)          
         hs_x = max(hs+2.*min(dia_f*(ue/wsST-1.5),0.),0.) + zL*0.5
         if(mgroundflare.eq.1) then
           hs_x = hs
           if(mflameout.ge.1 .or. isFlammable.eq.0 ) then
             hs_x = hs
           end if 
         end if 
c Pseudo---Stack gas Exit Temperture, Te (K)
         extemp = (alpha+tap)*(Tref)
c Pseudo---Diameter, De (m)
         if( FB_strip .eq. 0.D0 .or. extemp .eq. tamb) then
            de_x = dia_eff
         else 
            de_x = 2*beta*SQRT(FM_strip)/(grav*alpha)*SQRT(tap*(tap+alpha))
         end if 
         de_x2 = de_x * de_x
c Pseudo---Stack Gas Exit Velocity, Ve (m/s)
         if( FB_strip .eq. 0.) then 
            ue_x = max(uemin,min(c_flare_tgas,ue))
         else 
            ue_x = max(uemin,min(c_flare_tgas,grav*alpha/(beta*tap)))
         end if 
         ue_x2 = ue_x * ue_x

c Volumetric Flow ----  (sm3/s)
         flow_pseudo_v = ue_x*pi*0.25*de_x2*(Tref/extemp)*(plocal/Pref)
c Momentum        ----  m4/s2
         FM_pseudo = 0.25*Tamb/extemp*(ue_x*de_x)**2.0
c Buoyancy        ----  m4/s3
         FB_pseudo = 0.25*grav*(extemp-tamb)/extemp*ue_x*de_x2
          
c Correct Momentum Flux,m4/s2
          FM_check = 0.25*tamb/extemp *ue_x2*de_x2
c Correct  Buoyancy Flux (FB infinity) at site conditions,m4/s3
          tdiff   = extemp-tamb
          FB_check = 0.25*grav*tdiff/extemp*ue_x*de_x2

          
     
c Stripped gas source temperature, K
         ts = tstrip
c Stripped gas source temperature, K
         extemp = tstrip 

c These are re-adjusted for site conditions below      
c Buoyancy Flux Simplified Method	m4/s3
         FB_strip = FB_stripped   
c Momentum Flux	m4/s2
         FM_strip = max(FM_strip,0.)

c set temperature equal to ambient
c      tambk   = tamb
         tdiff   = tstrip-tamb
         if(tdiff.lt.0.)then
             tdiff = 0.
         end if 
         
      
c Pseudo Stack Gas Exit Velocity (VE)	m/s
         ue_x = ue
         if(FB_strip.ne.0 .and. tdiff.ne.0.) then
            ue_x = grav*FM_strip/FB_strip*tdiff/tamb
         end if   
         ue_x = min(c_flare_tflared,ue_x)
          if(ue_x .lt. uemin) then
            ue_x = uemin
            de_x = dia_f
            extemp = tamb
            FM = 0.
            FB = 0.
            FB_strip = 0.0
          end if 
         
c Effective Pseudo Stack Height (hS) Adjusted for Flare Tip Downwash	m
         hs_x = max(hs+2.*min(dia_f*(ue/wsST-1.5),0.)+zL*0.5,0.)

c Pseudo Stack Inside Diameter (DE)	m
         de_x = dia_f
         if(FB_strip.ne.0. .and. tdiff.ne.0.) then
            de_x = 2.*sqrt(FB_stripped*tstrip/(tdiff*grav*ue_x))
         else 
            if(FM .ne. 0.) then
               de_x = sqrt( FM/(0.25*tamb/extemp *ue_x*ue_x))
            end if 
         end if 

c 140812 : BWZ 
c Recalculate the estimated exit temperature to account for the possible 
c truncation of the ue_x  while conserving the FM and FB
c tflame is the estimated flame temperature
c extemp is the temperature used for modelling          
         if(FM_strip .ne. 0.0) then
            extemp = tamb*(FB_strip/FM_strip*ue_x/grav + 1.d0)
         else
            extemp = tamb
         end if 
         tdiff = extemp-tamb
         if(tdiff.lt.0.)then
             tdiff = 0.
         end if 
         
c calculate H2S emitted using combustion efficiency
         if(MSTRIP.eq.1) then
            qemit = e_h2s
         elseif(MSTRIP.eq.3) then
            qemit = e_h2s
         elseif(MSTRIP.eq.4) then
            qemit = e_OU
         elseif(MSTRIP.eq.5) then
            qemit = e_OU
         end if 

          FB = 0.25*grav*tdiff/extemp *ue_x*de_x*de_x
          FM = 0.25*tamb/extemp *ue_x*ue_x*de_x*de_x
      end if
c =============================================================================      
c End Pseudo-Parameters
c =============================================================================      

c =============================================================================      
c AERflare spreadsheet equivalent output
c =============================================================================      

c =============================================================================      
c END AERflare spreadsheet equivalent output
c =============================================================================      

      
c dump output      
      if(debug .gt. 0 ) then
      write(iolog,'(A82)') 'XOUTHDRX,wspd10m,wsST,ipgIN,tamb,FM,FB,hs_x,de_x,ue_x,extemp,tstrip,carb,e_so2,e_h2s,e_OU'
      write(iolog,'(A,1x,15G15.4)') 'XOUTX',wspd10m,wsST,ipgIN,tamb,FM,FB,hs_x,de_x,ue_x,extemp,tstrip,carb,e_so2,e_h2s,e_OU
      end if 
      
      if( FM .ne. 0. .and. MSTRIP.eq.0) then
          if( abs((FM_check-FM)) .gt. 0.00001*FM ) then
c     CALCULATION ERROR
              write(6,*) '*** ERROR *** bad calc in FM'
              write(6,*) 'FM_check :', FM_check
              write(6,*) '      FM :', FM
          end if 
      end if    
      if( FB .ne. 0. .and. MSTRIP.eq.0) then
          if( abs((FB_check-FB)) .gt. 0.00001*FB ) then
c     CALCULATION ERROR
              write(6,*) '*** ERROR *** bad calc in FB'
              write(6,*) 'FB_check :', FB_check
              write(6,*) '      FB :', FB
          end if 
          end if    

c =============================================================================      
c Comparison to AERflare
c =============================================================================      
      if(debug .gt. 0 ) then
        
      if (.true.) then
        io_free = get_free_io(99)
      open(unit=io_free ,file='aerflare_dump.txt',iostat=ier,status='unknown')        
      
      cell(5)=0
      cell(6)=0
      cell(7)=0
      cell(8)=0
      cell(9)=mix(jph2s)
      cell(10)=hs
      cell(11)=dia_f
      cell(12)=dia_unob
      cell(13)=dlift
      cell(14)=dfuel
      cell(15)=zcoord
      cell(16)=plocal
      cell(17)=totalvolume
      cell(18)=duration/3600
      cell(19)=MFASSISTSTEAM
      cell(20)=MFASSISTAIR
      cell(21)=MFASSISTHPFUEL
      cell(22)=pstm
      cell(23)=tstm-273.15
      cell(24)=0
      cell(25)=0
      cell(26)=0
      cell(27)=effminERCB
      cell(28)=effmin
      cell(29)=0
      cell(30)=0
      cell(31)=radloss
      cell(32)=0
      cell(33)=tgas-273.15
      cell(34)=tamb-273.15
      cell(35)=ipgin
      cell(36)=0
      cell(37)=0
      cell(38)=tradbkg
      cell(39)=tradtot
      cell(40)=u10min
      cell(41)=wspd10m
      cell(42)=wsmax
      cell(43)=0
      cell(44)=uemin
      cell(45)=zref
      cell(46)=tgrad
      cell(47)=0
      cell(48)=uofaa
      cell(49)=uofab
      cell(50)=0
      cell(51)=0
      cell(52)=tref-273.15
      cell(53)=pref
      cell(54)=ru
      cell(55)=grav
      cell(56)=0
      cell(57)=spv
      cell(58)=rk1
      cell(59)=rk2
      cell(60)=0
      cell(61)=rk4
      cell(62)=0
      cell(63)=0
      cell(64)=0
      cell(65)=fgrIN
      cell(66)=air_flow_assist
      cell(67)=hpfuel_flow
      cell(68)=0
      cell(69)=stm_flow
      cell(70)=air_flow_adjusted
      cell(71)=qliftgas
      cell(72)=rateIN
      cell(73)=totalflaringrate_ve3m3
      cell(74)=0
      cell(75)=0
      cell(76)=mix(1)
      cell(77)=mix(2)
      cell(78)=mix(3)
      cell(79)=mix(4)
      cell(80)=mix(5)
      cell(81)=mix(6)
      cell(82)=mix(7)
      cell(83)=mix(8)
      cell(84)=mix(9)
      cell(85)=mix(10)
      cell(86)=mix(11)
      cell(87)=mix(12)
      cell(88)=mix(13)
      cell(89)=mix(14)
      cell(90)=mix(15)
      cell(91)=mix(16)
      cell(92)=mix(17)
      cell(93)=mix(18)
      cell(94)=mix(19)
      cell(95)=mix(20)
      cell(96)=mix(21)
      cell(97)=mix(22)
      cell(98)=mix(23)
      cell(99)=mix(24)
      cell(100)=mix(25)
      cell(101)=mix(26)
      cell(102)=mix(27)
      cell(103)=mix(28)
      cell(104)=mix(29)
      cell(105)=mix(30)
      cell(106)=mix(31)
      cell(107)=mix(32)
      cell(108)=0
      cell(109)=rmm_netair
      cell(110)=cpmol_mix
      cell(111)=cpmass_mix
      cell(112)=rk_mix
      cell(113)=c_flare_tgas
      cell(114)=rnhv
      cell(115)=rnhv_mass
      cell(116)=rghv
      cell(117)=0
      cell(118)=0
      cell(119)=0
      cell(120)=0
      cell(121)=0
      cell(122)=0
      cell(123)=0
      cell(124)=raw_v
      cell(125)=raw_m
      cell(126)=lift_v
      cell(127)=lift_m
      cell(128)=fuel_v
      cell(129)=fuel_m
      cell(130)=hpfuel_v
      cell(131)=hpfuel_m
      cell(132)=mix0_v
      cell(133)=mix0_m
      cell(134)=stm_v
      cell(135)=stm_m
      cell(136)=air_v
      cell(137)=air_m
      cell(138)=air_assist_v
      cell(139)=air_assist_m
      cell(140)=totalflaringrate_v
      cell(141)=totalflaringrate_m
      cell(142)=rnetheatreleased_a
      cell(143)=0
      cell(144)=rnetheatreleased_c
      cell(145)=0
      cell(146)=fuel_carbons
      cell(147)=fuel_hydrogen
      cell(148)=mix(jpco2)
      cell(149)=0
      cell(150)=0
      cell(151)=0
      cell(152)=0
      cell(153)=e_h2s_v
      cell(154)=0
      cell(155)=0
      cell(156)=e_h2s_m
      cell(157)=e_h2s
      cell(158)=e_ou/1e6
      cell(159)=e_so2_m
      cell(160)=e_so2_m1
      cell(161)=e_s_m1
      cell(162)=e_s_mtotal
      cell(163)=e_so2
      cell(164)=0
      cell(165)=e_h2s
      cell(166)=mix(jph2s)
      cell(167)=e_co2_process
      cell(168)=e_co2_combustion
      cell(169)=e_co2_m1
      cell(170)=e_co2_total
      cell(171)=0
      cell(172)=0
      cell(173)=0
      cell(174)=0
      cell(175)=0
      cell(176)=0
      cell(177)=0
      cell(178)=0
      cell(179)=0
      cell(180)=0
      cell(181)=0
      cell(182)=0
      cell(183)=0
      cell(184)=0
      cell(185)=0
      cell(186)=0
      cell(187)=0
      cell(188)=0
      cell(189)=0
      cell(190)=0
      cell(191)=eff_max
      cell(192)=rnhv_m_98
      cell(193)=rate_98
      cell(194)=dia_98
      cell(195)=0
      cell(196)=0
      cell(197)=carb_wsmin
      cell(198)=carb_wsmax
      cell(199)=carb_wsmax
      cell(200)=carb
      cell(201)=fs
      cell(202)=fs_cold
      cell(203)=fs_hot
      cell(204)=comb
      cell(205)=comb_net
      cell(206)=0
      cell(207)=0
      cell(208)=0
      cell(209)=0
      cell(210)=0
      cell(211)=0
      cell(212)=0
      cell(213)=0
      cell(214)=0
      cell(215)=0
      cell(216)=ueflare
      cell(217)=uelift
      cell(218)=uefuel
      cell(219)=uestm
      cell(220)=ueair
      cell(221)=uehpfuel
      cell(222)=0
      cell(223)=0
      cell(224)=0
      cell(225)=0
      cell(226)=0
      cell(227)=0
      cell(228)=FM_flare
      cell(229)=FM_lift
      cell(230)=FM_fuel
      cell(231)=FM_stm
      cell(232)=FM_air
      cell(233)=FM_hpfuel
      cell(234)=-FM_stripped
      cell(235)=FM
      cell(236)=0
      cell(237)=ue
      cell(238)=dia_eff
      cell(239)=ue/c_flare_tflared
      cell(240)=0
      cell(241)=tsp
      cell(242)=tap
      cell(243)=pap
      cell(244)=alpha
      cell(245)=beta
      cell(246)=radloss
      cell(247)=hs_x
      cell(248)=de_x
      cell(249)=ue_x
      cell(250)=extemp
      cell(251)=tamb
      cell(252)=flow_pseudo_v
      cell(253)=FM_pseudo
      cell(254)=FB_pseudo
      cell(255)=0
      cell(256)=0
      cell(257)=0
      cell(258)=0
      cell(259)=0
      cell(260)=0
      cell(261)=0
      cell(262)=0
      cell(263)=0
      cell(264)=0
      cell(265)=0
      cell(266)=0
      cell(267)=0
      cell(268)=0
      cell(269)=0
      cell(270)=0
      cell(271)=0
      cell(272)=0
      cell(273)=0
      cell(274)=0
      cell(275)=0
      cell(276)=0
      cell(277)=0
      cell(278)=0
      cell(279)=0
      cell(280)=0
      cell(281)=0
      cell(282)=0
      cell(283)=0
      cell(284)=0
      cell(285)=0
      cell(286)=0
      cell(287)=0
      cell(288)=0
      cell(289)=0
      cell(290)=0
      cell(291)=0
      cell(292)=0
      cell(293)=0
      cell(294)=0
      cell(295)=0
      cell(296)=0
      cell(297)=0
      cell(298)=0
      cell(299)=0
      cell(300)=0
      cell(301)=0
      cell(302)=0
      cell(303)=0
      cell(304)=0
      cell(305)=0
      cell(306)=0
      cell(307)=0
      cell(308)=0
      cell(309)=0
      cell(310)=0
      cell(311)=0
      cell(312)=0
      cell(313)=0
      cell(314)=0
      cell(315)=0
      cell(316)=0
      cell(317)=0
      cell(318)=0
      cell(319)=0
      cell(320)=0
      cell(321)=0
      cell(322)=0
      cell(323)=0
      cell(324)=0
      cell(325)=0
      cell(326)=0
      cell(327)=0
      cell(328)=0
      cell(329)=0
      cell(330)=0
      cell(331)=0
      cell(332)=0
      cell(333)=0
      cell(334)=0
      cell(335)=0
      cell(336)=0
      cell(337)=0
      cell(338)=0
      cell(339)=0
      cell(340)=0
      cell(341)=0
      cell(342)=0
      cell(343)=0
      cell(344)=0
      cell(345)=0
      cell(346)=0
      cell(347)=0
      cell(348)=0
      cell(349)=0
      cell(350)=0
      cell(351)=0
      cell(352)=0
      cell(353)=0
      cell(354)=0
      cell(355)=0
      cell(356)=0
      cell(357)=0
      cell(358)=0
      cell(359)=0
      cell(360)=0
      cell(361)=0
      cell(362)=0
      cell(363)=0
      cell(364)=0
      cell(365)=0
      cell(366)=0
      cell(367)=0
      cell(368)=0
      cell(369)=0
      cell(370)=0
      cell(371)=0
      cell(372)=0
      cell(373)=0
      cell(374)=0
      cell(375)=0
      cell(376)=0
      cell(377)=0
      cell(378)=rLFLcomb
      cell(379)=rLFLflared
      cell(380)=rminoxyg
      cell(381)=rmaxinert
      cell(382)=isFlammable
      cell(383)=0
      cell(384)=cpmass_fuel_ave
      cell(385)=0
      cell(386)=0
      cell(387)=0
      cell(388)=rmamf_moc
      cell(389)=rmamf
      cell(390)=stoic_o2
      cell(391)=stoic_air
      cell(392)=fealfl
      cell(393)=rfsupo2
      cell(394)=exwet
      cell(395)=exdry
      cell(396)=rfh2o
      cell(397)=rfo2
      cell(398)=rfar
      cell(399)=rfn2
      cell(400)=rfhe
      cell(401)=rfne
      cell(402)=rfkr
      cell(403)=rfn2o
      cell(404)=rfxe
      cell(405)=rfco2
      cell(406)=rfso2
      cell(407)=excessair
      cell(408)=o2FracExWet
      cell(409)=o2FracExDry
      cell(410)=exFlow_m
      cell(411)=airFlow_v
      cell(412)=0
      cell(413)=exFlow_v
      cell(414)=0
      cell(415)=tflame
      cell(416)=exrate_v
      cell(417)=0
      cell(418)=mix_exhaust(jph2o)
      cell(419)=mix_exhaust(jpo2) 
      cell(420)=mix_exhaust(jpar) 
      cell(421)=mix_exhaust(jpn2) 
      cell(422)=mix_exhaust(jphe) 
      cell(423)=mix_exhaust(jpne) 
      cell(424)=mix_exhaust(jpkr) 
      cell(425)=mix_exhaust(jpn2o)
      cell(426)=mix_exhaust(jpxe) 
      cell(427)=mix_exhaust(jpco2)
      cell(428)=mix_exhaust(jpso2)
      cell(429)=0
      cell(430)=rmm_ex
      cell(431)=excp
      cell(432)=excp_m
      cell(433)=excp_texh_ave
      cell(434)=excpmass_texh_ave
      cell(435)=exfmr
      cell(436)=tafmr
      cell(437)=rexcessair
      cell(438)=xx_excp_texh_ave
      cell(439)=xx_excpmass_texh_ave
      cell(440)=0
      cell(441)=0
      cell(442)=0
      cell(443)=hf_raw
      cell(444)=hf_lift
      cell(445)=hf_fuel
      cell(446)=hf_stm
      cell(447)=hf_air
      cell(448)=hf_hpfuel
      cell(449)=hf_mix
      cell(450)=tmix
      cell(451)=0
      cell(452)=egoal_mix
      cell(453)=tflared
      cell(454)=0
      cell(455)=0
      cell(456)=h1
      cell(457)=h2
      cell(458)=h3
      cell(459)=h4
      cell(460)=h5
      cell(461)=h6
      cell(462)=h7
      cell(463)=heatp
      cell(464)=0
      cell(465)=t0
      cell(466)=t1
      cell(467)=t2
      cell(468)=t3
      cell(469)=t4
      cell(470)=t5
      cell(471)=t6
      cell(472)=t7
      cell(473)=t8
      cell(474)=tguess
      cell(475)=0
      cell(476)=egoal_ex
      cell(477)=tflame
      cell(478)=heat_p
      cell(479)=tflame
      cell(480)=0
      cell(481)=0
      cell(482)=0
      cell(483)=tamb
      cell(484)=plocal
      cell(485)=0
      cell(486)=0
      cell(487)=0
      cell(488)=den_air
      cell(489)=0
      cell(490)=0
      cell(491)=0
      cell(492)=0
      cell(493)=0
      cell(494)=tamb
      cell(495)=plocal
      cell(496)=cpmol_air_tamb
      cell(497)=0
      cell(498)=rk_air
      cell(499)=den_air_tamb
      cell(500)=DenRatioAir
      cell(501)=EntrainmentRatioAirAir
      cell(502)=0
      cell(503)=0
      cell(504)=0
      cell(505)=z_air_tamb
      cell(506)=c_air_tamb
      cell(507)=cpmol_air_ave_tamb
      cell(508)=cpmass_air_ave_tamb
      cell(509)=0
      cell(510)=tstm
      cell(511)=pstm
      cell(512)=cpmol_stm_tstm
      cell(513)=hf_stm_tstm
      cell(514)=rk_stm
      cell(515)=tstm_exit
      cell(516)=stm_press_exit
      cell(517)=stm_den
      cell(518)=0
      cell(519)=0
      cell(520)=0
      cell(521)=z_stm_tstm_exit
      cell(522)=c_stm_tstm
      cell(523)=cpmol_stm_ave_tstm
      cell(524)=cpmass_stm_ave_tstm
      cell(525)=0
      cell(526)=den_fuel_tgas
      cell(527)=DenRatioFuel
      cell(528)=EntrainmentRatioAirFuel
      cell(529)=0
      cell(530)=0
      cell(531)=fruf
      cell(532)=hre1
      cell(533)=hre2
      cell(534)=hre3
      cell(535)=hre4
      cell(536)=hre5
      cell(537)=hre6
      cell(538)=FB
      cell(539)=FB_infinity
      cell(540)=FB_alt
      cell(541)=FB_stm
      cell(542)=FB_air
      cell(543)=0
      cell(544)=fruf
      cell(545)=hre1_fo
      cell(546)=hre2_fo
      cell(547)=hre3_fo
      cell(548)=hre4_fo
      cell(549)=hre5_fo
      cell(550)=hre6_fo
      cell(551)=FB_fo
      cell(552)=FB_fo
      cell(553)=0
      cell(554)=0
      cell(555)=0
      cell(556)=totalflaringrate_m
      cell(557)=fs_cold
      cell(558)=stripped_m
      cell(559)=stripped_air
      cell(560)=stripped_tot
      cell(561)=tautoig-273.15
      cell(562)=tstrip
      cell(563)=smcp
      cell(564)=smaircp
      cell(565)=smtotcp
      cell(566)=rmmsm
      cell(567)=FB_stripped
      cell(568)=FB_strip2
      cell(569)=FB_strip3
      cell(570)=htot
      cell(571)=qstrip
      cell(572)=FM_stripped
      cell(573)=flow_stripped_ref_v
      cell(574)=0
      cell(575)=tap
      cell(576)=pap
      cell(577)=0
      cell(578)=0
      cell(579)=0
      cell(580)=0
      cell(581)=0
      cell(582)=0
      cell(583)=tamb
      cell(584)=0
      cell(585)=0
      cell(586)=0
      cell(587)=0
      cell(588)=0
      cell(589)=Vj_Ua
      cell(590)=rjet
      cell(591)=0
      cell(592)=0
      cell(593)=0
      cell(594)=0
      cell(595)=ZL
      cell(596)=XL
      cell(597)=flameangle
      cell(598)=0
      cell(599)=heat_released_nowind
      cell(600)=dia_h
      cell(601)=heat_flux
      cell(602)=z_flame_subsonic
      cell(603)=z_flame_sonic
      cell(604)=z_flame_nowind
      cell(605)=0
      cell(606)=uv
      cell(607)=uu
      cell(608)=plumeangle
      cell(609)=us
      cell(610)=dia2
      cell(611)=tflame
      cell(612)=conv_so2
      cell(613)=resid_h2s
      cell(614)=eta
      cell(615)=flame_length
      cell(616)=flame_volume
      cell(617)=exrate_v
      cell(618)=residence_time
      cell(619)=0
      cell(620)=0
      cell(621)=0
      cell(622)=0
      cell(623)=0
      cell(624)=0
      cell(625)=0
      cell(626)=0
      cell(627)=0
      cell(628)=0
      cell(629)=0
      cell(630)=0
      cell(631)=0
      cell(632)=0
      cell(633)=0
      cell(634)=den_flare
      cell(635)=0
      cell(636)=0
      cell(637)=0
      cell(638)=0
      cell(639)=0
      cell(640)=0
      cell(641)=0
      cell(642)=0
      cell(643)=0
      cell(644)=0
      cell(645)=0
      cell(646)=0
      cell(647)=0
      cell(648)=0
      cell(649)=den_flare
      cell(650)=0
      cell(651)=0
      cell(652)=0
      cell(653)=0
      cell(654)=0
      cell(655)=0
      cell(656)=0
      cell(657)=0
      cell(658)=0
      cell(659)=0
      cell(660)=0
      cell(661)=0

      write(io_free,'(g20.10)') (cell(i),i=1,700)
      close(io_free)
      end if 

      end if 
      
      return
      end                               
