c----------------------------------------------------------------------
c     ABflare
c----------------------------------------------------------------------
c     
c --- Version: 1.001    Level: 110706
c
c     Copyright (c) 2012 by Zelt Professional Services Inc.         
c
c Developed by 
c     B.Zelt, Zelt PSI
c     brian@zeltpsi.com
c     www.zeltpsi.com
c     Okotoks, Alberta, CANADA
c
c UPDATES
c----------------------------------------------------------------------
c 141217
c   Changed the emissions(), for when ue_x<uemin.  Now set to zero exit velocity
c   which will set the exit temperature to tamb and the resulting modelling 
c   momentum flux will be zero.
c
c ---   Version: 1.001    Level: 110706
c       (1) Initial Development
c
c
c----------------------------------------------------------------------     
c     
      program ABflare          
      use mod_progver 
      use mod_stddefs
      use mod_readinp
      use mod_combustion 
      use mod_met
      use mod_file1
      use mod_globals

c----------------------------------------------------------------------
c DECLARATIONS
      real(8) :: time_begin, time_end
      logical :: bOK  ! flag for check unit open status

      parameter (iolst = 20)	! lst file of inputs
      parameter (ioab  = 21)	! output io for abflare model output
      parameter (iomet = 22)	! met timeseries input file
      parameter (iolog = 23)	! lst/log file for debugging
      parameter (ioin  = 24)	! inp file of inputs
      
      character*mxpath :: FLOOP
      character*mxpath :: FIN
      character*mxpath :: FROOT
      character*mxpath :: inpfil
      character*mxpath :: cl_arg1
      character*8   :: timestr
      real(8) :: tempinitx

      character(len=5)    ::  ulcase(2)
      data ulcase/'UPPER','LOWER'/
      
      call cpu_time(time_begin)

c -- reset standard output 
      open(6,CARRIAGECONTROL='FORTRAN') 

      tamb = 0.
      
c --- get commandline input file name      
      inpfil = 'abflare.inp'
      call getcom(cl_arg1)
      if(len_trim(cl_arg1)>0) then
          call getcom(inpfil)
          write(6,*) 'Control file:[', trim(inpfil),']'
          write(6,*)
      endif         
      FIN = inpfil

c set gas compositions to zero      
      ier = initialize(gcraw,0.0d0)
      ier = initialize(gcraw_x,0.0d0)
      ier = initialize(gcfuel,0.0d0)
      ier = initialize(gcmix,0.0d0)
      ier = initialize(gclift,0.0d0)

c read inputs       
      ier = irdcontrol(ioin, FIN )

c
c --- open log file 'abflare.log'
c     to record overall status and messages
      call mk_case(-1, lcfiles, LSTDAT, ierr)
      open(iolst,file=LSTDAT,iostat=ierr,status='unknown')
      if(ierr .ne. 0) then
          write(6,*) 'Error opening LSTDAT:', LSTDAT    
          goto 9999
      end if 

      call mk_case(iolst, lcfiles, LOGDAT, ierr)
      call mk_case(iolst, lcfiles, METDAT, ierr)
      call mk_case(iolst, lcfiles, METDAT2, ierr)
      call mk_case(iolst, lcfiles, RECINC, ierr)
      call mk_case(iolst, lcfiles, OUTNAME, ierr)
      call mk_case(iolst, lcfiles, OUTPATH, ierr)

c open log file for recording debug information      
      DEBUG = 0
      bOK = .FALSE.
      if(MDEBUG .eq. 1) then
      if( len_real(LOGDAT) .gt. 0) then
          open(iolog,file=LOGDAT,iostat=ierr,status='unknown')
          inquire(unit=iolog, opened=bOK)
      if(ierr .ne. 0) then
          write(6,*) 'Error opening LOGDAT:', LOGDAT
          goto 9999
      end if 
      if(bOK) then 
          DEBUG = 1
      end if 
      end if           
      end if 
      
c --- brand lst file
      write(iolst,*)
     & 'Model:',mmodel,' Version:',mversion,' Level:',mlevel
      
c --- brand log file
      if(debug .gt. 0 ) then
      write(iolog,*)
     & 'Model:',mmodel,' Version:',mversion,' Level:',mlevel
      end if 
      
      FIN = LSTDAT  

c write out the read inputs       
      ier = iwrcontrol(iolst )

c check inputs
      ier = ichkinput(iolst)
      if( ier .gt.0 ) then
          write(iolst,*) 'Number of errors in input file:', ier
          call time(timestr)
          write(iolst,*) timestr
          goto 9999
      end if 

c determine the units
c      MUNITS = 2 ! assume km
c      if( leq(units,'m').eq.1 .or. leq(units,'M').eq.1 ) then
c          MUNITS = 1  ! set m
c      end if 
      
c load default variables
      call setup1()
	tamb = 0.0

c calculate site pressure based upon the elevation
      call InitLocal(zcoord)

c adjust steam assist from mass input (kg/s) to volume input (e3m3/d)
      if( MFASSIST.eq.1 .AND. MFASSISTSTEAM.eq.1) then
      if( stm_imassvol .eq. 1) then
          stm_flow = stm_flow/prop(ipmmass,jph2o)*spv*0.001*3600.*24.
      end if 
c      if( stm_press .le. 0. .and. stm_temp .le. 0.) then
c          write(6,*) 
c     &         'FATAL ERROR: steam temp and steam pressure both missing'
c          write(iolst,*) 
c     &         'FATAL ERROR: steam temp and steam pressure both missing'
c          goto 9999
c      end if
      end if
      if(( stm_press .le. 0. ) .and.( stm_temp .le. 0. )) then
           stm_temp  = 200. + 273.15;  ! default is 200C Saturated Steam
      end if    
      if( stm_press .le. 0. ) then
           stm_press = p_sat(stm_temp)  + plocal  
      else
          stm_press = stm_press + plocal
      end if                      
      if( stm_temp .le. 0. ) then
           stm_temp = t_sat(stm_press)   
      end if                      

      
c calculate start date stamp    
      call julday(iolst,iyys,imms,idds,ijjs)  
      call mkdat(iyys,ijjs,ihhs,idatestart)  
      call julday(iolst,iyye,imme,idde,ijje)
      call mkdat(iyye,ijje,ihhe,idateend)  

c calculated mixed gas
      if(rh2s .gt. 0.) then
          ierr = inormalizeh2s(gcraw, rh2s, gcraw_x)
      else
c basically, make a copy into gcraw_x
          ierr = inormalizeh2s(gcraw, gcraw(jph2s), gcraw_x) 
      end if 

      if(MINPUT .eq. 1 ) then
         tempinitx = tempinit 
         if( tempinitx .lt. 0. ) then
            tempinitx = tamb                  
         end if 
c --- Calculate blowdown curve
c
c                     0: steady
c                     1: vessel blowdown 
c                     2: user specified qmax
c                     3: depressurization Qave, 50% in 15min
c                     3: depressurization Qmax, 50% in 15min
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  STEADY
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if( mblowdown .eq. 0) then
            npuffs   = 1
            ipuffdur = max(itiminc,ipuffdur)
            qtotal   = qmax*ipuffdur/24./3600.
            pseq(1,1) = qmax
            pseq(1,2) = ipuffdur 
            pseq(1,3) = -1. ! use default
            pseq(1,4) = -1. ! use default
            
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  BLOWDOWN : VESSEL 
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         else if( mblowdown .eq. 1) then
c --- vessel exponential blowdown 
            ierr= ivessel(gcraw_x,pressinit,tempinitx,pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgas2   )
            ierr = iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)
            if(debug .gt.0 )then
               write(iolog,*) 'npuffs=',npuffs
               do i=1,npuffs
                  write(iolog,*) pseq(i,1), pseq(i,2)
               end do            
            end if 

c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  BLOWDOWN : QMAX & VOLUME
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         else if( mblowdown .eq. 2) then
            qmax_bd = qmax
            qtotal_bd = qtotal
            ierr = iblowdown(qmax_bd, qtotal_bd, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)

c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  BLOWDOWN : QAVE, PRATIO & TDEP
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         else if( mblowdown .eq. 3) then
            qmax_bd = -qmax*log(1.D0-frel)
            qtotal_bf = qmax_bd*(-Tdep/LOG(Pratio))/(24.D0*60.D0)
            ierr = iblowdown(qmax_bd, qtotal_bf, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)

c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  BLOWDOWN : QMAX, PRATIO & TDEP
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         else if( mblowdown .eq. 4) then
            qmax_bd = qmax
            qtotal_bf = qmax_bd*(-Tdep/LOG(Pratio))/(24.D0*60.D0)
            ierr = iblowdown(qmax_bd, qtotal_bf, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)

c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  BLOWDOWN : QAVE, TDEP and PRATIO
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      else if( mblowdown .eq. 5) then
            tau = -Tdep/LOG(Pratio)
            qmax_bd = qmax*(Tdep/tau)/(1.D0-EXP(-Tdep/tau))
            qtotal_bf = qmax*tau/(24.D0*60.D0)
            ierr = iblowdown(qmax_bd, qtotal_bf, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)
            
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c  ERROR
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         else 
            write(6,*) 'FATAL ERROR:  Unknown source type'
            write(6,*) 'MBLOWDOWN must be 0,1,2,3,4,5'
            write(iolst,*) 'FATAL ERROR: Unknown source type'
            write(iolst,*) 'MBLOWDOWN must be 0,1,2,3,4,5'
            goto 9999
         end if     
         qmax0 = qmax
         
      else if (MINPUT .eq. 2) then
c --- User input blowdown curve
          npuffs = nseq  
          qmax0 = pseq(1,1)
          if( qmax0 .le. 0. .and. MFGR .eq. 2) then
              qmax0 = qmax
              write(6,*) '***WARNING*** setting QMAX0=QMAX:',QMAX
              write(iolst,*) '***WARNING*** setting QMAX0=QMAX:',QMAX
          end if 
          
      else if (MINPUT .eq. 3) then
c --- Static user input source 
c sequential puffs must be maximum of hour long
          ipuffdur = max(itiminc,ipuffdur)
          npuffs = 1  
          pseq(1,1) = rqemit ! note this is kg/s
          pseq(1,2) = ipuffdur 
          pseq(1,3) = -1. ! use default
          pseq(1,4) = -1. ! use default
      else 
          write(6,*) 'FATAL ERROR:  Unknown source type'
          write(6,*) 'MINPUT must be 1 or 2 or 3'
          write(iolst,*) 'FATAL ERROR: Unknown source type'
          write(iolst,*) 'MINPUT must be 1 or 2 or 3'
          goto 9999
      end if 

      if( debug ) then
              write(iolog,*) 'pseq qmax,duration(s),h2s,fgr   ip=start'
              write(iolog,"(4(G11.4,1x))")
     &              ((pseq(ip,1),pseq(ip,2),pseq(ip,3),
     &              pseq(ip,4)),ip=1,npuffs)
      end if 


c ======================================================================
c The CALPUFF output format is selected
c
c
c
      if( MDSPMOD.eq.1) then
         call mkfilcp(ioab, iolst, iomet,iolog)
         
c ======================================================================
c The AERMOD output format is selected.
c Since AERMOD is a plume model, looping and NSEP are not required
c for the source model, but only separate runs for each stage of an
c NSEQ blowdown.  Post processing can then determine appropriate scaled
c hourly or daily averages by combining the results.
      else if( MDSPMOD.eq.2) then
         call mkfilam(ioab, iolst, iomet,iolog)
      end if    
c ======================================================================


c Normal Termination
      call cpu_time(time_end)
      write(iolst,*) 'CPU Time ', time_end-time_begin, ' seconds'
      write(6,*) 'Program Terminated'
      write(iolst,*) 'Program Terminated'
      if(DEBUG .gt. 0) then
          write(iolog,*) 'Program Terminated'
      end if           
      close(iolog)
      close(iolst)


      stop

c warnings/errors Termination
 9999 write(6,*)     'Program Terminated with ***ERRORS***'
      write(iolst,*) 'Program Terminated with ***ERRORS***'
      if(DEBUG .gt. 0) then 
         if(bOK) then 
            write(iolog,*)'Program Terminated with ***ERRORS***'
         end if 
      end if 
      close(iolst)
      close(iolog)
      close(ioab)
      close(iomet)
      stop
      end


c ---------------------------------------------------------------------
c subroutine mkfilcp()
c
c creates the series of output files for CALPUFF formated variable 
c emissions file
c ---------------------------------------------------------------------
      subroutine mkfilcp(io, iolst, iomet, iolog)
c --- MODULES      
      use mod_stddefs 
      use mod_readinp 
      use mod_combustion 
      use mod_met 
      use mod_file1
      use mod_globals 
      use mod_isccalcs 
c      implicit none
c --- INPUTS
      integer :: io
      integer :: iolst
      integer :: iomet
      integer :: iolog
c --- DECLARATIONS
      real(8) :: fgri
      
      integer iloop, ip, ih0, imet
      integer ier
      character*mxpath :: FROOT
      character*mxpath :: FLOOP
      
      integer :: len_real, ierr, iyr, ijul, ihr, isec, iyr1, ijul1, ihr1, isec1, i
      integer :: isplit, nloops
      integer :: ivessel, iemitt, iblowend, ij1, ih1, is1 
      real(8) :: qmax1, rh2s1, qtotal1, fgr1, ts_x, fmx

c determine if this is a split puff file or sequential
c      isplit= 0: file is sequential, puffs must be less than 1h duration,
c               each puff-hour follows the next. Simulates continuous.
c      isplit = 1: file is not sequential.  each puff begins on a new hour.
      
c adjust how many files are to be created
c if npuffs>1 then 24 hourly files are created
c if npuffs=1 then
c        if nsep=24 then 24 hourly files are created
c        if nsep=1 then a single file is created
      if(npuffs.gt.1 .or. npuffs.eq.0) then
          isplit = 1
      elseif(npuffs.eq.1 .and. nsep.gt.1) then
          isplit = 1
      else  
          isplit = 0
          nsep   = 1
      end if 

      nloops = 1
      if(isplit .eq. 1) then
          nloops = nsep
      end if 
      
      do iloop=1,nloops   
c create output path for this loop
          
          write(FROOT,"('\',A,'_',I2.2,'.DAT')")
     &           OUTNAME(1:LEN_REAL(OUTNAME)),iloop
          call concat(FLOOP,OUTPATH,FROOT)
      if(DEBUG.gt.0) then
          write(iolog,*) '---------------------------------------------'
          write(iolog,*) 'loop=',iloop
          write(iolog,*) FLOOP
      end if                     
c write ABflare.dat header, open a unit
         call mk_case(iolst, lcfiles, FLOOP, ierr)
         if( MDSPMOD.eq.1) then
            ier = iwrFLEMARBhdr(ioab, iolst, FLOOP)
         else if( MDSPMOD.eq.2) then
            write(iolst,*) 'Probable bad combination '
            write(6,*) 'Probable bad combination '
            stop
         endif
c calculate start date stamp  
          iyr  = iyys
          ijul = ijjs
          ihr  = ihhs
          isec = isss
          call MKDAT(iyr,ijul,ihr,idatecur)

c Open METDAT
         if(MMET .eq. 1) then
            call methdr(iomet,iolst,METDAT,ws,wd,tamb,ipg)  
         else if(MMET .eq. 2) then
            call methdr_aermet(iomet,iolst,METDAT,ws,wd,tamb,ipg,MNONSEQ)  
            if(MSCREENING .eq. 1) then
                ws = real(int(ws/.5))*0.5
                tamb = real(int(tamb/1.))*1.
            end if                 
         else            
            ws   = rwindsp
            if( rwinddr .ge. 0 ) then
               wd   = rwinddr
            end if 
            tamb     = rtamb
            ipg      = irpgclass
            ipgreal  = ipg
            zws      = zref
            if(DEBUG .gt. 0 ) then
c               write(iolog,*) ' ws,wd,tamb,ipg'
c               write(iolog,*)  ws,wd,tamb,ipg
            end if 
         end if 
         tambreal = tamb
         wsreal   = ws
         ipgreal  = ipg
          
c Assume end of puff
          if(iloop.gt. 1) then
              ip = npuffs + 1
              ih0 = ihr - (24-(iloop-1)) 
          else
              ip = 1
              ih0 = ihr
          end if 

c calculate start hour of blowdown puff sequence
          iyr1    = iyr
          ijul1   = ijul
          ihr1    = ihr
          isec1   = isec
          if( ip .gt. 1 ) then
              call incrs(iolst,iyr1,ijul1,ihr1,isec1,(iloop-1)*3600)
c these routines only allow a maximum of -24h backward steps
c so do look backwards in nsep/24 times            
c              call incrs(iolst,iyr1,ijul1,ihr1,isec1,-(24)*3600)
              do i=1,nsep/24
               call incrs(iolst,iyr1,ijul1,ihr1,isec1,-(24)*3600)
              end do
          end if 
          call MKDAT(iyr1,ijul1,ihr1,iblowstart)
          
c ----------------------------------------------------------------------
          do while (idatecur .le. idateend)

          write(6,8001) iloop,idatecur
8001      format('+',I4,1x,I10)               

      if( debug .gt. 0) then
            write(iolog,*) '---------------------------------'
            write(iolog,*) 'iloop: ',iloop, imet
            write(iolog,*) '---------------------------------'
              write(iolog,*)
              write(iolog,*)'idatecur,isec,',
     &        idatecur,isec
      end if 
          
              tamb   = tambreal 
              ws     = wsreal   
              ipg    = ipgreal
              fgri   = fgr

c get meteorology for datecur
            if(MMET .eq. 1) then
                call metget(iomet,iolst,idatecur,isec,ws,wd,tamb,ipg)
            else if(MMET .eq. 2) then
                call metget_aermet(iomet,iolst,idatecur,isec,ws,wd,tamb,
     &                       ipg, MNONSEQ)
                if(MNONSEQ.eq.1) then
c ... if non sequential, then set current date to the date just read
                   idatecur =  jdatebmi
                end if 
            if(MSCREENING .eq. 1) then
                ws = real(int(ws/.5))*0.5
                tamb = real(int(tamb/1.))*1.
            end if                 
            else
               ws   = rwindsp
               if( rwinddr .ge. 0 ) then
                     wd   = rwinddr
               end if 
               tamb = rtamb ! convert to K
               ipg  = irpgclass
               zws  = zref
            end if    
            tambreal = tamb
            wsreal   = ws
            ipgreal  = ipg

         if( debug .gt. 0) then
            write(iolog,*) 'tamb=',tambreal
            write(iolog,*) 'ws=',wsreal
            write(iolog,*) 'ipg=',ipgreal
         end if             
      

c --- vessel exponential blowdown 
c repeat blowdown calculations if vessel temperature is varying           
c with ambient temperature
         if( minput.eq. 1. .and. mblowdown .eq. 1) then
            tempinitx = tempinit
            if( tempinitx .lt. 0. ) then
               tempinitx = tambreal
            end if               
            ierr= ivessel(gcraw_x,pressinit,tempinitx,pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgas2   )
              
            ierr = iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)
         end if       

         if( minput.eq. 1. .and. mblowdown .eq. 1) then
              if(MTGAS.eq.0) then
                tgas = tginit
              else if (MTGAS.eq.1) then
                tgas = tamb 
              else if (MTGAS.eq.2) then
                tgas = tgas2 ! tgas2 is calculated in iVessel()
              end if 
         else    
            if(tginit .le. 0. .or. MTGAS.eq.1) then
               tgas = tamb
            else
               tgas = tginit
            end if 
         end if 
         if( debug .gt. 0) then
             write(iolog,*) 'tgas =',tgas
         end if 
      
c         tamb = 5
c         wd = 90
c         ws = 4.137 
c         tgas = tamb

c set puff 
              iemitt = 1
              ipuffdurx = nint(pseq(ip,2))
              
c adjust puff for start of next period
              if( ip .gt. npuffs) then
                  ip = 0
                  iemitt = 0
c calculate the ipuffdurx for the gap to the next cycle                  
                  if( isplit .eq. 1) then
c                      call deltsec(ihr,isec,ih0+nsep,0,ipuffdurx)
                   call DEDAT(iblowstart,iyr1,ijul1,ihr1)
                   call incrs(iolst,iyr1,ijul1,ihr1,isec1,(nsep)*3600)
                   call MKDAT(iyr1,ijul1,ihr1,iblowend)
                   call deltsec(idatecur,isec,iblowend,0,ipuffdurx)
                  else
c --- because the sequential puffs must be less than an hour, 
c we only need to test the seconds compared to timeinc
                      call deltsec(ihr,isec,ih0,itiminc,ipuffdurx)
                      if(ipuffdurx .ge. nsep*3600) then
                          ipuffdurx  = 0
                      end if 
                  end if
                  if(ipuffdurx.lt.0) then
                     ipuffdurx = 0
                  end if 
                  iy1 = iyr
                  ij1 = ijul
                  ih1 = ihr
                  is1 = isec
                  call incrs(iolst,iy1,ij1,ih1,is1,ipuffdurx)
                  ih0 = ih1
              end if  

c if zero duration, then reset to start of next sequence         
              if( ipuffdurx .eq. 0 .and. ip.eq.0 ) then
                  iemitt = 1
                  ip = 1
                  ipuffdurx = nint(pseq(ip,2))
                  call mkDAT(iyr,ijul,ihr,iblowstart)
              end if 

c set current emission rate 
              qmax1 = 0.0 
              if( ip .ne. 0) then 
                  qmax1     = pseq(ip,1)
              end if 
              if( qmax1 .le. 0.) then
                  iemitt = 0
              end if 
              
c set puff duration based upon source input
              if( MINPUT .eq. 2 .and. iemitt.eq.1) then
c set h2s to user input
                  rh2s1 = rh2s
                  if(pseq(ip,3) .ge. 0) then
                      rh2s1 = pseq(ip,3)
                  end if                       
c set fgr to user input
                  if(pseq(ip,4) .ge. 0) then
                      fgri = pseq(ip,4)
                  end if                       
                  ierr = inormalizeh2s(gcraw, rh2s1, gcraw_x)
              end if 

c calculate source conditions
c get xL, yL, zL
c get FB, FM
c get ds_x, hs_x, ts
              if(iemitt .eq. 1) then 
                  qtotal1   = qmax1*real(ipuffdurx)/24./3600. ! volume in this hour

c --- fuel gas is added proportional to current puff rate
                  if( MFGR .eq. 1) then
                      fgr1=fgri
c --- fuel gas is added proportional to qmax initial rate
c     need to back calculate effective FGR from qmax0
                  else if( MFGR .eq. 2) then
                      fgr1=(fgri*qmax0)/qmax1
                  if( debug .gt. 0) then
            write(iolog,*) 'fgri=',fgri
            write(iolog,*) 'fgr1=',fgr1
            write(iolog,*) 'qmax0=',qmax0
            write(iolog,*) 'qmax1=',qmax1
                     end if 
                  end if 

c for a blowdown, the fgr changes with time relative to the blowdown.
c that is, fuel gas rate is fixed
c must calculate the fgr based upon current qmax and the initial fuel 
c gas flow 
c            tamb = 4.
c            tgas = tamb
c            ws = 23.641

         if( debug .gt. 0) then
            write(iolog,*) 'qmax=',qmax1
            write(iolog,*) 'fgr1=',fgr1
         end if             

                  if(MINPUT .eq. 1 .or. MINPUT.eq. 2) then  
                  ! VARIABLES input
                  !  iolog, qmax1, fgr1, qtotal1
                  !  ws, wsx, ipgreal
                  !  gcraw_x
                  !
                  ! GLOBALS
                  ! tamb
                      call emissions(iolog,gcraw_x,qmax1,fgr1,
     &  qtotal1,ws,ipgreal)
c --- Now read these from 'modinp'
c     gcfuel,gclift,
c     &  hs,df,ds,zws
c     &  MSTRIP,MFUELGAS,MFASSIST,MFASSISTSTEAM,MFASSISTAIR,MFASSISTHPFUEL,
c     &  MLIFTGAS,MFLAMEOUT,
c     &  qliftgas,stm_press,stm_temp,stm_flow,
c     &  rairsteam,air_flow,
c     &  hpfuel_flow,odourppb)
                  
c                      call emissions(iolog,gcraw_x,qmax1,gcfuel,fgr1,gclift,
c     &  qtotal1,hs,df,ds,ws,zws,ipgreal,
c     &  MSTRIP,MFUELGAS,MFASSIST,MFASSISTSTEAM,MFASSISTAIR,MFASSISTHPFUEL,
c     &  MLIFTGAS,MFLAMEOUT,
c     &  qliftgas,stm_press,stm_temp,stm_flow,
c     &  rairsteam,air_flow,
c     &  hpfuel_flow,odourppb)
     
                  else if(MINPUT .eq. 3) then  
                      qemit  = rqemit
                      extemp = psts
                      ue_x   = psus 
                      de_x   = psds
                      hs_x   = pshs
                  end if 

                  XH(1)   = 0.0
                  DELH(1) = Hs
                  XH(2)   = XL
                  DELH(2) = hs_x

                  call calcDELH(wsreal, tambreal, ipgreal, 
     &            extemp, ue_x, de_x, hs_x, xh,DELH, zws)

      if( debug .gt. 0) then
            write(iolog,*) 'qemit=',qemit
                  write(iolog,*) 'extemp=',extemp
                  write(iolog,*) 'ue_x=',ue_x
                  write(iolog,*) 'de_x=',de_x
                  write(iolog,*) 'hs_x=',hs_x
                  write(iolog,*) 'ipg=',ipgreal
                  write(iolog,*)'FBisc,FMisc',fbisc,fmisc
                  write(iolog,*) 'DELH'
                  write(iolog,*) 'Hs',XH(1),DELH(1)
                  write(iolog,*) 'XL',XH(2),DELH(2)
                  write(iolog,*) 'XF*1/3',XH(3),DELH(3)
                  write(iolog,*) 'XF*2/3',XH(4),DELH(4)
                  write(iolog,*) 'XF', XH(5),DELH(5)
      end if 
                  ts_x  = extemp
                  fmx   = 0.
                  fmy   = 0.
                  fmz   = fm
                  usw   = ue_x
                  usu = 0. 
                  usv = 0.
c --- components
c            usw = uv
c            usu = uu*sin(wd*pi/180.)
c            usv = uu*cos(wd*pi/180.)
                  wd1 = (wd-180.)
                  if(wd1 .lt. 0.) wd1 = wd1 + 360.
                  x = XL*sin(wd1*pi/180.)
                  y = XL*cos(wd1*pi/180.)
                  z = hs_x
                  elev = zcoord
      if( debug .gt. 0) then
                  write(iolog,*) 'x,y,z',x,y,z
      end if
c                  if(MUNITS .eq. 1) then
c                      x = xcoord + x
c                      y = ycoord + y
c                  else
c                      x = xcoord+ x*0.001
c                      y = ycoord + y*0.001
c                  endif 
c Convert source offset (origin 0,0) to source coordinates (units KM utm)      
                  x = xcoord + x*0.001
                  y = ycoord + y*0.001
                  q(1) = qemit
                  sy = 0.
                  sz = 0.
              else
                  hs_x  = hs
                  de_x  = df
                  ts_x  = tamb
                  usw = 0. 
                  usu = 0. 
                  usv = 0.
                  fb    = 0.
                  fmx   = 0.
                  fmy   = 0.
                  fmz   = 0.
                  x = xcoord
                  y = ycoord
                  z = hs_x
                  elev  = zcoord
                  q(1) = 0.
                  sy = 0.
                  sz = 0.
              end if

c append the static emissions to the list              
              if(nspec.gt.1) then
              do ie=2,nspec
                 q(ie) = qlst(ie)
              end do 
              end if 
c increment time
              iy1 = iyr
              ij1 = ijul
              ih1 = ihr
              is1 = isec
              call incrs(iolst,iyr,ijul,ihr,isec,ipuffdurx)
              iy2 = iyr
              ij2 = ijul
              ih2 = ihr
              is2 = isec

c write ABflare puff
            ivar = 1
            if(MDSPMOD .eq. 1) then
c write the CALPUFF file               
               ier = iwrFLEMARBi(ioab,ivar,sname,
     &        iy1,ij1,ih1,is1,
     &        iy2,ij2,ih2,is2,
     &        x,y,z,elev,de_x,ts_x,usu,usv,usw,
     &        fb,fmx,fmy,fmz,sy,sz,q,nspec)
            else
c future 
c write the variable aermod file               
c               ier = iwrFLEMARBi_aermod(ioab,ivar,sname,
c     &        iy1,ij1,ih1,is1,
c     &        iy2,ij2,ih2,is2,
c     &        x,y,z,elev,de_x,ts_x,usu,usv,usw,
c     &        fb,fmx,fmy,fmz,sy,sz,q,nspec)
            end if 

c increment time
c         call incrs(iolst,iyr,ijul,ihr,isec,1)
              ip       = ip + 1
              call MKDAT(iyr,ijul,ihr,idatecur)
c reset blowdown start time
              if(ip .eq. 1 ) then
                  call mkDAT(iyr,ijul,ihr,iblowstart)
              end if 
          end do ! (datecur .le. idateend)
c ----------------------------------------------------------------------
          close (iomet)
          close (ioab)   
      end do ! iloop=1,nloops   

      return
      end 

c ---------------------------------------------------------------------
c subroutine mkfilam()
c
c creates the series of output files for AERMOD formated variable 
c emissions file
c ---------------------------------------------------------------------
      subroutine mkfilam(io, iolst, iomet, iolog)
c --- MODULES      
      use mod_stddefs 
      use mod_globals
      use mod_readinp 
      use mod_virtualsources 
      use mod_combustion
      use mod_met 
      use mod_isccalcs
      use mod_file1
c --- INPUTS
      integer :: io
      integer :: iolst
      integer :: iomet
      integer :: iolog
c --- DECLARATIONS
      real(8) :: fgri

      real(8) :: vTs ! corrected Ts adjusted for truncated Ds(isource)
      real(8) :: vUs ! corrected Us adjusted for truncated Ds(isource)
      
      integer iloop, ip, ih0, imet
      integer ier
      character*mxpath :: FROOT
      character*mxpath :: FLOOP
      character*mxpath :: FNAME
      character*12 :: srcname
      character*12 :: srcname2
      
      nloops = npuffs
      nsep   = 1
      ipuffdur  = 3600
      ipuffdurx = 3600

c ---------------------------------------------------------------------
c ----------------------------------------------------------------------
      do iloop=1,nloops                    
c ----------------------------------------------------------------------
c     cycle through all dates and create a plume rise list

c calculate start date stamp  
         iyr  = iyys
         ijul = ijjs
         ihr  = ihhs
         isec = isss
         call MKDAT(iyr,ijul,ihr,idatecur)

c Open METDAT
         if(MMET .eq. 1) then
            call methdr(iomet,iolst,METDAT,ws,wd,tamb,ipg)  
         else if(MMET .eq. 2) then
            call methdr_aermet(iomet,iolst,METDAT,ws,wd,tamb,ipg,MNONSEQ)  
            if(MSCREENING .eq. 1) then
                ws = real(int(ws/.5))*0.5
                tamb = real(int(tamb/1.))*1.
            end if                 
         else            
            ws   = rwindsp
            if( rwinddr .ge. 0 ) then
               wd   = rwinddr
            end if 
            tamb     = rtamb
            ipg      = irpgclass
            ipgreal  = ipg
            zws      = zref
         end if 
         tambreal = tamb
         wsreal   = ws
         ipgreal  = ipg
         imet     = 0 
c ----------------------------------------------------------------------
         do while (idatecur .le. idateend)

          write(6,8001) iloop,idatecur
8001      format('+',I4,1x,I10)       

         if( debug .gt. 0) then
            write(iolog,*) '---------------------------------'
            write(iolog,*) 'iloop: ',iloop, imet
            write(iolog,*) '---------------------------------'
            write(iolog,*) idatecur
         end if           
            tamb   = tambreal 
            ws     = wsreal   
            ipg    = ipgreal
            fgri   = fgr

c get meteorology for datecur
            if(MMET .eq. 1) then
                call metget(iomet,iolst,idatecur,isec,ws,wd,tamb,ipg)
            else if(MMET .eq. 2) then
c ... if MNONSEQ and its the first pass, then idatecur.eq.jdatebmi
c     Then don't read met since it is current                 
               if(MNONSEQ.eq.0 .or. idatecur.ne.jdatebmi) then
                call metget_aermet(iomet,iolst,idatecur,isec,ws,wd,
     &             tamb,ipg,MNONSEQ)
               end if 
                if(MNONSEQ.eq.1) then
                   call dedat(idatecur ,iyr,ijul,ihr)
c ... check for end of file for a MNONSEQ                   
                   if(jdatebm.eq.jdateem) then
                      idateend = idatecur
                   end if 
                   
c          write(6,8002) iloop,idatecur,idateend,imet,jdatebmi,jdateemi
c8002      format('',I4,1x,I10,1x,I10,1x,I10,1x,I10,1x,I10)
          
                end if                 
      if(MSCREENING .eq. 1) then
                ws = real(int(ws/.5))*0.5
                tamb = real(int(tamb/1.))*1.
      end if                 
            else
               ws   = rwindsp
               if( rwinddr .ge. 0 ) then
                     wd   = rwinddr
               end if 
               tamb = rtamb ! convert user entry from C to K
               ipg  = irpgclass
               ipgreal  = ipg
               zws  = zref
            end if    
            tambreal = tamb
            wsreal   = ws
            ipgreal  = ipg

         if( debug .gt. 0) then
            write(iolog,*) 'tamb=',tambreal
            write(iolog,*) 'ws=',wsreal
            write(iolog,*) 'ipg=',ipgreal
         end if             
         
            
c --- vessel exponential blowdown 
c repeat blowdown calculations if vessel temperature is varying           
c with ambient temperature
         if( minput.eq. 1. .and. mblowdown .eq. 1) then
            tempinitx = tempinit
            if( tempinitx .lt. 0. ) then
               tempinitx = tambreal
            end if               
            ierr= ivessel(gcraw_x,pressinit,tempinitx,
     &                 pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgas2   )
            ierr = iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)
         end if       

         if( minput.eq. 1. .and. mblowdown .eq. 1) then
            if(MTGAS.eq.0) then
               tgas = tginit
            else if (MTGAS.eq.1) then
               tgas = tamb 
            else if (MTGAS.eq.2) then
               tgas = tgas2 ! tgas2 is calculated in iVessel()
            end if 
         else    
            if(tginit .le. 0. .or. MTGAS.eq.1) then
               tgas = tamb
            else
               tgas = tginit
            end if 
         end if 
          
c         tamb = 5
c         wd = 90
c         ws = 4.137 
c         tgas = tamb

c set puff 
         ip       = iloop
         iemitt   = 1
         qmax1    = pseq(ip,1)
         
c set puff duration based upon source input
c         ipuffdurx = nint(pseq(ip,2))
         ipuffdurx = 3600

         if( MINPUT .eq. 2 ) then
c set h2s to user input
            rh2s1 = rh2s
            if(pseq(ip,3) .ge. 0) then
               rh2s1 = pseq(ip,3)
            end if                       
c set fgr to user input
            if(pseq(ip,4) .ge. 0) then
               fgri = pseq(ip,4)
            end if                       
            ierr = inormalizeh2s(gcraw, rh2s1, gcraw_x)
         end if 

c calculate source conditions
c get xL, yL, zL
c get FB, FM
c get ds_x, hs_x, ts
         qtotal1   = qmax1*real(ipuffdurx)/24./3600. ! Volume in this hour

c --- fuel gas is added proportional to current puff rate
         if( MFGR .eq. 1) then
               fgr1=fgri
c --- fuel gas is added proportional to qmax initial rate
c     need to back calculate effective FGR from qmax0
         else if( MFGR .eq. 2) then
               fgr1=(fgri*qmax0)/qmax1
c            write(iolog,*) 'fgri=',fgri
c            write(iolog,*) 'fgr1=',fgr1
c            write(iolog,*) 'qmax0=',qmax0
c            write(iolog,*) 'qmax1=',qmax1
         end if 

c for a blowdown, the fgr changes with time relative to the blowdown.
c that is, fuel gas rate is fixed
c must calculate the fgr based upon current qmax and the initial fuel 
c gas flow 
c            tamb = 4.
c            tgas = tamb
c            ws = 23.641

         if( debug .gt. 0) then
            write(iolog,*) 'qmax=',qmax1
            write(iolog,*) 'fgri=',fgr1
         end if             

         if(MINPUT .eq. 1 .or. MINPUT.eq. 2) then  

               call emissions(iolog,gcraw_x,qmax1,fgr1,
     &  qtotal1,ws,ipgreal)
         
c               call emissions(iolog,gcraw_x,qmax1,gcfuel,fgr1,gclift,
c     &  qtotal1,hs,df,ds,ws,zws,ipgreal,
c     &  MSTRIP,MFUELGAS,MFASSIST,MFASSISTSTEAM,MFASSISTAIR,MFASSISTHPFUEL,
c     &  MLIFTGAS,MFLAMEOUT,
c     &  qliftgas,stm_dia,stm_press,stm_temp,stm_flow,
c     &  stm_nports,rairsteam,air_dia,air_flow,air_nports,
c     &  hpfuel_dia,hpfuel_flow,hpfuel_nports,odourppb)
               
         else if(MINPUT .eq. 3) then  
            qemit  = rqemit
            extemp = psts
            ue_x   = psus 
            de_x   = psds
            hs_x   = pshs
         end if 

         XH(1)   = 0.0
         DELH(1) = Hs
         XH(2)   = XL
         DELH(2) = hs_x

            call calcDELH(wsreal, tambreal, ipgreal, 
     &            extemp, ue_x, de_x, hs_x, xh,DELH, zws)

         if( debug .gt. 0) then
            write(iolog,*) 'qemit=',qemit
            write(iolog,*) 'extemp=',extemp
            write(iolog,*) 'ue_x=',ue_x
            write(iolog,*) 'de_x=',de_x
            write(iolog,*) 'hs_x=',hs_x
            write(iolog,*) 'ipg=',ipgreal
            write(iolog,*)'FBisc,FMisc',fbisc,fmisc
            write(iolog,*) 'DELH'
            write(iolog,*) 'Hs',XH(1),DELH(1)
            write(iolog,*) 'XL',XH(2),DELH(2)
            write(iolog,*) 'XF*1/3',XH(3),DELH(3)
            write(iolog,*) 'XF*2/3',XH(4),DELH(4)
            write(iolog,*) 'XF', XH(5),DELH(5)
         end if 
         ts_x  = extemp
         fmx   = 0.
         fmy   = 0.
         fmz   = fm
         usw   = ue_x
         usu = 0. 
         usv = 0.
c --- components
c            usw = uv
c            usu = uu*sin(wd*pi/180.)
c            usv = uu*cos(wd*pi/180.)
         wd1 = (wd-180.)
         if(wd1 .lt. 0.) wd1 = wd1 + 360.
         x = XL*sin(wd1*pi/180.)
         y = XL*cos(wd1*pi/180.)
         z = hs_x
         elev = zcoord
         
c Convert source offset (origin 0,0) to source coordinates (units KM utm)      
         if(MSCREENING .eq. 1) then
            x = xcoord 
            y = ycoord
         else             
            x = xcoord + x*0.001
            y = ycoord + y*0.001
         end if 
         q(1) = qemit
         sy = 0.
         sz = 0.

c Save the information for later indexing         
         imet = imet + 1
         indx(imet) = imet
c V170324
c changed to use effective height instead of rise height         
c         vrise(imet) = DELH(5)
         vrise(imet) = hs_x 
         vdex(imet) = de_x
         vhsx(imet) = hs_x
         vusx(imet) = ue_x
         vtsx(imet) = extemp
         ve(imet) = qemit
         vx(imet) = x
         vy(imet) = y
         vfb(imet) = fbisc
         vfm(imet) = fmisc
         vta(imet) = tamb
         vua(imet) = ua
         vdate(imet) = idatecur
         vsec(imet) = isec
         
c increment time
         iy1 = iyr
         ij1 = ijul
         ih1 = ihr
         is1 = isec
         call incrs(iolst,iyr,ijul,ihr,isec,ipuffdurx)
         iy2 = iyr
         ij2 = ijul
         ih2 = ihr
         is2 = isec

         ip       = ip + 1
         call MKDAT(iyr,ijul,ihr,idatecur)
      end do ! (datecur .le. idateend)
c ----------------------------------------------------------------------
      close (iomet)

c =======================================
c determine virtual sources
      nmet = imet   
c Determine final rise heights limits
      Call HSORTrL(nmet, vrise, indx)

c determine average for each for the virtual sources
      do i=1,NSOURCES
         vsTs(i) = 0.
         vsUs(i) = 0.
c changed to use minimum height
c V170324         
c         vsHs(i) = 0.
         vsHs(i) = 1.e30
         vsDs(i) = 0.
         vsX(i) = 0.
         vsY(i) = 0.
         ivsNs(i) = 0
      end do    
      do imet = 1,nmet
         ix = indx(imet)
         j = 1 + int(real(imet)/(real(nmet)/ real(NSOURCES)))
         j = min(j,nsources)
         ivsNs(j) = ivsNs(j) + 1
         vsTs(j) = vsTs(j) + vtsx(ix)
         vsUs(j) = vsUs(j) + vusx(ix)
         vsDs(j) = vsDs(j) + vdex(ix)
c         vsHs(j) = vsHs(j) + vhsx(ix)
c changed to use minimum height
c V170324         
         vsHs(j) = min(vsHs(j),vhsx(ix))
         vsX(j) = vsX(j) + vx(ix)
         vsY(j) = vsY(j) + vy(ix)
         ivs(ix) = j 
      end do
      do i=1,NSOURCES
         If( ivsNs(i) .gt. 0 ) Then 
            vsTs(i) = vsTs(i) / ivsNs(i)
            vsUs(i) = vsUs(i) / ivsNs(i)
            vsDs(i) = vsDs(i) / ivsNs(i)
c changed to use minimum height
c V170324         
c            vsHs(i) = vsHs(i) / ivsNs(i)
            vsX(i) = vsX(i) / ivsNs(i)
            vsY(i) = vsY(i) / ivsNs(i)
            if( debug .gt. 0) then
               write(iolog,*) 'VS',i,vsTs(i),vsUs(i),vsds(i),vsHs(i),
     &         vsX(i), vsY(i)        
            end if    
         end if             
      end do    
      
      
c =======================================
c  Create AERMOD .inp file for this loop
c create output path for this loop
      write(FROOT,"(A,'_','MOD','_',I2.2,'.INX')")
     &      OUTNAME(1:LEN_REAL(OUTNAME)),iloop
      call concat(Fname,OUTPATH,FROOT)
      call mk_case(iolst, lcfiles, fname, ierr)

      write(FROOT,"(A,'_','MOD','_',I2.2,'.DAT')")
     &        OUTNAME(1:LEN_REAL(OUTNAME)),iloop
      call concat(FLOOP,OUTPATH,FROOT)
      call mk_case(iolst,lcfiles, FLOOP, ierr)
         
      call mkfilam_inp(io, iolst, fname, floop)
      
c ----------------------------------------------------------------------
c create output path for this loop
c write ABflare.dat header, open a unit
      if( MDSPMOD.eq.1) then
            write(iolst,*) 'Probable bad combination '
            write(6,*) 'Probable bad combination '
            stop
      else if( MDSPMOD.eq.2) then
c            ier = iwrFLEMARBhdr_aermod(ioab, iolst, FLOOP)
      endif
      
      if(DEBUG.gt.0) then
          write(iolog,*) '---------------------------------------------'
          write(iolog,*) 'loop=',iloop
          write(iolog,*) FLOOP
      end if                     
      
      call mkfilam_emarb(io, iolst, FLOOP)    
c =======================================

      
      end do ! iloop=1,nloops      
c ----------------------------------------------------------------------
c ----------------------------------------------------------------------
      
      
      return
      end 

c ----------------------------------------------------------------------
c
c Uses 
c     METDAT
c     METDAT2
c     RECINC
c     MSTRIP
c     mmodel, mversion, mlevel
c     NSOURCES    ! number of virtual sources
c     srcname     ! virtual source name/label
c     vsx(i)      ! virtual source location, x (m)
c     vsy(i)      ! virtual source location, y (m)
c     vshs(i)     ! virtual source height (m)
c     vsts(i)     ! virtual source temperature (K)
c     vsus(i)     ! virtual source vertical velocity (m/s)
c     vsds(i)     ! virtual source diameter (m)
c     zcoord      ! elevation of source
c
c ----------------------------------------------------------------------
      subroutine mkfilam_inp(ioab, iolst, fname, femarb)
c ----------------------------------------------------------------------
c MODULES
      use mod_progver
      use mod_stddefs
      use mod_globals
      use mod_readinp
      use mod_virtualsources
c inputs 
      integer :: ioab ! io control id
      character*mxpath :: fname 
      character*mxpath :: femarb
      
c declarations
      integer :: iyr  ! assessment year
      character*12 :: srcname
      character*12 :: srcname2
      character*mxpath :: FROOT
      integer :: ierr, i,n, n1, n2
      real(8) :: e1        ! virtual source emission (assigned to zero)

      iyr = iyys
      
      open(ioab,file=fname,iostat=ierr,status='unknown')

      write(ioab,"('CO STARTING')")
      write(ioab,
     &  "('CO TITLEONE flemarb by ',A,' Version ',A,' Level ',A)") 
     &           mmodel,mversion,mlevel
      if(MSTRIP.eq.0) then
         write(ioab,"('CO TITLETWO SO2 EMISSIONS')")
      elseif(MSTRIP.eq.1) then         
         write(ioab,"('CO TITLETWO H2S EMISSIONS')")
      elseif(MSTRIP.eq.2) then         
         write(ioab,"('CO TITLETWO SO2 EMISSIONS')")
      elseif(MSTRIP.eq.3) then         
         write(ioab,"('CO TITLETWO H2S EMISSIONS')")
      elseif(MSTRIP.eq.4) then         
         write(ioab,"('CO TITLETWO OU EMISSIONS')")
      elseif(MSTRIP.eq.5) then         
         write(ioab,"('CO TITLETWO OU EMISSIONS')")
      end if          
      if(MSCREEN.eq.1) then
         write(ioab,"('CO MODELOPT NOCHKD CONC NOSTD ELEV SCREEN')")
      else
         write(ioab,"('CO MODELOPT NOCHKD CONC NOSTD ELEV')")
      end if 
      write(ioab,"('CO AVERTIME 1')")
      if(MSTRIP.eq.0) then
         write(ioab,"('CO POLLUTID SO2')")
      elseif(MSTRIP.eq.1) then
         write(ioab,"('CO POLLUTID H2S')")
      elseif(MSTRIP.eq.2) then
         write(ioab,"('CO POLLUTID SO2')")
      elseif(MSTRIP.eq.3) then
         write(ioab,"('CO POLLUTID H2S')")
      elseif(MSTRIP.eq.4) then
         write(ioab,"('CO POLLUTID OU')")
      elseif(MSTRIP.eq.5) then
         write(ioab,"('CO POLLUTID OU')")
      end if          
      write(ioab,"('CO RUNORNOT RUN')")
      write(ioab,"('CO ERRORFIL AERMOD.err')")
      write(ioab,"('CO FINISHED')")
      
      write(ioab,"('SO STARTING')")
      write(ioab,"('SO ELEVUNIT METERS')")
c for H2S and SO2      
      if(MSTRIP.eq.0 .or. MSTRIP.eq.1 .or. MSTRIP.eq.2 .or. MSTRIP.eq.3) then
      write(ioab,"('SO EMISUNIT    .100000E+07   (GRAMS/SEC)          (MICROGRAMS/CUBIC-METER)')")
      elseif(MSTRIP.eq.4 .or. MSTRIP.eq.5 ) then
c for OU         
      write(ioab,"('SO EMISUNIT    .100000E+04   (GRAMS/SEC)          (MILLIGRAMS/CUBIC-METER)')")
      end if
      write(ioab,"('** SRCPARAM SOURCE1  QS    HS    TS    US    DS')")

      do i=1,NSOURCES
         if(i .lt. 10) then
            write(srcname,"('SOURCE',i1)") i
         else
            write(srcname,"('SOURCE',i2)") i
         end if 
         write(ioab,"('SO LOCATION ',A,' POINT',2(F13.1,1x),F13.1)")
     &               srcname,vsx(i)*1000.,vsy(i)*1000., zcoord
         e1 = 0.
         write(ioab,"('SO SRCPARAM ',A, 5(F10.3,1x))")
     &               srcname,e1,vshs(i),vsts(i),vsus(i),vsds(i)
      end do
c --- output file
      n=LEN_REAL(femarb)
      i=1
         if(i .lt. 10) then
            write(srcname,"('SOURCE',i1)") i
         else
            write(srcname,"('SOURCE',i2)") i
         end if 
      i=NSOURCES         
         if(i .lt. 10) then
            write(srcname2,"('SOURCE',i1)") i
         else
            write(srcname2,"('SOURCE',i2)") i
         end if 
      n1=LEN_REAL(srcname)
      n2=LEN_REAL(srcname2)
      write(ioab,"('SO HOUREMIS ',A,' ',A,'-',A)") 
     &       femarb(1:n),srcname(1:n1),srcname2(1:n2)
      write(ioab,"('SO SRCGROUP  ALL')")
      write(ioab,"('SO FINISHED')")
      
      write(ioab,"('RE STARTING')")
      n=LEN_REAL(RECINC)
      write(ioab,"('RE INCLUDED ',A)") RECINC(1:n)
      write(ioab,"('RE FINISHED')")
      
      write(ioab,"('ME STARTING')")
      n=LEN_REAL(METDAT)
      write(ioab,"('ME SURFFILE ',A)") METDAT(1:n)
      n=LEN_REAL(METDAT2)
      write(ioab,"('ME PROFFILE ',A)") METDAT2(1:n)
      write(ioab,"('ME SURFDATA  00011111 ',i4,' ONSITE')") iYr
      write(ioab,"('ME UAIRDATA  00022222 ',i4,' ONSITE')") iYr
      write(ioab,"('** set at plant site elevation')")
      write(ioab,"('ME PROFBASE ',f10.1,' METERS')"), zcoord
      write(ioab,"('ME FINISHED')")
      
      write(ioab,"('OU STARTING')")
      write(ioab,"('OU RECTABLE ALLAVE FIRST')")
      write(ioab,"('OU MAXTABLE ALLAVE 10')")
      write(ioab,"('OU PLOTFILE 1 ALL FIRST ',A)") 'aermod.dat'
      write(ioab,"('OU POSTFILE 1 ALL UNFORM ',A)") 'aermod.bin'
      write(ioab,"('OU FINISHED')")

      close(ioab)
      
      return
      end 
      

c ----------------------------------------------------------------------

c ----------------------------------------------------------------------
      subroutine mkfilam_emarb(ioab,iolst, fname)
c ----------------------------------------------------------------------
c MODULES
      use mod_stddefs
      use mod_globals
      use mod_readinp
      use mod_file1 
      use mod_virtualsources
c      implicit none
c inputs 
      integer :: ioab ! io control id
      integer :: iolst
      character*mxpath :: fname 
      
c declarations
      character*12 :: srcname
      character*mxpath :: FROOT
      integer :: ierr, i,n, imet
      real(8) :: e1        ! virtual source emission (assigned to zero)
      integer :: iyr, ijul, ihr, isec, iy1, ij1, ih1, is1, iy2, ij2, ih2, is2
      integer :: iyr1, imo1, iday1
      integer :: iyout
      real(8) :: term1
      real(8) :: vTs ! corrected Ts adjusted for truncated Ds(isource)
      real(8) :: vUs ! corrected Us adjusted for truncated Ds(isource)
      
      
      open(ioab,file=fname,iostat=ierr,status='unknown')
c calculae start date stamp  
       iyr  = iyys
       ijul = ijjs
       ihr  = ihhs
       isec = isss
       call MKDAT(iyr,ijul,ihr,idatecur)
          
c ----------------------------------------------------------------------
c          imet = 0
c          do while (idatecur .le. idateend)
          do imet=1,nmet
c          write(6,8002) iloop,idatecur
c8002  format('+',I4,1x,I10)               
      
c increment time
c            iy1 = iyr
c            ij1 = ijul
c            ih1 = ihr
c            is1 = isec
c            call incrs(iolst,iyr,ijul,ihr,isec,ipuffdurx)
c            iy2 = iyr
c            ij2 = ijul
c            ih2 = ihr
c            is2 = isec
            call dedat(vdate(imet),iy1,ij1,ih1)
            isc1 = vsec(imet)

            call grday(iolst,iy1,ij1, imo1,iday1)
            
c         imet = imet + 1   
         do i=1,NSOURCES
            e1 =  ve(imet)
            if(ivs(imet) .ne. i) then
               e1 = 0.
            end if 
            if(i .lt. 10) then
               write(srcname,"('SOURCE',i1)") i
            else
               write(srcname,"('SOURCE',i2)") i
            end if
c adjust year to two digits            
            if(iy1.gt.2000) then
               iyout = iy1 -2000
            else if (iy1.gt.1900) then
               iyout = iy1 -1900
            else
               iyout = iy1
            end if 
c Calculate adjusted Us and Ts so the the FB andFM are correct
c when using the forced values of Ds            
            if(ivs(imet) .eq. i ) then
               term1 = vfb(iMet) / (Grav * vsDs(i))
               vUs = 2.0 / vsDs(i) * (term1 + SQRT(term1 * term1 + vfm(iMet)))
               vTs = vta(iMet) * (vfb(iMet) * vUs / (vfm(iMet) * Grav) + 1.0)
            else
               vUs = vusx(imet)
               vTs = vtsx(imet)
            end if 
            
            write(ioab,8003) 'SO HOUREMIS',
     &            iyout,imo1,iday1,(ih1+1),srcname,
     &            e1,vTs,vUs
8003     format(A,4(1x,i3),1x,A12,3(ES12.6,1x))      
         end do
         
c increment time
c         call incrs(iolst,iyr,ijul,ihr,isec,1)
c              ip       = ip + 1
              call MKDAT(iyr,ijul,ihr,idatecur)
          end do ! (datecur .le. idateend)
c ----------------------------------------------------------------------
          close (ioab)   
          
      return
      end 
      

      
c ----------------------------------------------------------------------
c sort RA and index IB      
c ----------------------------------------------------------------------
      subroutine HSORTrL(n, RA, IB)
c ----------------------------------------------------------------------
c ... INPUTS
      integer :: n, IB(*)
      real(8) :: RA(*)
c ... DECLARATIONS
      integer :: L 
      integer :: ir
      integer :: i
      integer :: j, IIB
      real(8) :: RRA
      L = n / 2 + 1
      ir = n
10    CONTINUE
      If (L.gt.1) Then
         L = L - 1
         RRA = RA(L)
         IIB = IB(L)
      Else
         RRA = RA(ir)
         IIB = IB(ir)
         RA(ir) = RA(1)
         IB(ir) = IB(1)
         ir = ir - 1
         If (ir.eq.1) Then
            RA(1) = RRA
            IB(1) = IIB
            return
         End If
      End If
      i = L
      j = L + L
20    continue
      If (j .le.ir) Then
         If (j .lt. ir) Then
            If (RA(j) .lt. RA(j + 1)) Then 
               j = j + 1
            end if    
         End If
         If (RRA .lt. RA(j)) Then
            RA(i) = RA(j)
            IB(i) = IB(j)
            i = j
            j = j + j
         Else
            j = ir + 1
         End If
         GoTo 20
      End If
      RA(i) = RRA
      IB(i) = IIB
      GoTo 10
      return
      end 

c ----------------------------------------------------------------------
c HEAP SORT ON FIVE R and one I ARRAYS
c ----------------------------------------------------------------------
      subroutine HSORT5rL(n, RA, RB, RC, RD, RE, IB)
c ... INPUTS
      integer :: n, IB(*)
      real(8) :: RA(*), RB(*), RC(*), RD(*), RE(*) 
c ... DECLARATIONS
      integer :: L 
      integer :: ir
      integer :: i
      integer :: j, IIB
      real(8) :: RRA, RRB, RRC, RRD, RRE
      L = n / 2 + 1
      ir = n
10    CONTINUE
      If (L.gt.1) Then
         L = L - 1
         RRA = RA(L)
         RRB = RB(L)
         RRC = RC(L)
         RRD = RD(L)
         RRE = RE(L)
         IIB = IB(L)
      Else
         RRA = RA(ir)
         RRB = RB(ir)
         RRC = RC(ir)
         RRD = RD(ir)
         RRE = RE(ir)
         IIB = IB(ir)
         RA(ir) = RA(1)
         RB(ir) = RB(1)
         RC(ir) = RC(1)
         RD(ir) = RD(1)
         RE(ir) = RE(1)
         IB(ir) = IB(1)
         ir = ir - 1
         If (ir.eq.1) Then
            RA(1) = RRA
            RB(1) = RRB
            RC(1) = RRC
            RD(1) = RRD
            RE(1) = RRE
            IB(1) = IIB
            return
         End If
      End If
      i = L
      j = L + L
20    continue
      If (j .le.ir) Then
         If (j .lt. ir) Then
            If (RA(j) .lt. RA(j + 1)) Then 
               j = j + 1
            end if    
         End If
         If (RRA .lt. RA(j)) Then
            RA(i) = RA(j)
            RB(i) = RB(j)
            RC(i) = RC(j)
            RD(i) = RD(j)
            RE(i) = RE(j)
            IB(i) = IB(j)
            i = j
            j = j + j
         Else
            j = ir + 1
         End If
         GoTo 20
      End If
      RA(i) = RRA
      RB(i) = RRB
      RC(i) = RRC
      RD(i) = RRD
      RE(i) = RRE
      IB(i) = IIB
      GoTo 10
      return
      end 

      

c----------------------------------------------------------------------
c irdcontrol
c----------------------------------------------------------------------
c --- Developed by B.Zelt - www.zeltpsi.com
c
c     Reads the control file and parses the lines into keywords
c     and user inputs
c
c UPDATES
c ---   Version: 1.002    Level: 121112
c     - added lift gas
c     - added aermod capacity
c
c ---   Version: 1.001    Level: 110706
c     
c INPUTS
c     FIN     : char(128) : path of the file to read
c
c OUTPUT
c     ierr: integer   : 0  no errors found
c                     : 1  errors were found
c
c----------------------------------------------------------------------
      function irdcontrol(io, FIN )
     &result(ierr)

c   modules
      use mod_stddefs
      use mod_readinp 
      use mod_globals
      
c    INPUTS
      integer          :: io
      integer           :: ierr
      character*mxpath  :: FIN    ! path for file input
c    INPUTs Declarations
      integer        :: iversion   ! version of the input file
      character*128 :: dummy
      character*16  :: imodel
      character*16  :: specname
      integer       :: ier
      
      ierr = 1   
      ier = 0
      open(unit=io,file=FIN,iostat=ier,status='old')
      if(ier .ne. 0) goto 9000

      read(io,*) imodel,iversion

c      write(io,*) ' Model:',mmodel,' Version:',mversion,' Level:',mlevel
      ier = irdparam_a(io, 'STITLE1', stitle)
      ier = irdparam_a(io, 'SNAME', sname)
      ier = irdparam_a(io, 'OUTNAME', OUTNAME)
      ier = irdparam_a(io, 'OUTPATH', OUTPATH)
      ier = irdparam_a(io, 'METDAT', METDAT)
      ier = irdparam_a(io, 'LSTDAT', LSTDAT)
      ier = irdparam_a(io, 'LOGDAT', LOGDAT)
      ier = irdparam_a(io, 'LCFILES', dummy)
      lcfiles = .FALSE.
      if( dummy(1:1) .eq. 't' .or. dummy(1:1) .eq. 'T' ) then
          lcfiles = .TRUE.
      end if   

      ier = irdparam_i(io, 'MDSPMOD', MDSPMOD)
      ier = irdparam_i(io, 'MINPUT', MINPUT)
      ier = irdparam_i(io, 'MBLOWDOWN',   mblowdown )  ! 0:steady, 1:vessel, 2: Qmax/Vol, 3:Qave,50% 15min, 4: Qmax, 50% 15min
      ier = irdparam_i(io, 'MDIST',       mdist )  ! 1:equal duration; 2:equal mass
      ier = irdparam_i(io, 'MSTRIP', MSTRIP)
      ier = irdparam_i(io, 'MMET', MMET)
      ier = irdparam_i(io, 'MFGR', MFGR)
      ier = irdparam_i(io, 'MTGAS', MTGAS)
      ier = irdparam_i(io, 'MDEBUG', MDEBUG)
      ier = irdparam_i(io, 'NSEP', NSEP)  
      ier = irdparam_i(io, 'NSOURCES', NSOURCES)  
      ier = irdparam_i(io, 'MFLAMEOUT', MFLAMEOUT)
      ier = irdparam_i(io, 'MGROUNDFLARE', MGROUNDFLARE)
      ier = irdparam_i(io, 'MDLIFT', MDLIFT)
      ier = irdparam_i(io, 'MDFUEL', MDFUEL)

c read second met file for AERMOD files      
      if(MMET.eq.2) then
      ier = irdparam_a(io, 'METDAT2', METDAT2)
      ier = irdparam_a(io, 'RECINC', RECINC)
      ier = irdparam_i(io, 'MSCREENING', MSCREENING)
      ier = irdparam_i(io, 'MNONSEQ', MNONSEQ)
      ier = irdparam_i(io, 'MSCREEN', MSCREEN)
      end if 
      
      ier = irdparam_a(io, 'UTMHEM', UTMHEM)
      ier = irdparam_i(io, 'IUTMZN', IUTMZN)
      ier = irdparam_a(io, 'PMAP', PMAP)
      ier = irdparam_a(io, 'DATUM', DATUM)
c      ier = irdparam_a(io, 'UNITS', UNITS)
      ier = irdparam_a(io, 'ABTZ', ABTZ)
      ier = irdparam_i(io, 'NSPEC', nspec)
      ier = irdparam_av(io, 'CHEMLST', nspec, chemlst)
      ier = irdparam_rv(io, 'MW', nspec, mmlst)
      ier = irdparam_rv(io, 'EMIT', nspec, qlst)
      ier = irdparam_r(io, 'ODOURPPB', odourppb)

c      write(6,*) 'title=[',setitle,']'
c      ier = irdparam_i(io, 'idspl ',idspl  )
      ier = irdparam_i(io, 'YYS',     iyys  )
      ier = irdparam_i(io, 'MMS',     imms  )
      ier = irdparam_i(io, 'DDS',     idds  )
      ier = irdparam_i(io, 'HHS',     ihhs  )
      ier = irdparam_i(io, 'SSS',     isss  )
      ier = irdparam_i(io, 'YYE',     iyye  )
      ier = irdparam_i(io, 'MME',     imme  )
      ier = irdparam_i(io, 'DDE',     idde  )
      ier = irdparam_i(io, 'HHE',     ihhe  )
      ier = irdparam_i(io, 'SSE',     isse  )
      ier = irdparam_i(io, 'TIMEINC',  itiminc  )
      ier = irdparam_r(io, 'XCOORD',     xcoord   ) 
      ier = irdparam_r(io, 'YCOORD',     ycoord   )
      ier = irdparam_r(io, 'ZCOORD',     zcoord   )
      ier = irdparam_r(io, 'HS  ',     hs   )
      ier = irdparam_r(io, 'DF  ',     df  )
      ier = irdparam_r(io, 'DS  ',     ds  )
      ier = irdparam_r(io, 'RH2S',     rh2s )
      ier = irdparam_r(io, 'TGINIT',   tginit )

      if (MGROUNDFLARE .eq. 1) then
         ier = irdparam_i(io, 'NHEADS  ',     nheads  )
         if(ier .eq.0 ) then
      write(6,*)      '***FATAL ERROR*** Missing NHEADS with MGROUNDFLARE=1'
               stop
         endif 
      end if           
      
      if (MDLIFT .eq. 1) then
         ier = irdparam_r(io, 'DLIFT  ',     dlift  )
         if(ier .eq.0 ) then
      write(6,*)      '***FATAL ERROR*** Missing DLIFT with MDLIFT=1'
               stop
         endif 
      end if           

      if (MDFUEL .eq. 1) then
         ier = irdparam_r(io, 'DFUEL  ',     dfuel  )
         if(ier .eq.0 ) then
      write(6,*)      '***FATAL ERROR*** Missing DFUEL with MDFUEL=1'
               stop
         endif 
      end if           

c --- determine source configuration

***      ier = irdparam_r(io, 'frel',     frel )
***      ier = irdparam_r(io, 'duration', duration )

      if(MINPUT .eq. 1 ) then
          if( mblowdown .eq. 0) then
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'TEMPINIT',    tempinit )
              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              npuffs = 1
          else if( mblowdown .eq. 1 ) then         
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'PRESSINIT',       pressinit )
              ier = irdparam_r(io, 'TEMPINIT',        tempinit )
              ier = irdparam_r(io, 'PRESSFINAL',      pressfinal )
              ier = irdparam_r(io, 'VESSELDIA',       vesseldia )
              ier = irdparam_r(io, 'VESSELLEN',       vessellen )
              ier = irdparam_r(io, 'ORIFICEDIA',      orificedia )
              ier = irdparam_r(io, 'DISCHARGECOEF',   dischargecoef )
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
          else if( mblowdown .eq. 2) then
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'QTOTAL',   qtotal )
              ier = irdparam_r(io, 'FREL',     frel )
              ier = irdparam_r(io, 'TEMPINIT',    tempinit )
              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
          else if( mblowdown .eq. 3) then
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'FREL',     frel )
              ier = irdparam_r(io, 'TEMPINIT',        tempinit )
              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
              ier = irdparam_r(io, 'PRATIO',     pratio )
              ier = irdparam_r(io, 'TDEP',     tdep )
          else if( mblowdown .eq. 4) then
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'FREL',     frel )
              ier = irdparam_r(io, 'TEMPINIT',        tempinit )
              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
              ier = irdparam_r(io, 'PRATIO',     pratio )
              ier = irdparam_r(io, 'TDEP',     tdep )
          else if( mblowdown .eq. 5) then
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'FREL',     frel )
              ier = irdparam_r(io, 'TEMPINIT',        tempinit )
              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
              ier = irdparam_r(io, 'PRATIO',     pratio )
              ier = irdparam_r(io, 'TDEP',     tdep )          
          end if 
      else if( MINPUT .eq. 2 ) then
c first value is flowrate, second is duration, 3rd is h2s
          ier = irdparam_i(io, 'NSEQ', nseq )
          ier = irdparam_rv2(io, 'SEQ', nseq, 4, pseq, mxpuff, 4 )
          npuffs = nseq
      else if( MINPUT .eq. 3 ) then
          npuffs    = 1
          ipuffdur = 3600
c          ier = irdparam_i(io, 'puffdur',     ipuffdur )
          ier = irdparam_r(io, 'PSHS', pshs )
          ier = irdparam_r(io, 'PSDS', psds )
          ier = irdparam_r(io, 'PSTS', psts )
          ier = irdparam_r(io, 'PSUS', psus )
c          ier = irdparam_r(io, 'QEMIT', rqemit )
          rqemit = qlst(1)
      end if 

c --- Fuel gas      
      ier = irdparam_i(io, 'MFUELGAS',     mfuelgas )
      if(mfuelgas .eq. 0) then
          fgr = 0.
      else
          ier = irdparam_r(io, 'FGR',      fgr )
      end if           

c --- lift gas      
      ier = irdparam_i(io, 'MLIFTGAS',     mliftgas )
      if(mliftgas.eq. 0) then
          qliftgas = 0.
      else
          ier = irdparam_r(io, 'QLIFTGAS',      qliftgas )  ! e3m3/d
      end if           

      
c need QMAX for MFGR=1          
      if (MFGR .eq. 1) then
         ier = irdparam_r(io, 'QMAX',     qmax )
         if(ier .eq.0 ) then
      write(6,*)      '***FATAL ERROR*** Missing QMAX with MFGR=1'
               stop
         endif 
      end if           
      if (MFGR .eq. 3) then
          ier = irdparam_r(io, 'QFUELGAS',      qfuelgas )  ! e3m3/d
         if(ier .eq.0 ) then
      write(6,*)      '***FATAL ERROR*** Missing QFUELGAS with MFGR=3'
               stop
         endif 
      end if 
      
      
      ier = irdparam_i(io, 'MFASSIST',       MFASSIST )
      ier = irdparam_i(io, 'MFASSISTSTEAM',  MFASSISTSTEAM )
      ier = irdparam_i(io, 'MFASSISTAIR',    MFASSISTAIR )
      ier = irdparam_i(io, 'MFASSISTHPFUEL', MFASSISTHPFUEL )
c --- group FLARE ASSIST
c     initialize
      stm_press 	  = -1. 
      stm_temp 	  = -1. 
      stm_flow 	  = 0. 
      stm_imassvol  = 0 
      air_flow 	   = 0. 
      hpfuel_flow 	= 0. 
c --- steam assist
      if( MFASSIST .eq. 1 .AND. MFASSISTSTEAM .eq. 1) then
          ier = irdparam_i(io, 'STM_IMASSVOL',  stm_imassvol )
          ier = irdparam_r(io, 'STM_PRESS   ',  stm_press )
          ier = irdparam_r(io, 'STM_TEMP    ',  stm_temp )
          ier = irdparam_r(io, 'STM_FLOW    ',  stm_flow )
      end if
          ier = irdparam_r(io, 'RAIRSTEAM',  rairsteam)
 
c --- air assist
      if( MFASSIST .eq. 1 .AND. MFASSISTAIR .eq. 1) then
          ier = irdparam_r(io, 'AIR_FLOW   ',  air_flow )
      end if
          ier = irdparam_r(io, 'RAIRAIR',  rairair)
 
c --- HP FUEL assist
      if( MFASSIST .eq. 1 .AND. MFASSISTHPFUEL .eq. 1) then
          ier = irdparam_r(io, 'HPFUEL_FLOW   ',  hpfuel_flow )
      end if 
          ier = irdparam_r(io, 'RAIRFUEL',  rairfuel)

      
      ier = irdparam_r(io, 'RAW_H2O     ',  gcraw(jph2o     ) )
      ier = irdparam_r(io, 'RAW_H2      ',  gcraw(jph2      ) )
      ier = irdparam_r(io, 'RAW_HE      ',  gcraw(jphe      ) )
      ier = irdparam_r(io, 'RAW_N2      ',  gcraw(jpn2      ) )
      ier = irdparam_r(io, 'RAW_CO2     ',  gcraw(jpco2     ) )
      ier = irdparam_r(io, 'RAW_H2S     ',  gcraw(jph2s     ) )
      ier = irdparam_r(io, 'RAW_C1      ',  gcraw(jpch4     ) )
      ier = irdparam_r(io, 'RAW_C2      ',  gcraw(jpc2h6    ) )
      ier = irdparam_r(io, 'RAW_C3      ',  gcraw(jpc3h8    ) )
      ier = irdparam_r(io, 'RAW_IC4     ',  gcraw(jpic4h10  ) )
      ier = irdparam_r(io, 'RAW_NC4     ',  gcraw(jpnc4h10  ) )
      ier = irdparam_r(io, 'RAW_IC5     ',  gcraw(jpic5h12  ) )
      ier = irdparam_r(io, 'RAW_NC5     ',  gcraw(jpnc5h12  ) )
      ier = irdparam_r(io, 'RAW_NC6     ',  gcraw(jpnc6h14  ) )
      ier = irdparam_r(io, 'RAW_C7      ',  gcraw(jpc7  ) )
      ier = irdparam_r(io, 'RAW_C8      ',  gcraw(jpc8     ) )
      ier = irdparam_r(io, 'RAW_C9      ',  gcraw(jpc9     ) )
      ier = irdparam_r(io, 'RAW_C10     ',  gcraw(jpc10    ) )
      ier = irdparam_r(io, 'RAW_ETHYNE  ',  gcraw(jpc2h2  ) )
      ier = irdparam_r(io, 'RAW_ETHYLENE',  gcraw(jpc2h4 ) )
      ier = irdparam_r(io, 'RAW_PROPYLENE ',  gcraw(jpc3h6 ) )
      ier = irdparam_r(io, 'RAW_BUTENE  ',  gcraw(jpc4h8  ) )
      ier = irdparam_r(io, 'RAW_PENTENE ',  gcraw(jpc5h10 ) )
      ier = irdparam_r(io, 'RAW_HEXENE  ',  gcraw(jpc6h12  ) )
      ier = irdparam_r(io, 'RAW_CO      ',  gcraw(jpco      ) )
      ier = irdparam_r(io, 'RAW_NH3     ',  gcraw(jpnh3     ) )
      ier = irdparam_r(io, 'RAW_AR      ',  gcraw(jpar      ) )
      ier = irdparam_r(io, 'RAW_O2      ',  gcraw(jpo2      ) )

      if(mfuelgas .eq. 0) then
         ier = initialize(gcfuel,0.0D0)
      else          
          ier = irdparam_r(io, 'FUEL_H2O    ',  gcfuel(jph2o     ) )
          ier = irdparam_r(io, 'FUEL_H2     ',  gcfuel(jph2      ) )
          ier = irdparam_r(io, 'FUEL_HE     ',  gcfuel(jphe      ) )
          ier = irdparam_r(io, 'FUEL_N2     ',  gcfuel(jpn2      ) )
          ier = irdparam_r(io, 'FUEL_CO2    ',  gcfuel(jpco2     ) )
          ier = irdparam_r(io, 'FUEL_H2S    ',  gcfuel(jph2s     ) )
          ier = irdparam_r(io, 'FUEL_C1     ',  gcfuel(jpch4     ) )
          ier = irdparam_r(io, 'FUEL_C2     ',  gcfuel(jpc2h6    ) )
          ier = irdparam_r(io, 'FUEL_C3     ',  gcfuel(jpc3h8    ) )
          ier = irdparam_r(io, 'FUEL_IC4    ',  gcfuel(jpic4h10  ) )
          ier = irdparam_r(io, 'FUEL_NC4    ',  gcfuel(jpnc4h10  ) )
          ier = irdparam_r(io, 'FUEL_IC5    ',  gcfuel(jpic5h12  ) )
          ier = irdparam_r(io, 'FUEL_NC5    ',  gcfuel(jpnc5h12  ) )
          ier = irdparam_r(io, 'FUEL_NC6    ',  gcfuel(jpnc6h14  ) )
          ier = irdparam_r(io, 'FUEL_C7     ',  gcfuel(jpc7 ) )
          ier = irdparam_r(io, 'FUEL_C8      ',  gcfuel(jpc8     ) )
          ier = irdparam_r(io, 'FUEL_C9      ',  gcfuel(jpc9     ) )
          ier = irdparam_r(io, 'FUEL_C10     ',  gcfuel(jpc10    ) )
          ier = irdparam_r(io, 'FUEL_ETHYNE  ',  gcfuel(jpc2h2  ) )
          ier = irdparam_r(io, 'FUEL_ETHYLENE',  gcfuel(jpc2h4 ) )
          ier = irdparam_r(io, 'FUEL_PROPYLENE ',  gcfuel(jpc3h6 ) )
          ier = irdparam_r(io, 'FUEL_BUTENE  ',  gcfuel(jpc4h8  ) )
          ier = irdparam_r(io, 'FUEL_PENTENE ',  gcfuel(jpc5h10 ) )
          ier = irdparam_r(io, 'FUEL_HEXENE  ',  gcfuel(jpc6h12  ) )
          ier = irdparam_r(io, 'FUEL_CO     ',  gcfuel(jpco      ) )
          ier = irdparam_r(io, 'FUEL_NH3    ',  gcfuel(jpnh3     ) )
          ier = irdparam_r(io, 'FUEL_AR     ',  gcfuel(jpar      ) )
          ier = irdparam_r(io, 'FUEL_O2     ',  gcfuel(jpo2      ) )
      end if

      if(mliftgas.eq. 0) then
         ier = initialize(gclift,0.0D0)
      else
          ier = irdparam_r(io, 'LIFT_H2O    ',  gclift(jph2o     ) )
          ier = irdparam_r(io, 'LIFT_H2     ',  gclift(jph2      ) )
          ier = irdparam_r(io, 'LIFT_HE     ',  gclift(jphe      ) )
          ier = irdparam_r(io, 'LIFT_N2     ',  gclift(jpn2      ) )
          ier = irdparam_r(io, 'LIFT_CO2    ',  gclift(jpco2     ) )
          ier = irdparam_r(io, 'LIFT_H2S    ',  gclift(jph2s     ) )
          ier = irdparam_r(io, 'LIFT_C1     ',  gclift(jpch4     ) )
          ier = irdparam_r(io, 'LIFT_C2     ',  gclift(jpc2h6    ) )
          ier = irdparam_r(io, 'LIFT_C3     ',  gclift(jpc3h8    ) )
          ier = irdparam_r(io, 'LIFT_IC4    ',  gclift(jpic4h10  ) )
          ier = irdparam_r(io, 'LIFT_NC4    ',  gclift(jpnc4h10  ) )
          ier = irdparam_r(io, 'LIFT_IC5    ',  gclift(jpic5h12  ) )
          ier = irdparam_r(io, 'LIFT_NC5    ',  gclift(jpnc5h12  ) )
          ier = irdparam_r(io, 'LIFT_NC6    ',  gclift(jpnc6h14  ) )
          ier = irdparam_r(io, 'LIFT_C7     ',  gclift(jpc7 ) )
          ier = irdparam_r(io, 'LIFT_C8      ',  gclift(jpc8     ) )
          ier = irdparam_r(io, 'LIFT_C9      ',  gclift(jpc9     ) )
          ier = irdparam_r(io, 'LIFT_C10     ',  gclift(jpc10    ) )
          ier = irdparam_r(io, 'LIFT_ETHYNE  ',  gclift(jpc2h2  ) )
          ier = irdparam_r(io, 'LIFT_ETHYLENE',  gclift(jpc2h4 ) )
          ier = irdparam_r(io, 'LIFT_PROPYLENE ',  gclift(jpc3h6 ) )
          ier = irdparam_r(io, 'LIFT_BUTENE  ',  gclift(jpc4h8  ) )
          ier = irdparam_r(io, 'LIFT_PENTENE ',  gclift(jpc5h10 ) )
          ier = irdparam_r(io, 'LIFT_HEXENE  ',  gclift(jpc6h12  ) )
          ier = irdparam_r(io, 'LIFT_CO     ',  gclift(jpco      ) )
          ier = irdparam_r(io, 'LIFT_NH3    ',  gclift(jpnh3     ) )
          ier = irdparam_r(io, 'LIFT_AR     ',  gclift(jpar      ) )
          ier = irdparam_r(io, 'LIFT_O2     ',  gclift(jpo2      ) )
      end if         
      
c --- group MET
      if(MMET .eq. 3 ) then
          ier = irdparam_r(io, 'WINDSP',  rWINDSP )
          ier = irdparam_r(io, 'WINDDR',  rWINDDR)
          ier = irdparam_r(io, 'TAMB',    rTAMB )  ! user entry is in C
          ier = irdparam_i(io, 'PGCLASS', irPGCLASS )
          ier = irdparam_r(io, 'ZREF', ZREF )
      else
          rWINDSP    = 0.
          rWINDDR    = 0.
          rTAMB      = 0.
          irPGCLASS  = 0
          zref      = 10.
      end if

      ierr = 0
 9000 close(io)
      return
      end

     
c----------------------------------------------------------------------
c ichkinput
c----------------------------------------------------------------------
c --- Developed by B.Zelt - www.zeltpsi.com
c
c     Check the input values read for range of values or valid entries
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     io  :intger :  list file io number
c
c --- OUTPUTS
c     ierr: integer   : 0 no errors
c                     : 1 terminal errors were found
c
c----------------------------------------------------------------------
      function ichkinput(io )
     &result(ierr)
c   Common blocks
      use mod_stddefs 
      use mod_readinp
c    INPUTS
      integer     :: ierr
      integer     :: io
c    Declarations


      ierr =0
      
      if( MINPUT .lt. 1 .or. MINPUT .gt. 3) then
          write(io,*) 'FATAL ERROR: Unknown setting for MINPUT=',MINPUT
          ierr = ierr + 1
      end if 
      if( MBLOWDOWN .lt. 0 .or. MBLOWDOWN .gt. 5) then
          write(io,*) 'FATAL ERROR: Unknown setting for MBLOWDOWN=',
     &          MBLOWDOWN
          ierr = ierr + 1
      end if 
      if( MDIST .lt. 1 .or. MDIST .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MDIST=',MDIST
          ierr = ierr + 1
      end if 
      if( MSTRIP .lt. 0 .or. MSTRIP .gt. 5) then
          write(io,*) 'FATAL ERROR: Unknown setting for MSTRIP=',MSTRIP
          ierr = ierr + 1
      end if 
      if( MMET .lt. 1 .or. MMET .gt. 3) then
          write(io,*) 'FATAL ERROR: Unknown setting for MMET=',MMET
          ierr = ierr + 1
      end if 
      if( MFUELGAS .lt. 0 .or. MFUELGAS .gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFUELGAS=',
     &          MFUELGAS
          ierr = ierr + 1
      end if 
      if( MLIFTGAS .lt. 0 .or. MLIFTGAS.gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MLIFTGAS=',
     &          MLIFTGAS
          ierr = ierr + 1
      end if 
      if( MFGR .lt. 1 .or. MFGR .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFGR=',MFGR
          ierr = ierr + 1
      end if 
      if( MFASSIST .lt. 0 .or. MFASSIST .gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFASSIST=',
     &          MFASSIST
          ierr = ierr + 1
      end if 
      if( MFASSIST .EQ. 1) then 
      if( MFASSISTSTEAM .lt. 0 .or. MFASSISTSTEAM .gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFASSISTSTEAM=',
     &          MFASSISTSTEAM 
          ierr = ierr + 1
      end if 
      if( MFASSISTAIR .lt. 0 .or. MFASSISTAIR .gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFASSISTAIR=',
     &          MFASSISTAIR 
          ierr = ierr + 1
      end if 
      if( MFASSISTHPFUEL .lt. 0 .or. MFASSISTHPFUEL .gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFASSISTHPFUEL=',
     &          MFASSISTHPFUEL 
          ierr = ierr + 1
      end if 
      end if 

      if( MFLAMEOUT .lt. 0 .or. MFLAMEOUT .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFLAMEOUT=',
     &          MFLAMEOUT
          ierr = ierr + 1
      end if 
      
      if(nsep.eq.1) then 
      elseif( int(nsep/24)*24 .ne. nsep ) then
          write(io,*) 'FATAL ERROR: NSEP is not a multiple of 24'
          ierr = ierr + 1
      end if 
      if( nsep/24 .gt. 5) then
          write(io,*) 'FATAL ERROR: NSEP is greater than 5*24'
          ierr = ierr + 1
      end if 

      if( MTGAS .lt. 0 .or. MTGAS .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MTGAS=',MTGAS
          ierr = ierr + 1
      end if 
      if( MTGAS .eq. 0 .and. TGINIT.lt.0.) then
          write(io,*) 'FATAL ERROR: Gas temperature set to use TGINIT'
          write(io,*) '             but TGINIT is not defined, TGINIT=',TGINIT
          write(io,*) '             MTGAS=',MTGAS
          ierr = ierr + 1
      end if    
      if( odourppb .eq. 0.0) then 
         odourppb = 1.0;
          write(io,*) 'WARNING : Odour Threshold reset to '
          write(io,*) '             ODOURPPB=',odourppb
      end if 

      if( df .eq. 0.0) then 
          write(io,*) 'ERROR : Missing parameter DF'
          write(io,*) '             DF=',df
         df = ds;
          ierr = ierr + 1
      end if 
      if( ds .gt. df) then 
          write(io,*) 'ERROR : DS must be smaller or equal to DF'
          write(io,*) '             DF=',df
          write(io,*) '             DS=',ds
          ierr = ierr + 1
      end if 
      
      !if( MFASSISTAIR .GT. 0) then
      if( rairair .eq. 0.0) then 
          write(io,*) 'WARNING: Missing parameter RAIRAIR'
          write(io,*) 'WARNING: reset to default RAIRAIR'
         rairair = 0.35;
          write(io,*) '    RAIRAIR=',rairair
      end if 
      !end if 
      !if( MFASSISTSTEAM .GT. 0) then
      if( rairsteam .eq. 0.0) then 
          write(io,*) 'WARNING: Missing parameter RAIRSTEAM'
          write(io,*) 'WARNING: reset to default RAIRSTEAM'
         rairsteam = 0.35;
          write(io,*) '    RAIRSTEAM=',rairsteam
      end if 
      !end if 
      !if( MFASSISTHPFUEL .GT. 0) then
      if( rairfuel .eq. 0.0) then 
          write(io,*) 'WARNING: Missing parameter RAIRFUEL'
          write(io,*) 'WARNING: reset to default RAIRFUEL'
         rairfuel = 0.35;
          write(io,*) '    RAIRFUEL=',rairfuel
      end if 
      !end if 
      
c      if( nspec .gt. 2) then
c          write(io,*) 'FATAL ERROR: nspec is greater than 1'
c          ierr = ierr + 1
c      end if 

c      if( leq(units,'m') .eq.  0 .and. 
c     &     leq( units,'M') .eq. 0 .and. 
c     &     leq(units,'km') .eq. 0 .and. 
c     &     leq(units,'KM') .eq. 0) then
c          write(io,*) 'FATAL ERROR: bad units (m or km):', units
c          ierr = ierr + 1
c      end if 

c --------------------     
      if( MINPUT.eq.1 .and. MBLOWDOWN .eq. 1) then
      if( pressinit .lt. 1. .or. pressinit .gt. 25000. ) then
          write(io,*) 'WARNING: initial pressure may be out of bounds'
      end if 
      
      if( pressinit .le. pressfinal ) then
          write(io,*) 'FATAL ERROR: bad final presssure:', pressfinal
          ierr = ierr + 1
      end if 

      if( tginit .gt. 0. ) then
         if( tginit .lt. rk0-50. .or. tginit .gt. rk0+50.) then
             write(io,*) 'WARNING: tginit may be out of bounds'
         end if 
      else
          write(io,*) 'WARNING: Using tginit = tamb'
      end if 
      
      if( tempinit .gt. 0. ) then
         if( tempinit .lt. rk0-50. .or. tempinit .gt. rk0+50.) then
             write(io,*) 'WARNING: tempinit may be out of bounds'
         end if 
      else
          write(io,*) 'WARNING: Using tempinit = tamb'
      end if 
      
      if( vesseldia .lt. 0.01 .or. vesseldia .gt. 10. ) then
          write(io,*) 'WARNING: Vessel Diameter may be out of bounds', 
     &          vesseldia
      end if 
      
      if(vessellen .lt. 1. .or. vessellen.gt. 1000000. ) then
          write(io,*) 'WARNING: Vessel length may be out of bounds',
     &           vessellen
      end if 
      
      if(orificedia*0.001.lt. 0.0001 .or. orificedia*0.001 .gt. 10. ) then
          write(io,*) 'WARNING: orifice diameter may be out of bounds',
     &            orificedia          
      end if 

      if(dischargecoef.lt.0.001 .or. dischargecoef.gt. 1 ) then
          write(io,*) 'WARNING: ',
     &          'Discharge coefficient may be out of bounds',
     &          dischargecoef
      end if 
      end if 

c --------------------     
      if( MINPUT.eq.1 .and. MBLOWDOWN .eq. 2) then
c      if( ) then
c          write(io,*) 'FATAL ERROR: '
c          write(io,*) 'WARNING: '
c          ierr = ierr + 1
c     end if 
      end if     

c check tau
c check pufdur
c check npuffs
      
      return 
      end
     
c----------------------------------------------------------------------
c iwrcontrol
c----------------------------------------------------------------------
c
c     writes the input parameters and variables as they were parsed 
c
c Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.002    Level: 121112
c     - added lift gas
c     - added aermod capacity
c
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     io  : integer   : list file io number
c
c --- OUTPUTS
c     ierr: integer:  0: no errors
c                     1: errors occured
c----------------------------------------------------------------------
      function iwrcontrol(io )
     &result(ierr)

c   Common blocks
      use mod_stddefs
      use mod_readinp
      use mod_globals
c    INPUTS
      integer  :: io
      integer  :: ierr
c     DECLARATIONS
      character*1 :: cnote, cmark, cstar
      integer ::  n
      integer :: ier 
      
      
      cmark = '!'
      cstar = '*'
      
c    INPUTs Declarations
c      include 'h_interface.for'

      ierr = 1   
      ier = 0

c      write(io,*) ' Model:',mmodel,' Version:',mver,' Level:',mlevel
      n=LEN_REAL(stitle)
      write(io,'(A,A,A)') '! STITLE1 =', stitle(1:n),'!'
      n=LEN_REAL(sname)
      write(io,'(A,A,A)') '! SNAME   =', sname(1:n),'!'

      n=LEN_REAL(OUTNAME)
      write(io,'(A,A,A)') '! OUTNAME=', OUTNAME(1:n), '!'
      n=LEN_REAL(OUTPATH)
      write(io,'(A,A,A)') '! OUTPATH=', OUTPATH(1:n), '!'
      n=LEN_REAL(METDAT)
      write(io,'(A,A,A)') '! METDAT=', METDAT(1:n), '!'
      n=LEN_REAL(METDAT2)
      write(io,'(A,A,A)') '! METDAT2=', METDAT2(1:n), '!'
      n=LEN_REAL(RECINC)
      write(io,'(A,A,A)') '! RECINC=', RECINC(1:n), '!'
      n=LEN_REAL(LSTDAT)
      write(io,'(A,A,A)') '! LSTDAT=', LSTDAT(1:n), '!'
      n=LEN_REAL(LOGDAT)
      write(io,'(A,A,A)') '! LOGDAT=', LOGDAT(1:n), '!'
      if( lcfiles ) then
          write(io,'(A,A,A)') '! LCFILES= T !'
      else          
          write(io,'(A,A,A)') '! LCFILES= F !'
      end if   

      write(io,'(A,I,A)') '! MDSPMOD   =', MDSPMOD,'!'
      write(io,'(A,I,A)') '! MINPUT    =', MINPUT,'!'
      write(io,'(A,I,A)') '! MBLOWDOWN =', MBLOWDOWN,'!'
      write(io,'(A,I,A)') '! MDIST     =',   MDIST,'!'
      write(io,'(A,I,A)') '! MSTRIP    =', MSTRIP,'!'
      write(io,'(A,I,A)') '! MMET      =', MMET,'!'
      write(io,'(A,I,A)') '! MFUELGAS  =', MFUELGAS ,'!'
      write(io,'(A,I,A)') '! MFGR      =', MFGR,'!'
      write(io,'(A,I,A)') '! MLIFTGAS  =', MLIFTGAS ,'!'
      write(io,'(A,I,A)') '! MFASSIST  =', MFASSIST ,'!'
      write(io,'(A,I,A)') '! MFASSISTSTEAM  =', MFASSISTSTEAM ,'!'
      write(io,'(A,I,A)') '! MFASSISTAIR    =', MFASSISTAIR ,'!'
      write(io,'(A,I,A)') '! MFASSISTHPFUEL =', MFASSISTHPFUEL ,'!'
      write(io,'(A,I,A)') '! MTGAS          =', MTGAS,'!'
      write(io,'(A,I,A)') '! MSCREENING     =', MSCREENING,'!'
      write(io,'(A,I,A)') '! MDEBUG    =', MDEBUG ,'!'
      write(io,'(A,I,A)') '! NSEP      =', NSEP ,'!'
      write(io,'(A,I,A)') '! NSOURCES  =', NSOURCES ,'!'
      write(io,'(A,I,A)') '! MNONSEQ   =', MNONSEQ ,'!'
      write(io,'(A,I,A)') '! MSCREEN   =', MSCREEN,'!'
      write(io,'(A,I,A)') '! MFLAMEOUT =', MFLAMEOUT,'!'
      write(io,'(A,I,A)') '! MGROUNDFLARE =', MGROUNDFLARE,'!'
      write(io,'(A,I,A)') '! MDLIFT =', MDLIFT,'!'
      write(io,'(A,I,A)') '! MDFUEL =', MDFUEL,'!'
      

      write(io,'(A,I,A)') '! IUTMZN  =', IUTMZN,'!'
      n=LEN_REAL(UTMHEM)
      write(io,'(A,A,A)') '! UTMHEM  =', UTMHEM(1:n),'!'
      n=LEN_REAL(PMAP)
      write(io,'(A,A,A)') '! PMAP    =', PMAP(1:n),'!'
      n=LEN_REAL(DATUM)
      write(io,'(A,A,A)') '! DATUM   =', DATUM(1:n),'!'
c      n=LEN_REAL(UNITS)
c      write(io,'(A,A,A)') '! UNITS   =', UNITS(1:n),'!'
      n=LEN_REAL(ABTZ)
      write(io,'(A,A,A)') '! ABTZ    =', ABTZ(1:n),'!'

      write(io,'(A,I,A)') '! NSPEC   =', nspec,'!'
      write(io,'(A,10(A,A))') '! CHEMLST =', 
     &((chemlst(i)(1:3),','),i=1,nspec-1), chemlst(nspec)(1:3),'!'
      write(io,'(A,10(F,A))') '! MW =', 
     &((mmlst(i),','),i=1,nspec-1), mmlst(nspec),'!'
      write(io,'(A,10(F,A))') '! EMIT =', 
     &((qlst(i),','),i=1,nspec-1), qlst(nspec),'!'

      write(io,'(A,I,A)') '! YYS     =',     iyys ,'!'
      write(io,'(A,I,A)') '! MMS     =',     imms ,'!'
      write(io,'(A,I,A)') '! DDS     =',     idds  ,'!'
      write(io,'(A,I,A)') '! HHS     =',     ihhs  ,'!'
      write(io,'(A,I,A)') '! SSS     =',     isss  ,'!'
      write(io,'(A,I,A)') '! YYE     =',     iyye  ,'!'
      write(io,'(A,I,A)') '! MME     =',     imme  ,'!'
      write(io,'(A,I,A)') '! DDE     =',     idde  ,'!'
      write(io,'(A,I,A)') '! HHE     =',     ihhe  ,'!'
      write(io,'(A,I,A)') '! SSE     =',     isse  ,'!'
      write(io,'(A,I,A)') '! TIMEINC =',  itiminc  ,'!'
      write(io,'(A,F,A)') '! XCOORD  =',     xcoord   ,'!'
      write(io,'(A,F,A)') '! YCOORD  =',     ycoord   ,'!'
      write(io,'(A,F,A)') '! ZCOORD  =',     zcoord   ,'!'
      write(io,'(A,F,A)') '! HS      =',     hs   ,'!'
      write(io,'(A,F,A)') '! DF      =',     df  ,'!'
      write(io,'(A,F,A)') '! DS      =',     ds  ,'!'
      
      if(MGROUNDFLARE .eq. 1) then
         write(io,'(A,I,A)') '! NHEADS        =',  nheads ,'!'
      else
         write(io,'(A,I,A)') '* NHEADS        =',  nheads ,'*'
      end if 
      if(MDLIFT .eq. 1) then
         write(io,'(A,F,A)') '! DLIFT        =',  dlift ,'!'
      else
         write(io,'(A,F,A)') '* DLIFT        =',  dlift ,'*'
      end if 
      if(MDFUEL .eq. 1) then
         write(io,'(A,F,A)') '! DFUEL        =',  dfuel ,'!'
      else
         write(io,'(A,F,A)') '* DFUEL        =',  dfuel ,'*'
      end if 
      
      write(io,'(A,F,A)') '! RH2S    =',     rh2s ,'!'
      write(io,'(A,F,A)') '! TGINIT  =',     tginit ,'!'

            
      
      if(MINPUT .eq. 1 ) then
          if( mblowdown .eq. 0) then
              write(io,'(A,F,A)') '! QMAX        =',     qmax ,'!'
              write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
              write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
              write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
              write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
              write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
              write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
              write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
              write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
              write(io,'(A,F,A)') '* DISCHARGECOEF =',dischargecoef  ,'*'
              write(io,'(A,I,A)') '! NPUFFS      =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR     =',     ipuffdur,'!' ! one of 
              write(io,'(A,F,A)') '! DURATION    =',     duration ,'!'
              write(io,'(A,I,A)') '* PRATIO       =',     pratio,'*' 
              write(io,'(A,I,A)') '* TDEP         =',     tdep,'*'              
         else if( mblowdown .eq. 1 ) then         
              write(io,'(A,F,A)') '* QMAX        =',     qmax ,'*'
              write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
              write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
              write(io,'(A,F,A)') '! PRESSINIT     =',   pressinit  ,'!'
              write(io,'(A,F,A)') '! TEMPINIT      =',    tempinit  ,'!'
              write(io,'(A,F,A)') '! PRESSFINAL    =',  pressfinal  ,'!'
              write(io,'(A,F,A)') '! VESSELDIA     =',   vesseldia  ,'!'
              write(io,'(A,F,A)') '! VESSELLEN     =',   vessellen  ,'!'
              write(io,'(A,F,A)') '! ORIFICEDIA    =',  orificedia  ,'!'
              write(io,'(A,F,A)') '! DISCHARGECOEF =',dischargecoef  ,'!'
              write(io,'(A,I,A)') '! NPUFFS      =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR     =',     ipuffdur,'!' ! one of 
              write(io,'(A,F,A)') '! DURATION    =',     duration ,'!'
              write(io,'(A,I,A)') '* PRATIO       =',     pratio,'*' 
              write(io,'(A,I,A)') '* TDEP         =',     tdep,'*'              
         else if( mblowdown .eq. 2) then
              write(io,'(A,F,A)') '! QMAX        =',     qmax ,'!'
              write(io,'(A,F,A)') '! QTOTAL      =',   qtotal ,'!'
              write(io,'(A,F,A)') '! FREL        =',     frel ,'!'
              write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
              write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
              write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
              write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
              write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
              write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
              write(io,'(A,F,A)') '* DISCHARGECOEF =',dischargecoef  ,'*'
              write(io,'(A,I,A)') '! NPUFFS      =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR     =',     ipuffdur,'!' ! one of 
              write(io,'(A,F,A)') '! DURATION    =',     duration ,'!'
              write(io,'(A,I,A)') '* PRATIO       =',     pratio,'*' 
              write(io,'(A,I,A)') '* TDEP         =',     tdep,'*'              
         else if( mblowdown .eq. 3) then
              write(io,'(A,F,A)') '! QMAX        =',     qmax ,'!'
              write(io,'(A,F,A)') '! QTOTAL      =',   qtotal ,'!'
              write(io,'(A,F,A)') '! FREL        =',     frel ,'!'
              write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
              write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
              write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
              write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
              write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
              write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
              write(io,'(A,F,A)') '* DISCHARGECOEF =',dischargecoef  ,'*'
              write(io,'(A,I,A)') '! NPUFFS       =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR      =',     ipuffdur,'!' ! one of 
              write(io,'(A,I,A)') '! PRATIO       =',     pratio,'!' 
              write(io,'(A,I,A)') '! TDEP         =',     tdep,'!'
              
         else if( mblowdown .eq. 4) then
              write(io,'(A,F,A)') '! QMAX        =',     qmax ,'!'
              write(io,'(A,F,A)') '! QTOTAL      =',   qtotal ,'!'
              write(io,'(A,F,A)') '! FREL        =',     frel ,'!'
              write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
              write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
              write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
              write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
              write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
              write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
              write(io,'(A,F,A)') '* DISCHARGECOEF =',dischargecoef  ,'*'
              write(io,'(A,I,A)') '! NPUFFS       =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR      =',     ipuffdur,'!' ! one of 
              write(io,'(A,I,A)') '! PRATIO       =',     pratio,'!' 
              write(io,'(A,I,A)') '! TDEP         =',     tdep,'!'

         else if( mblowdown .eq. 5) then
              write(io,'(A,F,A)') '! QMAX        =',     qmax ,'!'
              write(io,'(A,F,A)') '! QTOTAL      =',   qtotal ,'!'
              write(io,'(A,F,A)') '! FREL        =',     frel ,'!'
              write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
              write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
              write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
              write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
              write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
              write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
              write(io,'(A,F,A)') '* DISCHARGECOEF =',dischargecoef  ,'*'
              write(io,'(A,I,A)') '! NPUFFS       =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR      =',     ipuffdur,'!' ! one of 
              write(io,'(A,I,A)') '! PRATIO       =',     pratio,'!' 
              write(io,'(A,I,A)') '! TDEP         =',     tdep,'!'
              
          end if 
          write(io,'(A,F,A)') '* PSHS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSDS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSTS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSUS      =',     0.  ,'*'
          write(io,'(A,I,A)') '* NSEQ     =',     NSEQ,'*'
      else if( MINPUT .eq. 2 ) then
          write(io,'(A,F,A)') '* QMAX        =',     qmax ,'*'
          write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
          write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
          write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
          write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
          write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
          write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
          write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
          write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
          write(io,'(A,F,A)')'* DISCHARGECOEF =',dischargecoef  ,'*'
          write(io,'(A,I,A)') '* NPUFFS      =',     npuffs  ,'*'
          write(io,'(A,I,A)') '* PUFFDUR     =',     ipuffdur,'*'
          write(io,'(A,F,A)') '* DURATION    =',     duration ,'*'
         write(io,'(A,I,A)') '* PRATIO       =',     pratio,'*' 
         write(io,'(A,I,A)') '* TDEP         =',     tdep,'*'              
          write(io,'(A,F,A)') '* PSHS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSDS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSTS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSUS      =',     0.  ,'*'
          write(io,'(A,I,A)') '! NSEQ     =',     NSEQ,'!'
          do i=1,NSEQ
              write(io,'(I,A,F,F,F,F,A)') i,'!=',
     &              pseq(i,1),pseq(i,2),pseq(i,3),pseq(i,4),'!'
          end do
      else if( MINPUT .eq. 3 ) then
          write(io,'(A,F,A)') '* QMAX        =',     qmax ,'*'
          write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
          write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
          write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
          write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
          write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
          write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
          write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
          write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
          write(io,'(A,F,A)')'* DISCHARGECOEF =',dischargecoef  ,'*'
          write(io,'(A,I,A)') '* NPUFFS      =',     npuffs  ,'*'
          write(io,'(A,I,A)') '* PUFFDUR     =',     ipuffdur,'*'
          write(io,'(A,F,A)') '* DURATION    =',     duration,'*'
         write(io,'(A,I,A)') '* PRATIO       =',     pratio,'*' 
         write(io,'(A,I,A)') '* TDEP         =',     tdep,'*'              
          write(io,'(A,F,A)') '! PSHS      =',     pshs  ,'!'
          write(io,'(A,F,A)') '! PSDS      =',     psds  ,'!'
          write(io,'(A,F,A)') '! PSTS      =',     psts  ,'!'
          write(io,'(A,F,A)') '! PSUS      =',     psus  ,'!'
          write(io,'(A,I,A)') '* NSEQ     =',     NSEQ,'*'
      end if 

      if(mfuelgas .eq. 1 ) then
          write(io,'(A,F,A)') '! FGR         =',     fgr     ,'!'
      else
          write(io,'(A,F,A)') '* FGR         =',     fgr     ,'*'
      end if
      if(mliftgas .eq. 1 ) then
          write(io,'(A,F,A)') '! QLIFTGAS    =',     qliftgas  ,'!'
      else
          write(io,'(A,F,A)') '* QLIFTGAS    =',     qliftgas     ,'*'
      end if


      if(MFASSISTSTEAM .eq. 1) then
         cnote = cmark
      else
         cnote = cstar
      end if 
      write(io,'(A,A,I,A)') cnote, ' STM_IMASSVOL=',  stm_imassvol,cnote
      write(io,'(A,A,F,A)') cnote, ' STM_PRESS   =',  stm_press,cnote
      write(io,'(A,A,F,A)') cnote, ' STM_TEMP    =',  stm_temp,cnote
      write(io,'(A,A,F,A)') cnote, ' STM_FLOW    =',  stm_flow,cnote
          
      if(MFASSISTAIR .eq. 1) then
         cnote = cmark
      else
         cnote = cstar
      end if 
      write(io,'(A,A,F,A)') cnote, ' AIR_FLOW    =',  air_flow, cnote

      if(MFASSISTHPFUEL .eq. 1) then
         cnote = cmark
      else
         cnote = cstar
      end if           
      write(io,'(A,A,F,A)') cnote, ' HPFUEL_FLOW    =',  hpfuel_flow, cnote
      write(io,'(A,A,F,A)') cnote, ' RAIRAIR    =',  rairair,cnote
      write(io,'(A,A,F,A)') cnote, ' RAIRSTEAM  =',  rairsteam,cnote
      write(io,'(A,A,F,A)') cnote, ' RAIRFUEL   =',  rairfuel,cnote

      write(io,'(A,F6.4,A)') '! RAW_H2O     =',  gcraw(jph2o     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_H2      =',  gcraw(jph2      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_HE      =',  gcraw(jphe      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_N2      =',  gcraw(jpn2      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_CO2     =',  gcraw(jpco2     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_H2S     =',  gcraw(jph2s     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C1      =',  gcraw(jpch4     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C2      =',  gcraw(jpc2h6    ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C3      =',  gcraw(jpc3h8    ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_IC4     =',  gcraw(jpic4h10  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NC4     =',  gcraw(jpnc4h10  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_IC5     =',  gcraw(jpic5h12  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NC5     =',  gcraw(jpnc5h12  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NC6     =',  gcraw(jpnc6h14  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C7      =',  gcraw(jpc7 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C8      =',  gcraw(jpc8 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C9      =',  gcraw(jpc9 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C10     =',  gcraw(jpc10 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_ETHYNE  =',  gcraw(jpc2h2 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_ETHYLENE =',  gcraw(jpc2h4 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_PROPYLENE =',  gcraw(jpc3h6 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_BUTENE  =',  gcraw(jpc4h8 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_PENTENE =',  gcraw(jpc5h10 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_HEXENE  =',  gcraw(jpc6h12 ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_CO      =',  gcraw(jpco      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NH3     =',  gcraw(jpnh3     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_AR      =',  gcraw(jpar      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_O2      =',  gcraw(jpo2      ) ,'!'

      if(mfuelgas .eq. 0 ) then
         cnote = cmark
      else
         cnote = cstar
      end if           
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_H2O    =', gcfuel(jph2o     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_H2     =', gcfuel(jph2      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_HE     =', gcfuel(jphe      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_N2     =', gcfuel(jpn2      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_CO2    =', gcfuel(jpco2     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_H2S    =', gcfuel(jph2s     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C1     =', gcfuel(jpch4     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C2     =', gcfuel(jpc2h6    ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C3     =', gcfuel(jpc3h8    ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_IC4    =', gcfuel(jpic4h10  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_NC4    =', gcfuel(jpnc4h10  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_IC5    =', gcfuel(jpic5h12  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_NC5    =', gcfuel(jpnc5h12  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_NC6    =', gcfuel(jpnc6h14  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C7      =',  gcfuel(jpc7 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C8      =',  gcfuel(jpc8 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C9      =',  gcfuel(jpc9 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_C10     =',  gcfuel(jpc10 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_ETHYNE  =',  gcfuel(jpc2h2 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_ETHYLENE =',  gcfuel(jpc2h4 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_PROPYLENE =',  gcfuel(jpc3h6 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_BUTENE  =',  gcfuel(jpc4h8 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_PENTENE  =',  gcfuel(jpc5h10 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_HEXENE  =',  gcfuel(jpc6h12 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_CO     =', gcfuel(jpco      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_NH3    =', gcfuel(jpnh3     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_AR     =', gcfuel(jpar      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' FUEL_O2     =', gcfuel(jpo2      ) , cnote

      
      if(mliftgas .eq. 0 ) then
         cnote = cmark
      else
         cnote = cstar
      end if           
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_H2O    =', gclift(jph2o     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_H2     =', gclift(jph2      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_HE     =', gclift(jphe      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_N2     =', gclift(jpn2      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_CO2    =', gclift(jpco2     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_H2S    =', gclift(jph2s     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C1     =', gclift(jpch4     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C2     =', gclift(jpc2h6    ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C3     =', gclift(jpc3h8    ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_IC4    =', gclift(jpic4h10  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_NC4    =', gclift(jpnc4h10  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_IC5    =', gclift(jpic5h12  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_NC5    =', gclift(jpnc5h12  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_NC6    =', gclift(jpnc6h14  ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C7      =',  gclift(jpc7 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C8      =',  gclift(jpc8 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C9      =',  gclift(jpc9 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_C10     =',  gclift(jpc10 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_ETHYNE  =',  gclift(jpc2h2 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_ETHYLENE =',  gclift(jpc2h4 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_PROPYLENE =',  gclift(jpc3h6 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_BUTENE  =',  gclift(jpc4h8 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_PENTENE  =',  gclift(jpc5h10 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_HEXENE  =',  gclift(jpc6h12 ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_CO     =', gclift(jpco      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_NH3    =', gclift(jpnh3     ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_AR     =', gclift(jpar      ) , cnote
      write(io,'(A,A,F6.4,A)')  cnote, ' LIFT_O2     =', gclift(jpo2      ) , cnote

      
      if(MMET .eq. 3) then
         cnote = cmark
      else
         cnote = cstar
      end if           
      write(io,'(A,A,F,A)')  cnote, ' WINDSP  =', 0.   , cnote
      write(io,'(A,A,F,A)')  cnote, ' WINDDR  =', 0.   , cnote
      write(io,'(A,A,F,A)')  cnote, ' TAMB    =', 0.   , cnote
      write(io,'(A,A,I,A)')  cnote, ' PGCLASS =', 0    , cnote
      write(io,'(A,A,F,A)')  cnote, ' ZREF    =', 0.    , cnote
      
      write(io,*) ' ---------------------------------' 
      write(io,*) 'end of input file'
      write(io,*) 
      write(io,*) 

      ierr = 0
 ! close(io)
 9000 return
      end


c----------------------------------------------------------------------
c Vessel initialize
c----------------------------------------------------------------------
c
c     Calculate the flow rate output of a pressurized vessel through
c     a specified orifice.
c
c
c Develped by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     mix           :real(mxcomp) :normalized raw gas combusted
c     pressinit     :real   :Expected Maximum Initial Pressure	kPa (gauge)
c     tempinit      :real   :Expected Minimum Initial Gas Temperature	K
c     pressfinal    :real   :Expected Minimum Final Pressure	kPa
c     vesseldia     :real   :Pipeline/Vessel Inside Diameter	m
c     vessellen     :real   :Pipeline/Vessel Length	m
c     orificedia    :real   :Minimum Orifice Diameter	mm
c     dischargecoef :real   :Discharge Coefficient
c     qmax          :real   :initial volume flow rate, e3m3/d (ideal gas)
c     qtotal        :real   :total volume released, e3m3 (ideal gas)
c     frel          :real   :fraction of volume released
c     tgasout       :real   :exit temperature of gas, K
c
c --- OUTPUTS
c     ierr: integer:  0: no errors
c                     1: errors were found
c----------------------------------------------------------------------
      function ivessel(mix,pressinit,tempinit,pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgasout )
     &result(ierr)
c includes
      use mod_stddefs 
      use mod_globals
c inputs      
      real(8):: mix(*)    ! normalized raw gas combusted
      real(8), intent (in) :: pressinit		! Expected Maximum Initial Pressure	kPa (gauge)
      real(8), intent (in) :: tempinit		! Expected Minimum Initial Gas Temperature	K 
      real(8), intent (in) :: pressfinal		! Expected Minimum Final Pressure	kPa
      real(8), intent (in) :: vesseldia		! Pipeline/Vessel Inside Diameter	m
      real(8), intent (in) :: vessellen		! Pipeline/Vessel Length	m
      real(8), intent (in) :: orificedia		! Minimum Orifice Diameter	mm
      real(8), intent (in) :: dischargecoef	! Discharge Coefficient	
      real(8), intent (out):: qmax           ! initial volume flow rate, e3m3/d (ideal gas)
      real(8), intent (out):: qtotal         ! total volume released, e3m3 (ideal gas)
      real(8), intent (out):: frel           ! fraction of volume released
      real(8), intent (out):: tgasout        ! exit temperature of gas, K
c declarations
      real(8) :: ap  ! area of vessel
      real(8) :: vp  ! volume of vessel
      real(8) :: ae  ! exit area
      real(8) :: rmm ! molar mass
      real(8) :: rs  ! gas constant 
      real(8) :: rhoref ! density at reference conditions
      real(8) :: cpi ! specific heat 
      real(8) :: pinit ! initial pressure
      real(8) :: tinit ! initial temperatue
      real(8) :: avek, ratiok1, ratiok2, ratiok3
      real(8) :: zo  ! compressibility, initial
      real(8) :: co  ! speed of sound, initial
      real(8) :: ze  ! compressibility, exit
      real(8) :: ce  ! speed of sound, exit
      real(8) :: rhoinit ! initial density
      real(8) :: rhoe    ! exit density
      real(8) :: teig_choked
      real(8) :: peig_choked
      real(8) :: teig
      real(8) :: peig
c      real(8) :: pratio 
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

c initializations
      ap      = 0.25*pi*vesseldia*vesseldia
      vp      = ap*vessellen
      ae      = 0.25*pi*orificedia*orificedia*.001*.001 ! m2
      rmm     = calc_molarmass(mix)       ! kg/kmole
      rs      = ru*1000./rmm              ! j/(kg.K)
      rhoref  = 1000.*pref/(tref*rs)      ! kg/m3
      cpi     = calc_byprop(mix, ipcpmole, ipcalc)/rmm*1000. ! j/(kg.K)
      pinit   = plocal + pressinit     ! kPa 
      tinit   = tempinit                  ! K

c Thermodynamics      
      avek    = cpi/(cpi-rs)  
      ratiok1 = 2./(avek+1.)
      ratiok2 = (avek+1.)/(avek-1.)
      ratiok3 = avek/(avek-1.)
      zo      = 1.0
      co      = SQRT(avek*zo*rs*tinit)        ! m/s
      rhoinit = pinit*1000./(zo*rs*tinit)     ! kg/m3
      teig_choked    = tinit*ratiok1          ! K
      peig_choked    = pinit*ratiok1**ratiok3 ! kPa
      ze      = 1.0
      if( peig_choked .lt. plocal) then
          peig    = plocal
          teig    = tinit
      else          
          peig    = peig_choked
          teig    = teig_choked/ze*zo
      end if 
      rhoe    = peig*1000./(rs*ze*teig)   ! kg/m3
      ce      = SQRT(avek*ze*rs*teig)     ! m/s

c exponential blowdown model
      rqtotal = rhoinit*vp               ! total mass released, kg       
      qtotal  = rqtotal/(rhoref*1000.)   ! total volume released, e3m3
      rqemit  = dischargecoef*(ze/zo*ratiok1)**(0.5*ratiok2)
     &         *SQRT(avek/(zo*rs*tinit))*pinit*ae*1000.   ! ideal gas initial mass release rate, kg/s     
      qmax    = rqemit/rhoref*3600.*24./1000.   ! max volume flow released, e3m3/d
      frel    = 1.-(plocal+pressfinal)/(plocal+pinit)   ! fraction released
c      tgasout = teig - rk0    ! degrees C
      tgasout = teig    ! degrees K
      
      ierr = 0
      return
      end
      
      
c----------------------------------------------------------------------
c iblowdown
c----------------------------------------------------------------------
c
c     Calculates a sequence of flowrates based upon an exponential
c     blowdown curve
c
c     Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c
c --- INPUTS
c     qmax          :real   :initial volume flow rate, e3m3/d (ideal gas)
c     qtotal        :real   :total volume released, e3m3 (ideal gas)
c     frel          :real   :fraction of volume released
c     mblowdown   : integer:  
c                     0: steady
c                     1: vessel blowdown 
c                     2: user specified qmax
c                     3: depressurization Qave, 50% in 15min
c                     4: depressurization Qmax, 50% in 15min
c                     5: depressurization Qave, 50% in time Tdep
c     mdist   : integer   :puff distribution selection
c                     1: constant duration
c                     2: constant mass
c
c --- OUTPUTS
c     ipuffdur  : integer : puff duration, s
c     npuffs    : integer : number of puffs used to represent blowdown
c     pseq(mxpuff,4) : real :
c         pseq(*,1) : flowrate e3m3/d
c         pseq(*,2) : duration, s
c         pseq(*,3) : rH2S
c         pseq(*,4) : 
c
c     ierr:   integer: 0 : no errors
c                      1 : errors were found
c----------------------------------------------------------------------
      function iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &    mblowdown,mdist)
     &result(ierr)
c includes
      use mod_stddefs 
      use mod_globals
c INPUTS
      real(8), intent (in)  :: qmax     ! intital, maximum flow rate, e3m3/d
      real(8), intent (in)  :: qtotal   ! total volume contained, e3m3
      real(8), intent (in)   :: frel     ! fraction of volum/mass released
      integer, intent (in out)   :: ipuffdur   ! puff duration, s
      real(8) :: pseq(mxpuff,4) ! puff average mass puffs in blowdown
      integer, intent (in out) :: npuffs  ! number of puffs used to represent blowdown
      integer, intent (in) :: mblowdown   ! 1:user qmax; 2:vessel exponential model selection
      integer, intent (in) :: mdist       ! puff distribution selection
c DECLARATIONS
      real(8)  :: frel1
      real(8)  :: tconst, tconsti 
      real(8)  :: r1,t0,t1,m0,m1,cum
      real(8) :: duration
      integer :: i
c      include 'h_interface.for'
      
c calculate the release time constant (seconds)
      tconst = timeconstant(qmax, qtotal)
      tconsti = -1.e30
      if(tconst .gt. 0 ) then
          tconsti = -1./tconst
      end if 

c limit the actual amount released 
      frel1    = min(frel,rmxfrel)  ! set limit

c actual duration      
      duration = -tconst*log(1.-frel1)    ! seconds

      if(npuffs .gt. mxpuff) then
          write(6,*) '***ERROR*** npuffs exceeds maximum puffs'
          write(6,*) 'npuffs    :',npuffs
          write(6,*) 'max puffs :',mxpuff
          STOP 'HALTED in routine BLOWDOWN()'
      end if 

c----------------------------------------------------------------------
c constant volume blow down
      if( MDIST .eq. 1)  then
          
c for user input determine npuffs and duration     
      if(npuffs.gt.1) then
          ipuffdur = nint(duration/npuffs)
      else
          npuffs   = nint(duration/ipuffdur)
      end if 
      duration = float(npuffs)*float(ipuffdur)
      
      r1    = qtotal*(24.*60.*60.)/float(ipuffdur   )
      cum   = 0.
      t0    = 0.
      t1    = t0 + ipuffdur
      m0    = 0
      m1    = exp(tconsti*t0) - exp(tconsti*t1)
      cum   = m1 + cum

      do 100 i=1,npuffs
c --- if mass in proposed puff is greater than limit
c     then truncate
          if(cum .gt. frel1) then
              m1 = frel1 - (cum-m1)
          end if 
          pseq(i,1)  = r1*(m1)
          pseq(i,2)  = ipuffdur
          pseq(i,3)  =  -1.   ! use default
          pseq(i,4)  =  -1.   ! use default
          
          t0       = t1
          m0       = m1
          t1       = t0 + ipuffdur
          m1       = exp(tconsti*t0) - exp(tconsti*t1)
          cum      = m1 + cum
 100  end do
c --- ensure that puffs selection is not missing some mass
c     else add it to the last puff
c     typically this shouldn't be required since NINT(...) above adds
c     an extra puff if required.
      if (cum .lt. frel1) then
          pseq(npuffs,1) = pseq(npuffs,1) + r1*(frel1-cum)
      end if

c----------------------------------------------------------------------
      else if( MDIST .eq. 2) then
c --- constant mass puffs           
c mass fraction released on each puff step      
      r1    = frel/float(npuffs)
      t0    = 0.0
      do 200 i=1,npuffs
          t1         = -tconst*log(1.-float(i)*r1)    ! second
          pseq(i,2)  = t1  -t0                           ! seconds duration
          vol  = qmax/(3600.*24.)*
     &          (EXP(-t0/tconst)-EXP(-t1/tconst))*(tconst)*1000.
          pseq(i,1)  = vol/(t1-t0)*3600.*24./1000. ! e3m3/d
          pseq(i,3)  =  -1.   ! use default
          pseq(i,4)  =  -1.   ! use default
          t0 = t1
 200  end do

      else
          write(6,*) 'MDIST=',MDIST 
          STOP 'HALTED in routine BLOWDOWN()'
      end if 
      
      ierr   = 0
      return
      end





c----------------------------------------------------------------------
c timeconstant
c----------------------------------------------------------------------
c
c     Returns calculated time constant (seconds)
c
c     Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     qmax          :real   :initial volume flow rate, e3m3/d (ideal gas)
c     qtotal        :real   :total volume released, e3m3 (ideal gas)
c
c --- OUTPUTS
c
c----------------------------------------------------------------------
      function timeconstant(qmax, qtotal)
c INPUTS
      real(8), intent (in)  :: qmax     ! intital, maximum flow rate, e3m3/d
      real(8), intent (in)  :: qtotal   ! total volume contained, e3m3
      real(8) :: timeconstant  ! time constant
c DECLARATION
c      include 'h_interface.for'

      timeconstant = 0.
      if(qtotal .gt. 0. ) then
          timeconstant = qtotal/qmax  *24. * 60. * 60.
      end if 
c      timeconstant = tconst
      return
      end


c----------------------------------------------------------------------
      BLOCK DATA

c----------------------------------------------------------------------
      use mod_stddefs 
      use mod_globals
      
      end


c----------------------------------------------------------------------
c  setup1
c----------------------------------------------------------------------
c
c     hard coding of several variables
c
c     DEVELOPED by B.Zelt  www.zeltpsi.com
c
c --- UPDATES
c
c
c --- INPUTS
c     none
c
c --- OUTPUTS
c     none
c----------------------------------------------------------------------
      subroutine setup1()
      use mod_stddefs 
      use mod_globals
      
c standard constants
      effminERCB   = .99 ! Minimum Conversion and Combustion Efficiency ERCB
                         ! updated to 99% to match CSA Z620.2022   
      effmin   = .50    ! Minimum Conversion and Combustion Efficiency
      radloss  = .25    ! Flame Radiation Loss (RAD fraction)
      tgas  = 278.15    ! Raw Gas and Fuel Gas Temperature before Combustion
      tamb  = 278.15    ! Average Ambient Air Temperature
      nhvrec   = 20.    ! Minimum Recommended Net Heating Value, MJ/m (15C,101.325 kPa)
      nhvmin   = 12.    ! Minimum Allowed Net Heating Value MJ/m (15C,101.325 kPa)
      tradbkg  = 1.04   ! Background Thermal Radiation, KW/m2
      tradtot  = 4.73   ! Maximum Recommended Total Thermal Radiation
      u10min   = 1.     ! Minimum Wind Speed at 10 m reference height, m/s
      ustabf   = 1.     ! Stability Class F Plume Rise Criterion Wind Speed
      tgrad    = .035   ! Stability Class F Vertical Potential Temperature Gradient, K/m
      slopemx  = .05    ! Maximum Slope Criterion
      uofaa    = 133.3  ! UofA Flare Research Project constant A, (MJ/kg)^3
      uofab    = .317   ! UofA Flare Research Project constant B
      tref     = 288.15   ! K
      pref     = 101.325  ! kPa (absolute)
      rlapse    = 0.0065
c      uemin  =0.1D0
      uemin  =0.00001D0 ! minimum pseudo exit velocity, m/s
c      fn2air   = 0.791 ! now using input gas composition of air

      wsmin = 1.0       ! minimum wind speed 
      wsmax = 10.0      ! maximum wind speed 

      spv      = ru*tref/pref
c Heat Release Rate Constant (K1)
c (caldm)/(s10mMJ)
      rk1      = 1000./24./3600.*1000000./4.1868  !
c SO2 Emission Rate Constant (K2)
c (gd)/(s10m%H2S)
      rk2      = 1000./24./3600./spv*prop(ipmmass,jpso2)*1000./100.
c Heat Release Rate Constant (K4)
c (Jdm)/(s10mMJ)
      rk4      = 1000./24./3600.*1000000. 

c reset properties for molar specific heat
      do i=1,mxcomp
         prop(ipcpmole,i)  = calc_cp(i, tref)
         prop(ipcpmass,i)  = prop(ipcpmole,i)/prop(ipmmass,i)
      end do 

c initialize the properties to get exact air values      
      call mk_aircoefficents()

      return
      end

c----------------------------------------------------------------------
c InitLocal
c----------------------------------------------------------------------
c
c  Initialize local/site variables
c
c     DEVELOPED B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     elev    : real  : site elevation (m) above sea level
c
c --- OUTPUTS
c     plocal : real: (common block)  : site pressure
c 
c----------------------------------------------------------------------
      subroutine InitLocal(elev)
      use mod_stddefs
      use mod_globals
      real(8) :: elev

c initialize local pressure from the elevation
      plocal   = calc_plocal(elev)

      return
      end

c----------------------------------------------------------------------
c calc_totalrate()
c----------------------------------------------------------------------
c
c     Calculates the total rate
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     rate    : real  : raw gas flow rate
c     fgr     : real  : fuel gas to raw gas ratio
c
c --- OUTPUTS
c     calc_totalrate :    real : total flow rate =(raw gas + fuel gas)
c
c----------------------------------------------------------------------
      function calc_totalrate(rate, fgr)
c INPUTS
      real(8)  :: rate
      real(8)  :: fgr
      real(8) :: calc_totalrate 
      calc_totalrate = rate *(1.+fgr)
      return
      end 

c----------------------------------------------------------------------
c calc_fuelrate()
c----------------------------------------------------------------------
c
c     calculates the fuel gas flow rate from the total rate
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     rate    : real :  total flare rate (=raw gas + fuel gas rate)
c     fgr     : real  : fuel gas to raw gas ratio
c
c --- OUTPUTS
c     calc_fuelrate   : real : calculated fuel gas rate
c
c
c----------------------------------------------------------------------
      function calc_fuelrate(rate, fgr)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: rate
      real(8)  :: fgr
      real(8)   :: calc_fuelrate 
      calc_fuelrate = rate *(fgr)
      return
      end 

      
c----------------------------------------------------------------------
c initialize
c----------------------------------------------------------------------
c
c     initializes a gas composition to a value = rval
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp) : real  : gas composition
c     rval        : real  : initialization value
c
c --- OUTPUTS
c     returns   0: no errors
c               1: errors were found
c----------------------------------------------------------------------
      function initialize(mix,rval)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8)  :: rval
      integer :: initialize
c DECLARATIONS
      integer   :: j
      do j=1,mxcomp
          mix(j) =   rval
      end do
      initialize = 0
      return 
      end

      
c----------------------------------------------------------------------
c irenormalize
c----------------------------------------------------------------------
c 
c     Re-Normlized gas composition  
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
      function irenormalize(mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: mix(mxcomp)
      integer :: irenormalize
c DECLARATIONS
      integer   :: j
      real(8) :: sum 
      sum  = 0.0d0
      do j=1,mxcomp
          if( prop(ipcalc,j) .gt. 0) then
              sum = sum + mix(j)
          end if 
      end do
      do j=1,mxcomp
          if( prop(ipcalc,j) .gt. 0) then
              mix(j) = mix(j)/sum
          end if 
      end do

      irenormalize = 0
      return 
      end

c----------------------------------------------------------------------
c inormalize
c----------------------------------------------------------------------
c 
c     Creates a normlized gas composition from raw gas, fuel gas 
c     and fgr 
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     raw(mxcomp)     : real : raw gas composition
c     fuel(mxcomp)    : real : fuel gas composition
c     fgr             : real : fuel gas to raw gas ratio
c
c --- OUTPUTS
c     mix(mxcomp) : real : mixed gas composition
c
c --- RETURNS
c     inormalize = 0: no errors 
c                = 1: errors were found
c----------------------------------------------------------------------
      function inormalize(raw, fuel, fgr, mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: raw(mxcomp)
      real(8) :: fuel(mxcomp)
      real(8) :: mix(mxcomp)
      real(8)  :: fgr
      integer :: inormalize
c DECLARATIONS
      integer   :: j
      do j=1,mxcomp
          if( prop(ipcalc,j) .gt. 0) then
              mix(j) =   (raw(j)+fgr*fuel(j))/(1.+fgr)
          end if 
      end do

      inormalize = 0
      return 
      end

c----------------------------------------------------------------------
c inormalizeh2s
c----------------------------------------------------------------------
c
c     Normalizes a gas composition with the addition of a new H2S 
c     mole fraction
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     raw(mxcomp)     : real : raw gas composition
c     fh2s            : real : mole fraction of H2S
c
c --- OUTPUTS
c     mix(mxcomp) : real : mixed gas composition
c
c --- RETURNS
c     inormalizeh2s = 0: no errors 
c                   = 1: errors were found
c
c
c----------------------------------------------------------------------
      function inormalizeh2s(raw, fh2s, mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: raw(mxcomp)
      real(8) :: fh2s
      real(8) :: mix(mxcomp)
      integer :: inormalizeh2s
c DECLARATIONS
      real(8)      :: fnorm
      integer   :: j
      if(raw(jph2s).eq.1.) then
         do j=1,mxcomp
            mix(j) =   0.0
         end do
      else    
         fnorm    = (1.-fh2s)/(1.-raw(jph2s))
         do j=1,mxcomp
             if( prop(ipcalc,j) .gt. 0) then
                 mix(j) =   raw(j)*fnorm
             end if 
         end do
      end if    
      mix(jph2s) = fh2s
      inormalizeh2s = 0
      return 
      end

c----------------------------------------------------------------------
c calc_molarmass()
c----------------------------------------------------------------------
c
c     Calculates the molar mass from a gas mixture
c
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c     
c --- OUTPUTS
c
c --- RETURNS
c     calc_molarmass  : real : molar mass  (kg/kmol)
c
c----------------------------------------------------------------------
c --- kg/kmol
      function calc_molarmass(mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(*)  ! mxcomp
      real(8) :: calc_molarmass      
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

      calc_molarmass = calc_byprop(mix,ipmmass,ipcalc)
      return 
      end

c----------------------------------------------------------------------
c calc_nhv
c----------------------------------------------------------------------
c
c     Calculates the net heating value (lower heating value) from a 
c     gas composition mixture
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_nhv    :real   : net heating value  (MJ/m3)
c
c----------------------------------------------------------------------
c --- MJ/m3
      function calc_nhv(mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_nhv      
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

      calc_nhv = calc_byprop(mix, ipnhv, ipcalc)
      return 
      end

c----------------------------------------------------------------------
c calc_ghv()
c----------------------------------------------------------------------
c
c     Calculates the gross heating value (higher heating value) of a 
c     gas composition mixture
c
c     DEVELOPED B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_ghv : real : gross heating value  (MJ/m3)
c----------------------------------------------------------------------
c --- MJ/m3
      function calc_ghv(mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_ghv      
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

      calc_ghv = calc_byprop(mix, ipghv, ipcalc)
      return 
      end

c----------------------------------------------------------------------
c calc_nhvmass
c----------------------------------------------------------------------
c
c     Calculates the net heating value in mass units
c
c     Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mmass   : real : molar mass
c     rnhv    : real : net heating value  (MJ/m3)
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_nhvmass : real: net heat value (MJ/kg)
c
c --- uses
c     spv : real: specific volume
c----------------------------------------------------------------------
      function calc_nhvmass(mmass, rnhv)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mmass, rnhv
      real(8) :: calc_nhvmass
      
      calc_nhvmass= rnhv/mmass * spv
      return 
      end


      
c----------------------------------------------------------------------
c calculate coefficients for air based upon air ref composition      
c----------------------------------------------------------------------
      subroutine  mk_aircoefficents()
      use mod_stddefs
      use mod_globals
c INPUTS
c      real(8) :: calc_aircoefficents
c DECLARATIONS
      real(8) :: calc
      real(8) :: mix(mxcomp) ! (mxcomp)
      real(8) :: coeff(mxnasa)
      integer :: inprop
      integer :: incalc
      integer  :: i
      integer  :: j
      integer :: ier
      real(8)  :: rinc

      do i=1,mxcomp
         mix(i) = prop(ipair,i)
      end do 
      ier  = irenormalize(mix)      
      
c calculate the air molar mass
      prop(ipmmass,jpair)=calc_molarmass(mix)
      
c calculate the nasa coefficients       
      call mk_coeff(mix, coeff)
      
      do i=1,ipbb2-ipa1+1
         incalc = ipa1-1+i
         prop(incalc,jpair) = coeff(i)
      end do 
      
      return 
      end

c----------------------------------------------------------------------
c creates a set of coefficients of properties based upon provide mix 
c----------------------------------------------------------------------
     
      subroutine mk_coeff(mix, coeff)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: coeff(mxnasa)
      real(8) :: tempk
c      real(8) :: calc_coeff      
c DECLARATIONS
      integer  :: j, i, incalc
      real(8) :: calc
      real(8) :: t1000

      do i=1,mxnasa
         incalc = ipa1-1+i
         calc =0.
         do j=1,mxcomp
            calc= calc +  mix(j) *prop(incalc ,j)
         end do
         coeff(i) = calc
      end do
c      calc_coeff = 0.
      return
      end 
      
c----------------------------------------------------------------------
c calc_byprop()
c----------------------------------------------------------------------
c
c     Calculates a property from a gas composition based upon
c     the requested inprop and incalc. inprop2
c     allows more complex calculations.
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c     mix      : mole fraction of gas mix
c     inprop   : property to sumproduct (see stddefs.h)
c     incalc   : conditional property
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c     inprop  : integer : property to be calculated
c     incalc  : integer : conditional property.  If the incalc property
c                     is set, then the component is included in the calculation
c     inprop2 : integer : OPTIONAL
c                     if inprop2 is set then this property
c                     is added to the product
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_byprop : real : sumproduct calculation
c----------------------------------------------------------------------
      function calc_byprop(mix, inprop, incalc, inprop2)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: calc_byprop      
      real(8) :: mix(*) ! (mxcomp)
      integer :: inprop
      integer :: incalc
      integer,optional :: inprop2
c DECLARATIONS
      integer  :: j
      real(8)  :: rinc

      calc_byprop =0.
      do j=1,mxcomp
          if( prop(incalc,j) .gt. 0) then
          
          r = prop(inprop,j)
              rinc = mix(j)*prop(inprop,j)
              if(present(inprop2)) then
                  rinc = rinc * prop(inprop2,j)
              end if 
              calc_byprop= calc_byprop+  rinc
          end if 
      end do
      return 
      end


c----------------------------------------------------------------------
c
c B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hydrocarbons(mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_hydrocarbons      
c DECLARATIONS
      integer  :: j

      calc_hydrocarbons = 0
      do j=1,mxcomp
          if( prop(iph,j) .gt. 0) then
              calc_hydrocarbons  = calc_hydrocarbons +  mix(j)
          end if 
      end do
      return 
      end

c --- 
c----------------------------------------------------------------------
c
c B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_lfl(mix )
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_lfl      
c DECLARATIONS
      integer  :: j
      real(8)     :: sum_comb

      sum_comb =0.
      calc_lfl =0.
      do j=1,mxcomp
          if( prop(iplfl,j) .gt. 0.) then
              calc_lfl= calc_lfl+ mix(j)*1./prop(iplfl,j)*prop(ipcomb,j)
              sum_comb = sum_comb + mix(j)*prop(ipcomb,j)
          end if 
      end do
      if(calc_lfl .gt. 0.) then
          calc_lfl = 0.01/(calc_lfl/sum_comb)
      end if
      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c
c NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c http://www.grc.nasa.gov/WWW/CEAWeb/ceaguiDownload-win.htm
c ' cp0/R=a1T^-2 +a2T^-1+a3+a4T+a5T^2+a6T^3+a7T^4
c
c
c --- UPDATES
c     170120 - updated to NASA Calcs
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_cp      
c DECLARATIONS
        
      calc_cp = calc_cp_coeff(prop(ipa1,jcomp),tempk)
      return 
      end

c----------------------------------------------------------------------
c----------------------------------------------------------------------
      function calc_cp_coeff(coeff,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      real(8) :: coeff(mxnasa)
      real(8) :: calc_cp_coeff      
c DECLARATIONS
      real(8) :: t1000
      real(8) :: sum
      integer :: ii
      real(8) :: a
      real(8) :: b

      t1000 = 1000.   ! default match temperature K         
      if( tempK .gt. t1000) then
         ii = ipb1-ipa1+1
      else
         ii = ipa1-ipa1+1
      end if 
      ii = ii -1
      
      sum = 0.
      do i = 1, 7
         sum = sum + coeff(ii+i) * tempk**real(i - 3)
      end do 
         
      calc_cp_coeff = sum * ru
      return 
      end

      
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c
c NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c http://www.grc.nasa.gov/WWW/CEAWeb/ceaguiDownload-win.htm
c  delH_T = (H_t-H_Tref)
c
c
c --- UPDATES
c     170120 - updated to NASA Calcs
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      Function calc_hf(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf      
c DECLARATIONS
      real(8) :: h, href
c     tref is defined in h_stddefs.for   

      h = calc_hf1(jcomp,tempk) !*tempk
      href = calc_hf1(jcomp,Tref)!* Tref
      calc_hf = (h - href) !/ tempk
      End Function

      
c----------------------------------------------------------------------
c----------------------------------------------------------------------
      Function calc_hf_coeff(coeff,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      real(8) :: coeff(mxnasa)
      real(8) :: calc_hf_coeff      
c DECLARATIONS
      real(8) :: h, href

      h = calc_hf1_coeff(coeff,tempk) !*tempk
      href = calc_hf1_coeff(coeff,Tref)!* Tref
      calc_hf_coeff = (h - href) !/ tempk
      End Function
      
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c
c NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c http://www.grc.nasa.gov/WWW/CEAWeb/ceaguiDownload-win.htm
c H0_T/(RT)=-a1T^-2 +a2ln(T)/T +a3 +a4/2*T +a5/3*T^2 +a6/4*T^3 +a7/5*T^4 + b1/T
c
c
c --- UPDATES
c     170120 - updated to NASA Calcs
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c     returns h/(Ru*T)
c----------------------------------------------------------------------
      function calc_hf1(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf1      
c DECLARATIONS

      calc_hf1 = calc_hf1_coeff(prop(ipa1,jcomp), tempk)
      return 
      end

c
c
      function calc_hf1_coeff(coeff,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: tempk
      real(8) :: coeff(mxnasa)
      real(8) :: calc_hf1_coeff
c DECLARATIONS
      real(8) :: t1000
      real(8) :: sum
      integer :: ii

      t1000 = 1000.   ! default match temperature K         
      if( tempK .gt. t1000) then
         ii = ipb1-ipa1+1
      else
         ii = ipa1-ipa1+1
      end if 
      ii = ii -1
      
      sum = 0
      i = 1
      sum = sum - coeff(ii+i) * (tempk**real(i - 3))
      i = 2
      sum = sum + coeff(ii+i)* Log(tempk) / tempk
c      do i = 3,7
c         sum = sum + coeff(ii+i) * (tempk**real(i - 3)) / real(i - 2)
c      end do !Next i
      i = 3
      sum = sum + coeff(ii+i) * (tempk**real(i - 3)) / real(i - 2)
      i = 4
      sum = sum + coeff(ii+i) * (tempk**real(i - 3)) / real(i - 2)
      i = 5
      sum = sum + coeff(ii+i) * (tempk**real(i - 3)) / real(i - 2)
      i = 6
      sum = sum + coeff(ii+i) * (tempk**real(i - 3)) / real(i - 2)
      i = 7
      sum = sum + coeff(ii+i) * (tempk**real(i - 3)) / real(i - 2)
      i = 8
      sum = sum + coeff(ii+i) / tempk     
         
      calc_hf1_coeff = sum  * ru * tempk
      return 
      end
      
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c http://www.grc.nasa.gov/WWW/CEAWeb/ceaguiDownload-win.htm
c 'So(T)/R = a1T2/2  a2T1 + a3lnT + a4T + a5T 2/2 + a6T 3/3 + a7T 4/4 + b2
c
c
c --- UPDATES
c     170120 - updated to NASA Calcs
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_s(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_s      
c DECLARATIONS
      real(8) :: t1000
      real(8) :: sum, rk
      integer :: ii

      t1000 = 1000.   ! default match temperature K         
      if( tempK .gt. t1000) then
         ii = ipb1
      else
         ii = ipa1
      end if 
      ii = ii -1
      
      sum = 0.
      do i = 1,7
         rk = 1.
         If (i .lt. 3) Then
            rk = -1.
         End If
         If (i .eq. 3) Then
            sum = sum + rk * prop(ii+i,jcomp) * Log(tempk)
         Else
            sum = sum + rk * prop(ii+i,jcomp) * (tempk**(i - 3)) / Abs(real(i - 3))
         End If
      end do ! next i
      i = 9
      sum = sum + prop(ii+i,jcomp)
         
      calc_s = sum * ru
      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf
c http://www.grc.nasa.gov/WWW/CEAWeb/ceaguiDownload-win.htm
c 'So(T)/R = a1T2/2  a2T1 + a3lnT + a4T + a5T 2/2 + a6T 3/3 + a7T 4/4 + b2
c
c
c --- UPDATES
c     170120 - updated to NASA Calcs
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_g(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_g      
c DECLARATIONS
      real(8) :: t1000
      real(8) :: sum, rk, a1,a2,a3,a4,a5,a6,a7,b1,b2
      integer :: ii

      t1000 = 1000.   ! default match temperature K         
      if( tempK .gt. t1000) then
         ii = ipb1
      else
         ii = ipa1
      end if 
      ii = ii -1
      a1 = prop(ii+1,jcomp) 
      a2 = prop(ii+2,jcomp) 
      a3 = prop(ii+3,jcomp)
      a4 = prop(ii+4,jcomp) 
      a5 = prop(ii+5,jcomp) 
      a6 = prop(ii+6,jcomp) 
      a7 = prop(ii+7,jcomp) 
      b1 = prop(ii+8,jcomp) 
      b2 = prop(ii+9,jcomp) 
      
      sum = -0.5 * a1 / (tempk*tempk) 
     &     + a2 * (1. + Log(tempk)) / tempk 
     &     + a3 * (1. - Log(tempk)) 
     &     - a4 * tempk / 2. 
     &     - a5 * (tempk*tempk) / 6. 
     &     - a6 * (tempk*tempk*tempk) / 12. 
     &     - a7 * (tempk*tempk*tempk*tempk) / 20. 
     &     + b1 / tempk 
     &     - b2
         
      calc_g = sum * ru
      return 
      end
      
      
    
c----------------------------------------------------------------------
c
c B.Zelt - www.zeltpsi.com
c
c Calculations the autoignition temperature, K 
c v130113
c 
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_autoig(mix )
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8)  :: calc_autoig      
c DECLARATIONS
      integer  :: j
      real(8)  :: sum_comb

      sum_comb =0.
      calc_autoig =0.
      do j=1,mxcomp
          if( prop(ipautoig,j) .gt. 0.) then
              calc_autoig= calc_autoig+ mix(j)*prop(ipautoig,j)*
     &              prop(ipcomb,j)
              sum_comb = sum_comb + mix(j)*prop(ipcomb,j)
          end if 
      end do
      if(sum_comb .gt. 0.) then
          calc_autoig= calc_autoig/sum_comb
      end if
      calc_autoig = calc_autoig + rk0
      return 
      end
      
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c Obsolete ... now using 
c ' NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf      
c
c ' Burcat
c old
c    Hf = aT + b/2T^2 + c/3T^3 + d/4T^4 
c new 
c    Eq. 91: H0_T/(RT)=a1 +a2/2*T +a3/3*T^2 +a4/4*T^3 +a5/5*T^4 +a6/T
c
c --- UPDATES
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hf_obsolete(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf_obsolete      
c DECLARATIONS
      real(8) :: cp_tref, cp_lo, cp_hi,a1corr,b1corr
      real(8) :: t1000

      cp_tref = calc_cp_ref(jcomp,tref)
      a1corr = (prop(ipcpmole,jcomp)-cp_tref)/ru

      t1000 = 1000.   ! default match temperature K         
      if( tempK .le. t1000) then
          calc_hf = ((prop(ipa1,jcomp)+a1corr)+
     &    tempk*(prop(ipa2,jcomp)/2.+
     &    tempk*(prop(ipa3,jcomp)/3.+
     &    tempk*(prop(ipa4,jcomp)/4.+
     &    tempk*prop(ipa5,jcomp)/5. ))))
          calc_hf = calc_hf* tempk*ru
c calc a6 correction for <1000K
          hf_lo = calc_hf_ref(jcomp,tref) +a1corr*1000.*ru
          calc_hf_obsolete = calc_hf + hf_lo
      else
          cp_lo = calc_cp(jcomp,t1000)
          cp_hi = calc_cp_hi(jcomp,t1000)
          b1corr = (cp_lo - cp_hi)/ ru
          calc_hf = ((prop(ipb1,jcomp)+b1corr)+
     &    tempk*(prop(ipb2,jcomp)/2.+
     &    tempk*(prop(ipb3,jcomp)/3.+
     &    tempk*(prop(ipb4,jcomp)/4.+
     &    tempk*prop(ipb5,jcomp)/5. ))))
          calc_hf = calc_hf* tempk*ru
c calc a6 correction for 1000K
          hf_lo = calc_hf_ref(jcomp,t1000) +a1corr*1000.*ru
          hf_hi = calc_hf_hi(jcomp,t1000)  +b1corr*1000.*ru
          calc_hf_obsolete = calc_hf + (hf_lo-hf_hi)
      end if

      return 
      end

c
c
c B.Zelt - www.zeltpsi.com
c
c Obsolete ... now using 
c ' NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf      
c
c 
c Burcat
c --- UPDATES
c
c --- INPUTS
c       tempk : K
c
c --- OUTPUTS
c
c --- RETURNS
      function calc_hf_ref(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf_ref      
c DECLARATIONS

      calc_hf_ref= tempk*((prop(ipa1,jcomp))+
     &tempk*(prop(ipa2,jcomp)/2.+
     &tempk*(prop(ipa3,jcomp)/3.+
     &tempk*(prop(ipa4,jcomp)/4.+
     &tempk*prop(ipa5,jcomp)/5. )))) *ru

      return 
      end

c----------------------------------------------------------------------
c
c
c B.Zelt - www.zeltpsi.com
c
c
c Obsolete ... now using 
c ' NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf      
c
c 
c Burcat
c --- UPDATES
c
c --- INPUTS
c     temp : K
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hf_hi(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf_hi      
c DECLARATIONS

      calc_hf_hi= tempk*((prop(ipb1,jcomp))+
     &tempk*(prop(ipb2,jcomp)/2.+
     &tempk*(prop(ipb3,jcomp)/3.+
     &tempk*(prop(ipb4,jcomp)/4.+
     &tempk*prop(ipb5,jcomp)/5. )))) *ru

      return 
      end



c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c 
c ' NASA Glenn Coefficients for Calculating Thermodynamic Properties.pdf      
c
c 
c Burcat
c calculates the molar average enthalpy
c    Eq. 91: H0_T/(RT)=a1 +a2/2*T +a3/3*T^2 +a4/4*T^3 +a5/5*T^4 +a6/T
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hfbar(mix,temp)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_hfbar            
c DECLARATIONS
      integer  :: j
c      include 'h_interface.for'

      calc_hfbar=0.
      do j=1,mxcomp
          calc_hfbar= calc_hfbar+  calc_hf( j,temp)* mix(j)
      end do

      return 
      end

c----------------------------------------------------------------------
c 
c B.Zelt - www.zeltpsi.com
c calculates the mix enthalpy
c NASA
c
c --- UPDATES
c     131128 - changed default temperature to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c    return h/(Ru*T)
c----------------------------------------------------------------------
      function calc_hf1mix(mix,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_hf1mix      
c DECLARATIONS
      integer  :: j, i
      real(8) :: coeff(mxnasa)
      real(8) :: calc
      real(8) :: t1000

      call mk_coeff(mix, coeff)      
     
      calc_hf1mix = calc_hf1_coeff(coeff, tempk)

      return 
      end
      
c----------------------------------------------------------------------
c OBSOLETE
c B.Zelt - www.zeltpsi.com
c calculates the mix enthalpy
c    Eq. 91: H0_T/(RT)=a1 +a2/2*T +a3/3*T^2 +a4/4*T^3 +a5/5*T^4 +a6/T
c
c --- UPDATES
c     131128 - changed default temperature to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hf1mix_obsolete(mix,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_hf1mix_obsolete      
c DECLARATIONS
      integer  :: j
      real(8) :: a(7)
      real(8) :: b(7)
      real(8) :: t1000
c      include 'h_interface.for'
      interface
      function calc_cpbar(mix,tempk,inprop2)
         real(8) :: mix(*)
         real(8) :: tempk
         real(8) :: calc_cpbar      
         integer,optional :: inprop2
      end function         
      end interface

      
c calc MIX
      do i=1,7
          a(i) = 0.
          b(i) = 0.
      end do
      do j=1,mxcomp
          do i=1,5
              a(i) = a(i) + prop(ipa1+i-1,j) * mix(j) * ru
              b(i) = b(i) + prop(ipb1+i-1,j) * mix(j) * ru
          end do
      end do 
      a(6) = 0.0
      b(6) = 0.0

      t1000 = 1000. ! match temperature
      
      cp_ref= (a(1)+
     &tref*(a(2)+
     &tref*(a(3)+
     &tref*(a(4)+
     &tref*a(5) ))))
      acorr = calc_cpbar(mix,tref) - cp_ref

      if( tempK .gt. t1000) then
          cp_lo= (a(1)+ acorr+
     &    t1000*(a(2)+
     &    t1000*(a(3)+
     &    t1000*(a(4)+
     &    t1000*a(5) ))))

          cp_hi= (b(1)+
     &    t1000*(b(2)+
     &    t1000*(b(3)+
     &    t1000*(b(4)+
     &    t1000*b(5) ))))
          bcorr = (cp_lo - cp_hi)
c calculate b6 correction
          hf_lo= 0.
          hf_hi= 0.
          do i=5,1,-1
              hf_lo= t1000*(a(i)/float(i) + hf_lo)
              hf_hi= t1000*(b(i)/float(i) + hf_hi)
          end do
          hf_lo = hf_lo + acorr*t1000*ru
          hf_hi = hf_hi + bcorr*t1000*ru
          b(6) = (hf_lo-hf_hi)
          b(1) = b(1) + bcorr

      end if
      a(1) = a(1) + acorr

      if(tempK .le. t1000) then
          do i=1,6
              b(i) = a(i)
          end do
      end if 

c calc h
      calc_hf1mix = 0.
      do i=5,1,-1
          calc_hf1mix= tempK*(b(i)/float(i) + calc_hf1mix)
      end do
      calc_hf1mix_obsolete= calc_hf1mix+ b(6)

      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c calculates the mix enthalpy
c    Eq. 90: cp0/R=a1 +a2*T +a3*T^2 +a4*T^3 +a5*T^4 
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cpmix(tempk,mix)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_cpmix      
c DECLARATIONS
      integer  :: j, i
      real(8) :: coeff(mxnasa)
      real(8) :: calc
      real(8) :: t1000

      call mk_coeff(mix, coeff)
      
      calc_cpmix = calc_cp_coeff(coeff, tempk)

      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c pass T in C
c   Eq. 90: cp0/R=a1 +a2T +a3*T^2 +a4*T^3 +a5*T^4
c
c --- UPDATES
c     131128
c        changed temperatue to default K 
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_obsolete(jcomp,tempk)
      use mod_stddefs
      use mod_globals
c INPUTS
      integer :: jcomp
      real(8)     :: tempk
      real(8) :: calc_cp_obsolete      
c DECLARATIONS
      real(8)      :: cp_tref
      real(8) :: t1000
c      include 'h_interface.for'

      cp_tref = calc_cp_ref(jcomp,tref)
      a1corr  = (prop(ipcpmole,jcomp)-cp_tref)/ru

      t1000 = 1000.  ! match point at 1000.K
      if( tempK .le. t1000 ) then
          cp_lo   = calc_cp_ref(jcomp,tempk)
          calc_cp_obsolete= cp_lo + a1corr*ru
      else
          cp_lo = calc_cp_ref(jcomp,t1000) + a1corr*ru
          cp_hi = calc_cp_hi(jcomp,t1000)
          b1corr = (cp_lo - cp_hi)/ ru
          calc_cp_obsolete= calc_cp_hi(jcomp,tempk)+ ru*b1corr
      end if
      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_ref(jcomp,tempk)
c MODULES      
      use mod_stddefs
      use mod_globals
c INPUTS
      integer :: jcomp
      real(8) :: tempk
      real(8) :: calc_cp_ref      
c DECLARATIONS

      calc_cp_ref= (prop(ipa1,jcomp)+
     &tempk*(prop(ipa2,jcomp)+
     &tempk*(prop(ipa3,jcomp)+
     &tempk*(prop(ipa4,jcomp)+
     &tempk*prop(ipa5,jcomp) )))) * ru
      return
      end



c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c pass T in C
c   Eq. 90: cp0/R=a1 +a2T +a3*T^2 +a4*T^3 +a5*T^4
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_hi(jcomp,tempk)
c MODULES      
      use mod_stddefs
      use mod_globals
c INPUTS
      integer :: jcomp
      real(8)     :: tempk
      real(8) :: calc_cp_hi      
c DECLARATIONS

      calc_cp_hi= (prop(ipb1,jcomp)+
     &tempk*(prop(ipb2,jcomp)+
     &tempk*(prop(ipb3,jcomp)+
     &tempk*(prop(ipb4,jcomp)+
     &tempk*prop(ipb5,jcomp) )))) * ru

      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c pass T in C
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cpbar(mix,tempk,inprop2)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_cpbar      
      integer,optional :: inprop2
c DECLARATIONS
      integer :: j
      real(8) :: rinc
c      include 'h_interface.for'

      calc_cpbar=0.
      do j=1,mxcomp
         rinc = mix(j)*calc_cp(j,tempk)
         if(present(inprop2)) then
            if(prop(inprop2,j).gt.0) then
               rinc = rinc * prop(inprop2,j)
            else
               rinc = 0.
            end if                
         end if 
         calc_cpbar= calc_cpbar+  rinc
      end do

      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c CPaverage
c
c --- UPDATES
c
c --- INPUTS
c     t1:
c     t2:
c     mix
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_ave(t1 , t2 , jcomp) 
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: mix(mxcomp)
      real(8) :: t1
      real(8) :: t2
      real(8) :: calc_cpaverage
      integer :: jcomp
c DECLARATIONS
      integer :: j
      real(8) :: rinc
      real(8) :: h1
      real(8) :: h2

      If (t2 .ne. t1) Then
      
c enthalpy method should be the same
         h2 = calc_hf(jcomp,t2)
         h1 = calc_hf(jcomp,t1)
   
         ! calc_cp_ave = (h2 * t2 * Ru  -  h1 * t1 * Ru) / (t2 - t1)
         calc_cp_ave = (h2   -  h1 ) / (t2 - t1)
      Else
         calc_cp_ave = calc_cp(jcomp, t1)
      End If
      
      return
      end
      

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c CPaverage from mixture
c
c --- UPDATES
c
c --- INPUTS
c     t1:
c     t2:
c     mix
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_ave_mix(t1 , t2 , mix) 
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: mix(mxcomp)
      real(8) :: t1
      real(8) :: t2
      real(8) :: calc_cpaverage
c      real(8) :: jcomp
c DECLARATIONS
      integer :: j
      real(8) :: rinc
      real(8) :: h1
      real(8) :: h2

c enthalpy method should be the same
      h2 = calc_hf1mix(mix,t2)
      h1 = calc_hf1mix(mix,t1)
   
      If (t2 .ne. t1) Then
         !calc_cp_ave_mix = (h2 * t2 * Ru  -  h1 * t1 * Ru) / (t2 - t1)
         calc_cp_ave_mix = (h2 -  h1) / (t2 - t1)
      Else
         calc_cp_ave_mix = calc_cpmix(t1, mix)
      End If
      
      return
      end
      
      
      
c----------------------------------------------------------------------
c wsstkht
c
c Calculate the wind speed at stack height
c
c B.Zelt - www.zeltpsi.com
c
c
c
c      
      subroutine wsstkht(wref, zref, ipg, hs, ws)      
c --- INPUTS
      real(8) :: wref   
      real(8) :: zref   
      real(8) :: hs   
      real(8) :: ws   
      integer :: ipg
c --- DECLARATIONS
      real(8) :: plx0(6)  ! wind profile power law exponents (6) 

c --- ISC rural defaults for wind profile power law exponents (6)
      data plx0/.07,.07,.10,.15,.35,.55/ 

      ws = wref
      
c --- Adjust Wind Speed -- Assume Wind Speed Constant Below 10 meters
      If (hs .ge. 10.) Then
         ws = ws * (hs / Zref)**plx0(ipg)
      ElseIf (Zref .gt. 10.) Then
         ws = ws * (10. / Zref)**plx0(ipg)
      End If

      return
      end

      
c----------------------------------------------------------------------
c sonic 
c
c Calculate the sonic velocity of a gas at temperture (temp)
c
c B.Zelt - www.zeltpsi.com
c
c
      subroutine sonicVelocity(cpmol, rmm, temp, z, sonic)

      use mod_stddefs
      
c --- INPUTS
      real(8) :: cpmol  ! molar specific heat, kJ/(kmolK) 
      real(8) :: rmm    ! molar mass, kg/kmole
      real(8) :: temp   ! temperature, K
      real(8) :: z      ! compressibility factor
c --- DECLARATIONS
      real(8) :: sonic  ! sonic velocity, m/s
      real(8) :: k      ! ratio of specific heats

      
      k = (cpmol/(cpmol-ru))
      sonic = sqrt(k * ru * temp * z * 1000. /rmm)
      
      return
      end 

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c Carbon efficiency
c from UofA Flare Research Project 
c (note molar efficiency same as mass efficiency for C and S)
c     
c     common :  rk13, grav
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
c
      function carbeff(rnhv, ws, dia, ue  )
      use mod_stddefs
      use mod_globals
c INPTUS
      real(8)  :: rnhv  ! net heating value of flared gas, MJ/kg
      real(8)  :: ws    ! m/s, wind speed at reference height
      real(8)  :: dia   ! m, flare stack tip inside diameter
      real(8)  :: ue    ! m/s, maximum exit speed at flare tip
      real(8)  :: carbeff 
c DECLARATIONS

c  Maximum Possible Conversion Efficiency 
      carbeff= (uofaa/rnhv**3.D0)
      carbeff= carbeff*exp(uofab*ws/(grav*ue*dia)**(rk13))
      carbeff = 1. - carbeff
      carbeff= max(effmin, carbeff )
      return
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c calc_plocal
c
c COMMON
c     pref; rlapse; tref;rk0;grav;ru;
c     plocal      
c Calculate site pressure
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_plocal(elev)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8):: elev
      real(8):: calc_plocal
      real(8) :: n
c DECLARATIONS

      calc_plocal = (1.-rlapse*elev/tref)
      n = grav/(ru*1000.)* prop(ipmmass,jpair)/rlapse 
      calc_plocal = calc_plocal**n
      calc_plocal = calc_plocal*pref
c spreadsheet rounds to 3
c ROUND(psite,3)
      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c calc_trad()
c Calculate the thermal radiation
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_trad(rate, rnhv, hs, zL)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: rate   ! flaring rate , m3/s
      real(8) :: rnhv   ! net heating value, MJ/m3
      real(8) :: hs     ! effective stack height, m
      real(8) :: zL     ! Brzustowski Flare Model, height, m
      real(8) :: calc_trad      
c DECLARATIONS
      real(8) :: heatreleased
      real(8) :: hheat
      real(8) :: radgnd

c total heat released
      heatreleased = rate*rnhv*radloss *1000000./1000.   ! with 25% loss
c Fireball Height at half of flame
      hheat        = hs -zL*(1.-0.82)
c radiation at the ground
      radgnd       = heatreleased /(4.*pi*hheat*hheat)
      calc_trad    = radgnd + tradbkg      
      return 
      end
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c Brzustowski Flare Model
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      subroutine brzflare(mix, dia, uexit, ws, rhoe, rhoa, XL, zL, FA)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(*)
      real(8)  :: dia   ! exit diamter,m
      real(8)  :: uexit ! exit velocity, m/s
      real(8)  :: ws    ! windspeed at stack top, m/s
      real(8)  :: rhoe  ! release density at ambient temperature & local pressure
      real(8)  :: rhoa  ! air density
      real(8)  :: XL
      real(8)  :: ZL
      real(8)  :: FA
c DECLARATIONS
c      real(8)  :: rLFL
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

c combustible fraction of mixture
      combfrac = calc_byprop(mix, ipcomb, ipcalc) 
c Lower Flammability Limit (LFL) of combustible flared gas
      rLFLcomb    = calc_lfl(mix)
c Lower Flammability Limit (LFL) of flared mixture
      rLFLflared  = 0.
      if(combfrac .gt.0.)then
          rLFLflared  = rLFLcomb/combfrac
      end if 
      rmm  = calc_molarmass(mix)       ! kg/kmole
      rmmair =  prop(ipmmass,jpair)  ! calc_molarmass(gcair) very similar to  ! kg/kmole
      rnhv = calc_nhv(mix)             ! MJ/m3

c Vj/Ua - Jet to Air velocity ratio 
      Vj_Ua = min(110.,uexit/ws)
c R - Jet to Air momentum flux ratio
      rjet = rhoe/rhoa*Vj_Ua*Vj_Ua
c CL - dimensionless lean limit concentration of fuel
      CL   = rLFLflared *Vj_Ua*(rmm/rmmair)
c SL - dimensionless coordinate along flame
      IF(CL .lt. 0.5) then 
          SL = 2.04*(CL**-1.03)
      else   
          SL = 2.51*(CL**-0.625)
      end if   
c XL - dimensionless horizontal coordinate along flame
      IF(CL .lt. 0.5) then
          XLL=SL-1.65
      else
          IF(SL .gt. 2.35) then
              XLL=SL-1.65
          else
              xguess = (SL/2.05)**(1/0.28)
              XLL=calc_XLL(xguess, SL)
          end if
      end if
c ZL - dimensionless vertical coordinate along flame
      ZLL   = 2.05*XLL**0.28
c zL - vertical coordinate upward from flare tip (used)
      ZL    = ZLL*dia*sqrt(rjet)
c xL - horizontal coordinate downwind from flare tip
      XL    = XLL*dia*sqrt(rjet)
c Angle from flare tip to flame tip, from horizontal
      FA    = DEGREES(ATAN2(ZL,XL))
      return
      end

c----------------------------------------------------------------------
c calc_XLL
c B.Zelt - www.zeltpsi.com
c
c
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_XLL(xinit, SL)
c INPUTS
      real(8) :: xinit
      real(8) :: SL
      real(8) :: calc_XLL 
c DECLARATIONS
      real(8) :: xguess 
      real(8) :: diff
      real(8) :: slope  

      xguess = xinit
      slope = 2.*1.04*xguess + 0.28*2.05*xguess**(0.28-1.)

      maxloops = 100
      do i=1,maxloops
          SLnew  = 1.04*xguess*xguess + 2.05*xguess**0.28
          slope  = 2.*1.04*xguess + 0.28*2.05*xguess**(0.28-1.)
          diff   = SLnew-SL
          xguess = xguess-diff/slope
          if (abs(diff) .lt. 0.00001) Then 
              goto 100
          end if
      end do 

c 100  xguess = (eGoal - Intercept) / slope
 100  calc_XLL = xguess
      return 
      end


c ---------------------------------------------------------------
c
c ---------------------------------------------------------------
      subroutine calc_hcp(tempk,fv,df)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8) :: tempk, fv, df 
c DECLARATIONS
      common /vars_hcp/ goalh, goalcoeff
      real(8) :: goalh
      real(8) :: goalcoeff(mxnasa)
      
      fv = calc_hf_coeff(goalcoeff, tempk)  - goalh
      df = calc_cp_coeff(goalcoeff, tempk)
      return 
      End 


c----------------------------------------------------------------------
c calc_flametemp(tguess,mix, nhv, exit_fuel_ratio)
c B.Zelt - www.zeltpsi.com
c 
c MIX CONTAINS
c air
c h2o
c o2
c N2
c CO2
c SO2
c
c --- UPDATES
c     131128 - change tref default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_flametemp(tinit, mix, eGoal)
      use mod_stddefs
      use mod_globals
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tinit
      real(8) :: eGoal
      real(8)  :: calc_flametemp
c DECLARATIONS
      real(8) :: calc
      real(8) :: tguess
      real(8) :: Intercept, difference, cp, enthalpy_tg
      real(8) :: enthalpy_tr
      integer :: i, maxloops
      real(8) :: tg0, tg1
      real(8) :: junk
        interface
        
          subroutine calc_hcp(X1,FV,DF)
            real(8),intent(in) :: X1
            real(8),intent(out) :: FV
            real(8),intent(out) :: DF
          end subroutine calc_hcp
          
         FUNCTION RTSAFE(FUNCD,X1,X2,XACC)
            external :: FUNCD
            real(8) :: x1
            real(8) :: x2
            real(8) :: xacc
         end function
        end interface   
      

      common /vars_hcp/ goalh, goalcoeff
      real(8) :: goalh
      real(8) :: goalcoeff(mxnasa)

      call mk_coeff(mix,goalcoeff)
      
      goalh = egoal
      tg0 = 200.
      tg1 = 3000.
      calc_flametemp  = rtsafe(calc_hcp, tg0, tg1, 0.00001)
      
      return       
      end 


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
      function iwrFLEMARBhdr(io,iodb,FIN)
c----------------------------------------------------------------------
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  writes the header for the FLEMARB.DAT
c               arbitrary data file
c
c --- UPDATES:
c --- INPUT:
cio        - integer    - io unit
ciodb        - integer    - io debug unit
cFIN       - characeter - path for file output
csname     - character  - source name
ciutmzn     - integer    - utm zone  11,12 are ALBERTA
cABTZ    - character  - time zone in UTC format
cUTMHEM      - character  - N or S, for hemisphere
cDATUM    - character  - projection used for data
cUNITS     - character - m or km used to reference location
cPMAP
cnspec     - integer    - number of chemical species emitted
cchemlst() - character  - names of the chemicals emitted
cmmlst()   - real       - molar weights of chemicals emitted
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      use mod_stddefs
      use mod_globals
      use mod_readinp
c INPUTS
      CHARACTER *mxpath :: FIN
      CHARACTER *8 :: dummy
      integer :: io
      integer :: iodb
      integer :: iwrFLEMARBhdr
c DECLARATIONS
      integer :: ier, nsrc,ncom,n
      real(8)  :: params(8)
      data params/1.,0.,0.,0.,0.,0.,0.,0./

      open(unit=io,file=FIN,iostat=ier,status='unknown')
      if(ier .ne. 0) goto 9000

      nsrc   = 1              ! number of sources 

      write(io,101) 
 101  format('FLEMARB.DAT     1.0             Initial configuration')
      ncom = 7
      write(io,102) ncom
 102  format(i4)
      write(io,*) "Time-invariant (8 variables) var1=stack 
     &input type  var2-var8=not used"
      write(io,*) "     var1=1:  T(K), U, V, W (m/s)"
      write(io,*) "     var1=2:  Fb, Fmx, Fmy, Fmz  
     &(i.e., buoyancy and momentum fluxes)"
      write(io,*) "Time-variant (if var1=1)"
      write(io,*) "  X(km),Y(km),Z(mAGL),Elev(mMSL),
     &D(m),T(K),U,V,W,Sigy(m),Sigz(m),Q(g/s)..."
      write(io,*) "Time-variant (if var1=2)"
      write(io,*) "  X(km),Y(km),Z(mAGL),Elev(mMSL),
     &D(m),Fb,Fmx,Fmy,Fmz,Sigy(m),Sigz(m),Q(g/s)..."
      n=LEN_REAL(PMAP)
      write(io,'(A)') PMAP(1:n)
 103  format(A3)
      write(io,104) iutmzn, UTMHEM
 104  format(i4,A1)
c      NAS-C   02-21-2003  
      n=LEN_REAL(DATUM)
      write(io,'(A)') DATUM(1:n)
 105  format(A)      
      if( MUNITS.eq.1 ) then
          write(io,106) '   M'
      else
          write(io,106) '  KM'
      end if 
 106  format(a4)     
      write(io,107) ABTZ
 107  format(a8)     

c 1990 009  04  0000  1990 009  10 0000
c    1   5
c 'SO2'   'NO'  'NO2'  'PM10'  'H2S'
c       64.000      30.000      46.000      30.000      34.000      
c 'Flare1'      1.0    0.0    0.0    0.0    0.0    0.0    0.0    0.0
      call julday(iodb,iyys,imms,idds,jjds)
      call julday(iodb,iyye,imme,idde,jjde)

      write(io,108) iyys,jjds,ihhs,isss, iyye,jjde,ihhe,isse
 108  format(3i4,1x,i4,2x,3i4,1x,i4)
      write(io,109) nsrc, nspec
 109  format(2i4)

c 110    format(("'",A,"'",3x))
      do i=1,nspec
          dummy = chemlst(i)
          n = LEN_REAL(dummy)
          write(io,110) chemlst(i)(1:n)
      end do

      write(io,*)

 110  format(("'",A,"'   "),$)


      write(io,111) (mmlst(i),i=1,nspec)
 111  format(10F12.3)
      
      n = LEN_REAL(sname)
      write(io,112) sname(1:n), (params(i),i=1,8)
 112  format("'",A,"'",8F7.1)       
      iwrFLEMARBhdr  = 0

      return 
 9000 iwrFLEMARBhdr = ier
      return 
      end

c----------------------------------------------------------------------
c     iwrFLEMARBi
c     
c --- Version: 1.001.0001    Level: 110706
c     B.Zelt - www.zeltpsi.com
c
c --- PURPOSE:  
c   
c  Time-invariant (8 variables) var1=stack input type  var2-var8=not used
c       var1=1:  T(K), U, V, W (m/s)
c       var1=2:  Fb, Fmx, Fmy, Fmz  (i.e., buoyancy and momentum fluxes)
c  Time-variant (if ivar1=1)
c       X(km), Y(km), Z(mAGL), Elev(mMSL), D(m), T(K), U, V, W, Sigy(m), Sigz(m), Q(g/s)...
c  Time-variant (if ivar1=2)
c       X(km), Y(km), Z(mAGL), Elev(mMSL), D(m), Fb, Fmx, Fmy, Fmz, Sigy(m), Sigz(m), Q(g/s)...
c            
c            
c
c --- UPDATES:
c --- INPUT:
c           iy1,ij1,ih1,is1,iy2,ij2,ih2,is2
c           x
c           y
c           z
c           elev
c           ds
c           ts
c           us
c           vs
c           ws
c           fb
c           fmx
c           fmy
c           fmz
c           sy
c           sz
c           q
c           nchem  (integer) :: number of chemical species emitted
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function iwrFLEMARBi(io,ivar,sname,
     &iy1,ij1,ih1,is1,iy2,ij2,ih2,is2,
     &x,y,z,elev,ds,ts,us,vs,ws,
     &fb,fmx,fmy,fmz,sy,sz,q,nchem)
c MODULES
      use mod_stddefs
      use mod_globals
c INPUTS
      integer :: io
      integer :: ivar
      character*32 :: sname
      real(8) :: x,y,z,elev,ds,ts,us,vs,ws
      real(8) :: fb,fmx,fmy,fmz,sy,sz
      real(8)  :: q(mxchem)
      integer  :: nchem
c DECLARATIONS
      iwrFLEMARBi = 0

c use 0-23 clock
      write(io,111) iy1,ij1,ih1,is1,iy2,ij2,ih2,is2
 111  format(2(3x,i04,1x,i03,1x,i02,1x,i04))
c   1990  009  04 0000    1990  009  05 0000
c'Flare1'  340.011  4870.443   92.513  160.4 40.33 1122.17 0.0   0.0   0.725 0.0   0.0  9746.0  0.0  0.0  0.0  105.81

c  Time-invariant (8 variables) var1=stack input type  var2-var8=not used
c       var1=1:  T(K), U, V, W (m/s)
c       var1=2:  Fb, Fmx, Fmy, Fmz  (i.e., buoyancy and momentum fluxes)
      if ( ivar .eq. 1) then
c  Time-variant (if ivar1=1)
c       X(km), Y(km), Z(mAGL), Elev(mMSL), D(m), T(K), U, V, W, Sigy(m), Sigz(m), Q(g/s)...
          n = LEN_REAL(sname)
          if( MUNITS.eq.1 ) then
              write(io,112) sname(1:n),x,y,
     &        z,elev,ds,ts,us,vs,ws,sy,sz,
     &        (q(i),i=1,nchem)
          else
              write(io,114) sname(1:n),x,y,
     &        z,elev,ds,ts,us,vs,ws,sy,sz,
     &        (q(i),i=1,nchem)
          end if
 112      format("'",A,"'",2(1x,F10.2),1x,F10.4,1x,F10.3,1x,F11.5,1x,
     &    F10.5,3(1x,F9.5),2(1x,F15.3),
     &    10(1x,G15.8))
 114      format("'",A,"'",2(1x,F10.5),1x,F10.4,1x,F10.3,1x,F11.5,1x,
     &    F10.5,3(1x,F9.5),2(1x,F15.3),
     &    10(1x,G15.8))

      else if (ivar .eq. 2) then
c  Time-variant (if ivar1=2)
c       X(km), Y(km), hs(mAGL), Elev(mMSL), D(m), Fb, Fmx, Fmy, Fmz, Sigy(m), Sigz(m), Q(g/s)...
          n = LEN_REAL(sname)
          if( MUNITS.eq.1 ) then
              write(io,113) sname(1:n),x,y,
     &        z,elev,ds,fb,fmx,fmy,fmz,sy,sz,
     &        (q(i),i=1,nchem)
          else
              write(io,115) sname(1:n),x,y,
     &        z,elev,ds,fb,fmx,fmy,fmz,sy,sz,
     &        (q(i),i=1,nchem)
          end if 
 113      format("'",A,"'",2(1x,F10.2),1x,F10.3,1x,F10.3,4(1x,F15.3),
     &    2(1x,F15.3),10(1x,G15.8))
 115      format("'",A,"'",2(1x,F10.5),1x,F10.3,1x,F10.3,4(1x,F15.3),
     &    2(1x,F15.3),10(1x,G15.8))

      else
c  write error 
          write(6,*) 
     &    ' *** BAD PARAMETER *** in iwrFLEMARBi, ivar must be 1 or 2'
          stop
      end if
      iwrFLEMARBi = 1
      return 
      end

      subroutine calcDELH(Ua, Ta, Kst, Ts, Vs, Ds, Hs, XH, DELH, zref)
c***********************************************************************
c                 DELH Module of ISC2 Model
c
c        PURPOSE: To Calculate Final Plume Rise
c
c        PROGRAMMER: Roger Brode, Jeff Wang
c                    updates, B.Zelt - www.zeltpsi.com
c
c Date:           March 2, 1992
c
c        MODIFIED:   To use calling argument
c                    R. W. Brode, PES, Inc. - 9/30/94
c
c                    B.Zelt
c                    Sept, 14, 2011 
c                    To calculate array of gradual rise heights 
c                    to final rise
c        OUTPUTS: Final Plume Rise, DHFOUT (m)
c***********************************************************************
c MODULES
      use mod_isccalcs
c INPUTS      
      real(8) :: Ua        ! amb wind speed (m/s)
      real(8) :: Ta        ! amb temperature (k)
      integer :: Kst    ! PG stability class
      real(8) :: Ts        ! source temperature (K)
      real(8) :: Vs        ! source exit velocity (m/s)
      real(8) :: Ds        ! source diameter (m)
      real(8) :: Hs        ! source stack height (m)
      real(8) :: XH(5)        ! distance to final rise (m)
      real(8) :: DELH(5)      ! plume rise (m)
      real(8) :: zref      ! reference windspeed height (m)
c --- DECLARATIONS
      real(8) :: TS1       ! source temperature
      real(8) :: Uref      ! amb wind speed (m/s) at reference height
      real(8) :: Us        ! amb wind speed (m/s) at stack tip
      real(8) :: DtDz      ! potental temperature gradient (K/m)
      real(8) :: PI
      real(8) :: ptg0(2)

c --- ISC defaults for array of potential temperatures (2)
      data ptg0/0.020,0.035/
      data PI/3.141593/

c     Variable Declarations
c      INCLUDE 'MAIN1.INC'
c      INCLUDE 'MAIN2.INC'
c      INCLUDE 'MAIN3.INC'


c ---       Use default potential temperature lapse rate with the
c ---       surface temperature from the nearest station
      if(KST.LE.4) then
          dtdz=0.0
      elseif(KST.EQ.5) then
          dtdz=ptg0(1)
      else
          dtdz=ptg0(2)
      endif

c      Zref = 10.  !m, reference windspeed height
      Uref = Ua   !m/s, wind speed
      G = 9.80665 !m/s2

      If (DtDz .gt. 0. .and. Ta .gt. 0.) Then
          S = G * DtDz / Ta
          RTOFS = sqrt(S)
      Else
          S = 0.0000000001
          RTOFS = 0.0000000001
      End If


c --- Set Stability Category Logical Flags and Adjust for KST > 6
      UNSTAB = .false.
      NEUTRL = .false.
      STABLE = .false.
      If (Kst .gt. 6) Then Kst = 6
      If (Kst .lt. 4) Then
         UNSTAB = .true. 
      ElseIf (Kst .eq. 4) Then
         NEUTRL = .true. 
      ElseIf (Kst .gt. 4) Then
         STABLE = .true. 
      End If

      Ts1 = Ts  
      If (Ts1 .lt. Ta) Then Ts1 = Ta
         FBISC = (0.25 / Ts1) * (Vs * Ds * Ds) * G * (Ts1 - Ta)
         FMISC = (0.25 / Ts1) * (Vs * Ds * Ds) * Vs * Ta

c Wind speed is entered at stack tip height, so 
c don't need to adjust
c              US = Uref
c --- Adjust Wind Speed -- Assume Wind Speed Constant Below 10 meters
      call wsstkht(uref, zref, kst, hs, us)      

c --- Do Not Allow Stack Height Wind Speed < 1.0 m/s
      If (US .lt. 1.) Then
         US = 1.
      End If

c --- Calculate Delta-T
      DELT = Ts1 - Ta

      If(UNSTAB .or. NEUTRL) Then
         If (FBISC .gt. 55.) Then
            DTCRIT = 0.00575 * Ts1 * ((Vs * Vs) / Ds)**0.333333
         Else
            DTCRIT = 0.0297 * Ts1 * (Vs / (Ds * Ds))**0.333333
         End If
         If (DELT .ge. DTCRIT) Then
            If (FBISC .ge. 55.) Then
               DHFOUT = 38.71 * (FBISC**0.6) / US
            Else
               DHFOUT = 21.425 * (FBISC**0.75) / US
            End If
         Else
            DHFOUT = 3. * Ds * Vs / US
         End If

      ElseIf (STABLE) Then
        DTCRIT = 0.019582 * Vs * Ta * RTOFS
        If (DELT .ge. DTCRIT) Then
            DHFOUT = 2.6 * (FBISC / (US * S))**0.333333
c            Compare to Final Plume Rise for Calm Winds, DHCLM
          DHCLM = 4. * (FBISC**0.25) / (S**0.375)
          If (DHCLM .lt. DHFOUT) Then DHFOUT = DHCLM
          Else
            DHFOUT = 1.5 * (FMISC / (US * RTOFS))**0.333333
c           Compare to Maximum Momentum Rise for UNSTABLE/NEUTRAL, DHCHK
              DHCHK = 3. * Ds * Vs / US
          If (DHCHK .lt. DHFOUT) Then DHFOUT = DHCHK
          End If
      End If

C***********************************************************************
C        DISTF Module of ISC2 Short Term Model - ISCST2
C        PURPOSE: Calculates Distance to Final Plume Rise
C        PROGRAMMER: Roger Brode, Jeff Wang
C        DATE:    March 2, 1992
C        OUTPUTS: Distance to Final Plume Rise, XF (m), and Distance
C                 to Final Buoyant Rise (XFB) and Final Momentum Rise (XFM)
C***********************************************************************
      IF (UNSTAB .OR. NEUTRL) THEN
          IF (FBISC .GE. 55.) THEN
              XFB = 119. * FBISC**0.4
          ELSE IF (FBISC .GT. 0.) THEN
              XFB = 49. * FBISC**0.625
          ELSE
              XFB = 4.*DS*(VS+3.*US)*(VS+3.*US)/(VS*US)
          END IF
          XFM = 4.*DS*(VS+3.*US)*(VS+3.*US)/(VS*US)
          XF = XFB
          if(XFM.gt.XFB) then
              XF = XFM
          end if 
      ELSE IF (STABLE) THEN
          XFB = 2.0715*US/RTOFS
          XFM = 0.5*PI*US/RTOFS
          XF = XFB
          if(XFM.gt.XFB) then
              XF = XFM
          end if 
      END IF

      XH(3)   = XF*.333
      XH(4)   = XF*.667
      XH(5)   = XF

      XARG = XH(3)
      call calcDHPHS(XARG,DHFOUT,US,VS,DELH(3))
      XARG = XH(4)
      call calcDHPHS(XARG,DHFOUT,US,VS,DELH(4))

c add offset of flare flame tip location      
      XH(3) = XH(3) + XH(2)
      XH(4) = XH(4) + XH(2)
      XH(5) = XH(5) + XH(2)
c add offset of flare flame tip height
      DELH(3) = HS + DELH(3)
      DELH(4) = HS + DELH(4)
      DELH(5) = HS + DHFOUT

      return 
      end 


                  SUBROUTINE calcDHPHS(XARG,DHFINAL,US,VS,DHPOUT)
C***********************************************************************
C                 DHPHS Module of ISC2 Model
C
C        PURPOSE: Calculates Distance-dependent Plume Rise for
C                 Huber-Snyder Downwash Algorithm and for BID
C
C        PROGRAMMER: Roger Brode, Jeff Wang
C
C        DATE:    March 2, 1992
C
C        MODIFIED:   To use calling arguments
C                    R. W. Brode, PES, Inc. - 9/30/94
C
C        INPUTS:  Arrays of Source Parameters
C                 Buoyancy and Momentum Fluxes
C                 Meteorological Variables for One Hour
C                 Wind Speed Adjusted to Stack Height
C                 Downwind Distance
C
C        OUTPUTS: Distance-dependent Plume Rise, DHPOUT (m)
C
C        CALLED FROM:   PHEFF
C                       BID
C***********************************************************************
c MODULES                  
      use mod_isccalcs
c --- INPUTS
      real(8) :: XARG        ! distance (m)
      real(8) :: DHFINAL     ! final rise height (m)
      real(8) :: US          ! windspeed at stack tip height (m/s)
      real(8) :: VS          ! source vertical speed (m/s)
      real(8) :: DHPOUT      ! gradual rise at distacnce XARG
C     Variable Declarations
c      USE MAIN1
c      IMPLICIT NONE
c      CHARACTER MODNAM*12
c      SAVE

c --- Declarations
      real(8) :: XP

C     Variable Initializations
c      MODNAM = 'DHPHS'

C     Calculate Distance-dependent Buoyant Plume Rise (Eqn. 1-22)
      XP = MIN(XARG,XFB)
      IF (XP .LT. 1.0)  XP = 1.0
      IF (FBISC .LT. 1.0E-10)  FBISC = 1.0E-10
      DHPB = 1.60 * (FBISC*XP*XP)**0.333333 / US

C     Calculate Dist-dependent Momentum Plume Rise          ---   CALL DHPMOM
      CALL calcDHPMOM(XARG,US,VS,DHPM)

C     Select Maximum of Buoyant or Momentum Rise for Gradual Rise
      DHPOUT = MAX(DHPB, DHPM)
C     Compare to Final Rise and Select Smaller Value for Gradual Rise
      DHPOUT = MIN(DHPOUT, DHFINAL)

      RETURN
      END

      SUBROUTINE calcDHPMOM(XARG,US,VS,DHPM)
C***********************************************************************
C                 DHPMOM Module of ISC2 Model
C
C        PURPOSE: Calculates Distance-dependent Momentum Plume Rise
C
C        PROGRAMMER: Roger Brode, Jeff Wang
C
C        DATE:    March 2, 1992
C
C        INPUTS:  Arrays of Source Parameters
C                 Buoyancy and Momentum Fluxes
C                 Meteorological Variables for One Hour
C                 Wind Speed Adjusted to Stack Height
C                 Downwind Distance
C
C        OUTPUTS: Distance-dependent Momentum Plume Rise, DHPM (m)
C
C        CALLED FROM:   WAKFLG
C                       DHPHS
C***********************************************************************
c MODULES      
      use mod_isccalcs
C     Variable Declarations
c      USE MAIN1
c      IMPLICIT NONE
c      CHARACTER MODNAM*12
c      SAVE
      real(8) :: XARG, XP, BETAJ
C     Variable Initializations
c      MODNAM = 'DHPMOM'

C     Calculate BETAJ Parameter (Entrainment Coefficient)
      BETAJ = 0.333333 + US/VS

      IF (UNSTAB .OR. NEUTRL) THEN
          XP = MIN(XARG,XFM)
          DHPM = (3.*FMISC*XP/(BETAJ*BETAJ*US*US))**0.333333
      ELSE IF (STABLE) THEN
          XP = MIN(XARG,XFM)
          DHPM = 3.*FMISC*SIN(RTOFS*XP/US) / (BETAJ*BETAJ*US*RTOFS)
C        Set Lower Limit for DHPM to Avoid Negative Arg for Cube Root
          DHPM = MAX(1.0E-10, DHPM)
          DHPM = DHPM ** 0.333333
      END IF

C     Do Not Let Gradual Rise Exceed Final Momentum Rise
      DHPM = MIN(DHPM, 3.*DS*VS/US)

      RETURN
      END


c ref: The International Association for the Properties of Water and Steam, 2007	
c Revised Release on the IAPWS Industrial Formulation 1997 for the Thermodynamic Properties of Water and Steam	
c Dimensionless steam saturation equation coefficients	
c i	ni
c 1 :	1167.052145
c 2 :	-724213.167
c 3 :	-17.07384694
c 4 :	12020.8247
c 5 :	-3232555.032
c 6 :	14.91510861
c 7 :	-4823.265736
c 8 :	405113.4054
c 9 :	-0.238555576
c 10:	650.1753484


c----------------------------------------------------------------------
c  p_sat()
c----------------------------------------------------------------------
c     
c     Calculates saturation pressure for steam
c
c
c B.Zelt - www.zeltpsi.com
c ref: The International Association for the Properties of Water and Steam, 2007
c Revised Release on the IAPWS Industrial Formulation 1997 for the Thermodynamic Properties of Water and Steam
c Dimensionless saturation equations
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- UPDATES
c --- INPUT
c     T   : real :    (K) temperature
c
c --- OUTPUT
c     P   : real (abs pressure in MPa, converted to kPa abs)
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function p_sat(T)
c --- inputs      
      real(8) :: T
      real(8) :: p_sat      
c --- declarations      
c      real(8) :: sum
      real(8) :: gibbs(10)
      real(8) :: A
      real(8) :: b
      real(8) :: C
      real(8) :: V

      data gibbs/1167.052145,-724213.167,-17.07384694,12020.8247,
     &           -3232555.032,14.91510861,-4823.265736,405113.4054,
     &           -0.238555576,650.1753484/
      
      V = T + gibbs(9) / (T - gibbs(10))
   
      A = V * V + gibbs(1) * V + gibbs(2)
      b = gibbs(3) * V * V + gibbs(4) * V + gibbs(5)
      C = gibbs(6) * V * V + gibbs(7) * V + gibbs(8)
c Psat in MPa
      p_sat = 2. * C / (-b + sqrt(b * b - 4. * A * C))
      p_sat = p_sat * p_sat
      p_sat = p_sat * p_sat
      p_sat = p_sat * 1000. ! return kPa
      return
      end 

c----------------------------------------------------------------------
c t_sat()
c----------------------------------------------------------------------
c     
c     Calculates the saturated temperature for steam
c
c B.Zelt - www.zeltpsi.com
c ref: The International Association for the Properties of Water and Steam, 2007
c Revised Release on the IAPWS Industrial Formulation 1997 for the Thermodynamic Properties of Water and Steam
c Dimensionless saturation equations
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES
c --- INPUT
c     p   : real :  pressure kPa
c
c
c --- OUTPUT
c     t   : saturation temerpature, K
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function t_sat(p)
c INPUTS
      real(8) :: p       ! pressure kPa
      real(8) :: t_sat      
c DECLARATIONS
c      real(8) :: sum
c      real(8) :: ni
      real(8) :: gibbs(10)
      real(8) :: D 
      real(8) :: E 
      real(8) :: F 
      real(8) :: G 
      real(8) :: b 
   
      data gibbs/1167.052145,-724213.167,-17.07384694,12020.8247,-
     &           3232555.032,14.91510861,-4823.265736,405113.4054,
     &           -0.238555576,650.1753484/

c p in MPA
      b = (p * 0.001)**0.25
   
      E = b * b + gibbs(3) * b + gibbs(6)
      F = gibbs(1) * b * b + gibbs(4) * b + gibbs(7)
      G = gibbs(2) * b * b + gibbs(5) * b + gibbs(8)
   
      D = 2. * G / (-F - sqrt(F*F - 4. * E * G))
c t_sat in K
      t_sat = gibbs(10) + D - sqrt((gibbs(10) + D)**2. - 
     &        4.*(gibbs(9) + gibbs(10) * D))
      t_sat = t_sat * 0.5
      return
      end 


c----------------------------------------------------------------------
c fnZ()
c----------------------------------------------------------------------
c  
c     Calculates the compressibility factor from Tc,Pc
c        
c B.Zelt - www.zeltpsi.com
c http://www.chem.mtu.edu/~tbco/cm3450/Compressibility_from_Redlich_Kwong.pdf
c Redlich-Kwong equation
c based on Cutlip and Shacham, 2008, pp. 101-103 (see Seader, Henley & Roper, 3rd Ed, 2011)
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c
c --- UPDATES
c --- INPUTS
c     T   : real : temperature (K)
c     P   : real : pressure (kPa)
c     Tc  : real : critical temperature (K)
c     Pc  : real : critical pressure (kPa)
c
c --- OUTPUTS
c
c --- RETURNS
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      Function fnZ(T, P, Tc, Pc)
c INPUTS      
      real(8) :: T
      real(8) :: P
      real(8) :: Tc
      real(8) :: Pc
      real(8) ::fnZ      
c DECLARATIONS      
      real(8) :: Tr
      real(8) :: Pr
      real(8) :: A
      real(8) :: b
      real(8) :: q
      real(8) :: r
      real(8) :: rone
   
      Tr = T / Tc
      Pr = p / Pc
   
      A = 0.42747 * Pr / (Tr**(5. / 2.))
      b = 0.08664 * Pr / Tr
      r = A * b
      q = b * b + b - A
      rone = 1.
   
      fnZ = rmcroot(rone, -rone, -q, -r)
      return
      end 


c----------------------------------------------------------------------
c rmcroot()
c----------------------------------------------------------------------
c     
c http://www.chem.mtu.edu/~tbco/cm3450/Compressibility_from_Redlich_Kwong.pdf
c Redlich-Kwong equation
c based on Cutlip and Shacham, 2008, pp. 101-103 (see Seader, Henley & Roper, 3rd Ed, 2011)
c
c Computes the maximum real root of the cubic equation
c a3 x^3 + a2 x^2 + a1 x + a0 = 0
c
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- UPDATES
c --- INPUT
c     a3   : real :
c     a2   : real :
c     a1   : real :
c     a0   : real :
c
c --- OUTPUT
c
c --- RETURNS
c     rmcroot :   real : maximum real root of the cubic equation 
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function rmcroot(a3, a2 , a1, a0)
     & result(z)       
c INPUTS
      real(8) :: a3
      real(8) :: a2
      real(8) :: a1
      real(8) :: a0
      real(8) :: z
c      real(8) :: rmcroot      
c DECLARATIONS      
      real(8) :: A
      real(8) :: b
      real(8) :: C
      real(8) :: Disc
      real(8) :: rk13
      
      rk13 = 1./3.

      A = a2 / a3
      b = a1 / a3
      C = a0 / a3
      p = (-A*A*rk13 + b)*rk13
      q = (9.*A*b - 2.*A*A*A - 27.*C) / 54.
      Disc = q*q + p*p*p
      if( Disc .gt. 0.) Then
          h = q + Disc**0.5
          y = abs(h)**rk13
          if( h .lt. 0.) y = -y
          z = y - p / y - A*rk13
      Else
          theta = atan((-Disc)**0.5 / q)
          c1 = cos(theta*rk13)
          if( q .lt. 0. ) Then
              s1 = Sin(theta*rk13)
              c1 = (c1 - s1 * 3**0.5)*0.5
          End If
          z1 = 2. * (-p)**0.5 * c1 - A*rk13
          m = A + z1
          r = (m*m - 4. * (b + m * z1))**0.5
          z2 = (-m + r)*0.5
          z3 = (-m - r)*0.5
          z = z1
          if(z2 .gt. z) z = z2
          if(z3 .gt. z) z = z3
      end if
c      mcroot = z
      return
      end 

c    
c rtsafe(...)
c uses the newton-raphson method with bisection to safely and quickly find the root.
c
c funcd() is a routine that returns both the function value and the first derivative
c sub funcd(x1,fv,df)
c
c
      FUNCTION RTSAFE(FUNCD,X1,X2,XACC)
      real(8) :: x1
      real(8) :: x2
      real(8) :: xacc
      
c      USE,INTRINSIC :: IEEE_ARITHMETIC
        interface
          subroutine FUNCD(X1,FV,DF)
            real(8),intent(in) :: X1
            real(8),intent(out) :: FV
            real(8),intent(out) :: DF
           end subroutine FUNCD
        end interface   
c DECLARATIONS
      real(8) :: xl
      real(8) :: xh
      real(8) :: dxold
      real(8) ::dx
      real(8) :: swap
      real(8) :: fl, f
      real(8) :: fh, df
      real(8) :: temp
      integer :: j
      
      PARAMETER (MAXIT=100)
      
      CALL FUNCD(X1,FL,DF)
      CALL FUNCD(X2,FH,DF)
      
      IF(FL*FH.GE.0.) then
         rtsafe = -1. ! IEEE_VALUE(IEEE_QUIET_NAN)
         return
c         PAUSE 'root must be bracketed'
      end if    
      IF(FL.LT.0.)THEN
        XL=X1
        XH=X2
      ELSE
        XH=X1
        XL=X2
        SWAP=FL
        FL=FH
        FH=SWAP
      ENDIF
      RTSAFE=.5*(X1+X2)
      DXOLD=ABS(X2-X1)
      DX=DXOLD
      CALL FUNCD(RTSAFE,F,DF)
      DO 11 J=1,MAXIT
        IF((((RTSAFE-XH)*DF-F)*((RTSAFE-XL)*DF-F).GE.0.)
     *      .OR. (ABS(2.*F).GT.ABS(DXOLD*DF)) ) THEN
          DXOLD=DX
          DX=0.5*(XH-XL)
          RTSAFE=XL+DX
          IF(XL.EQ.RTSAFE)RETURN
        ELSE
          DXOLD=DX
          DX=F/DF
          TEMP=RTSAFE
          RTSAFE=RTSAFE-DX
          IF(TEMP.EQ.RTSAFE)RETURN
        ENDIF
        IF(ABS(DX).LT.XACC) RETURN
        CALL FUNCD(RTSAFE,F,DF)
        IF(F.LT.0.) THEN
          XL=RTSAFE
          FL=F
        ELSE
          XH=RTSAFE
          FH=F
        ENDIF
11    CONTINUE
c      PAUSE 'RTSAFE exceeding maximum iterations'
      rtsafe = -1 ! IEEE_VALUE(IEEE_QUIET_NAN)
      RETURN
      END

      
      
c----------------------------------------------------------------------
c EntrainmentRatio()
c
c Calculates the volumetric entraintment ratio
c
c Inputs:
c     R : Air to X entrainment constant (wt/wt)
c     DenRatio : Air to X density ratio 
c     T : temperature of X, K
c     P : pressure of X, kPa
c
c----------------------------------------------------------------------
      function EntrainmentRatio(R, DenRatio, T, P)
      use mod_globals
      
      real(8) :: r
      real(8) :: DenRatio
      real(8) :: T
      real(8) :: P
      real(8) :: EntrainmentRatio
      
      EntrainmentRatio=2.D0*R/(sqrt(DenRatio)*(DenRatio+1.D0))*(T/Tref)*(Pref/P)
     
      return 
      end 
      

      
c----------------------------------------------------------------------
c     
c----------------------------------------------------------------------
      integer*4 function get_free_io (lu_max)
!
!   get_file_unit returns a unit number that is not in use
      integer*4 lu_max,  lu, m, iostat
      logical   opened
!
      m = lu_max  ;  if (m < 1) m = 97
      do lu = m,1,-1
         inquire (unit=lu, opened=opened, iostat=iostat)
         if (iostat.ne.0) cycle
         if (.not.opened) exit
      end do
!
      get_free_io= lu
      return
      end function get_free_io      